var minTimestamp = 1535760000000;
var maxTimestamp = 1704067199000;

// A standard set of colours.
var colors =
  [
    '#008FFB',
    '#4CAF50',
    '#FEB019',
    '#FF4560',
    '#775DD0',
    '#7CDF80',
    '#546E7A',
    '#8D5B4C',
    '#C5D86D',
    '#EE00FB',
    '#CCCCCC'
  ];
  
// Disk speed colors
var diskColors =
  [
    '#C4BBAF',
    '#A5978B',
    '#00E396',
    '#4CAF50',
    '#F9CE1D',
    '#FF9800',
  ];

// Filesystem speed colors
var filesystemColors =
  [
    '#A5978B',
    '#4CAF50',
    '#FF9800',
  ];

var grey = '#CCCCCC';

var machineTypes =
  {
  "iMac" : "imac",
  "iMac Pro" : "imac",
  "Mac mini": "mini",
  "Mac Pro": "pro",
  "MacBook": "mb",
  "MacBook Pro": "mbp",
  "MacBook Air": "air",
  "Mac Studio": "ms"
  };
  
var cpuCodes =
  {
  "Intel Core 2": 0,
  "Intel Core i3": 1,
  "Intel Core M": 2,
  "Intel Core i5": 3,
  "Intel Core i7": 4,
  "Intel Core i9": 5,
  "Intel Xeon": 6,
  "Apple M1": 7,
  "Apple M2": 8,
  "Apple M3": 9,
  "Apple M4": 10
  };
  
var currentSection = null;

var next =
  {
  "intro": function() { drawIntro() },
  "problem": function() { drawProblems() },
  "problemdescription": function() { drawProblemDescriptions() },
  "performance": function() { drawPerformance() },
  "systemspeed": function() { drawMachinePerformance() },
  "machine": function() { drawMachines() },
  "models": function() { drawModels() },
  "os": function() { drawOS() },
  "cpu": function() { drawCPU() },
  "oschanges": function() { drawOSChanges() },
  "av": function() { drawAV() },
  "avvendors": function() { drawAntivirusVendors() },
  "topav_filesystemtime": function() { drawTopAVFileSystemTime() },
  "topav_score": function() { drawTopAVFileScore() },
  "topav_hasadware": function() { drawTopAVFileHasAdware() },
  "tm": function() { drawTimeMachine() },
  "diskspeed": function() { drawDiskSpeed() },
  "filesystem": function() { drawFilesystemSpeed() },
  "hdfailure": function() { drawHdFailure() },
  "topmods": function() { drawTopMods() },
  "systemmods": function() { drawSystemMods() },
  "more": function() { drawMore() },
  "close": function() { closeWindow() }
  };
  
// Draw the insights.
function draw(section)
  {
  var fn = next[section];
  
  fn();
  }
  
// Draw the intro.
function drawIntro()
  {
  updateCurrentSection('intro');
  
  // Create the next button.
  createNavigationButtons('intro');
  
  drawHTML('intro');
  
  if(insights["os"] == undefined)
    {
    var commentarySection = document.getElementById("commentary")

    var freeDiv = document.createElement("div");
    
    freeDiv.innerHTML = commentary["intro_free"];
    
    commentarySection.appendChild(freeDiv);
    }
  }
  
// Draw more insights.
function drawMore()
  {
  updateCurrentSection('more');

  // Create the next button.
  createNavigationButtons('more');
  
  createNavigationButton('next', localization["close"], next["close"]);
  
  drawHTML('more');

  if(insights["os"] == undefined)
    {
    var commentarySection = document.getElementById("commentary")

    var freeDiv = document.createElement("div");
    
    freeDiv.innerHTML = commentary["more_free"];
    
    commentarySection.appendChild(freeDiv);
    }
    
  var p = document.createElement("p");
  
  p.innerHTML = powerUserPrice;
  
  p.setAttribute("class", "buynow");
  
  var button = document.createElement("button");
  
  button.setAttribute("id", "poweruserbutton");
  button.setAttribute("onclick", "buyPowerUser()");
  button.setAttribute("title", powerUserTitle);
  
  button.innerHTML = powerUserTitle;
  
  p.appendChild(button);
  
  commentarySection.appendChild(p);
  }
  
// Draw EtreCheck problems.
function drawProblems()
  {
  var options =
    {
    chart: 
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'pie',
      width: '80%',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    plotOptions:
      {
      pie:
        {
        expandOnClick: true
        }
      },
    labels: getLabels(text['problem']['labels'], 'problem'),
    series: getSeries('problem'),
    colors: getColors('problem'),
    tooltip:
      {
      y: 
        {
        formatter: (value) => { return value + '%' }
        }
      },
    dataLabels:
      {
      dropShadow:
        {
        enabled: false
        }
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      }
    };

  // Create the next button.
  createNavigationButtons('problem');
  
  drawChart('problem', options);
  }
  
// Draw EtreCheck problem descriptions.
function drawProblemDescriptions()
  {
  var options =
    {
    chart: 
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'pie',
      width: '70%',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    
    labels:
      getLabels(text['problemdescription']['labels'], 'problemdescription'),
    
    series: getSeries('problemdescription'),
    colors: getColors('problemdescription'),
    tooltip:
      {
      y: 
        {
        formatter: (value) => { return value + '%' }
        }
      },
    dataLabels:
      {
      dropShadow:
        {
        enabled: false
        }
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      }
    };

  // Create the next button.
  createNavigationButtons('problemdescription');
    
  drawChart('problemdescription', options);
  }

// Draw chart of EtreCheck performance results.
function drawPerformance()
  {
  var options =
    {
    chart: 
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'pie',
      width: '80%',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    labels: getLabels(text['performancelabels'], 'performance'),
    series: getSeries('performance'),
    colors: getColors('performance'),
    tooltip:
      {
      y: 
        {
        formatter: (value) => { return value + '%' }
        }
      },
    dataLabels:
      {
      dropShadow:
        {
        enabled: false
        }
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      }
    };

  // Create the next button.
  createNavigationButtons('performance');
    
  drawChart('performance', options);
  }

// Draw chart of EtreCheck runtimes per machine.
function drawMachinePerformance()
  {
  var seconds = machineData["seconds"];
  
  var max = undefined;
  var annotations = undefined;
  
  if(seconds != undefined)
    {
    var time = seconds / 60.0;
    
    if(time > 4)
      max = time + .5;
      
    annotations =
      {
      yaxis:
        [{
        strokeDashArray: 0,
        y: time,
        borderColor: "#E30000",
        label:
          {
          borderColor: "#E30000",
          style:
            {
            color: "#ffffff",
            background: "#E30000"
            },
          text: text["yourmachine"]
          }
        }]
      };
    }
    
  var options =
    {
    chart: 
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'bar',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    labels: getRawLabels('systemspeed'),
    series: getMachinePerformanceSeries('systemspeed'),
    colors: getColors('systemspeed'),
    dataLabels:
      {
      formatter: (value) => { return value.toFixed(2) + ' min' }
      },
    tooltip:
      {
      y: 
        {
        formatter: (value) => { return value.toFixed(2) + ' min' }
        }
      },
    yaxis:
      {
      max: max,
      labels:
        {
        formatter: (value) => { return value.toFixed(0) + ' min' },
        style:
          {
          colors: textColor()
          }
        }
      },
    xaxis:
      {
      labels:
        {
        style:
          {
          colors: textColor()
          },
        rotate: -45,
        rotateAlways: true
        }
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      },
    grid:
      {
      borderColor: gridColor()
      },
    annotations: annotations
    };

  // Create the next button.
  createNavigationButtons('systemspeed');
    
  drawChart('systemspeed', options);
  }

function getMachinePerformanceSeries(datatype)
  {
  var data =
    {
    name: text['oschanges']['xaxis'],
    data: []
    };
    
  for(var key in insights[datatype].data)
    data.data.push(parseFloat(insights[datatype].data[key]));
    
  var series = new Array;
  
  series.push(data);
    
  return series;
  }

// Draw a chart of the machine population.
function drawMachines()
  {
  var options =
    {
    chart: 
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'pie',
      width: '80%',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    labels: getRawLabels('machine'),
    series: getSeries('machine'),
    colors: getColors('machine'),
    tooltip:
      {
      y: 
        {
        formatter: (value) => { return value + '%' }
        }
      },
    dataLabels:
      {
      dropShadow:
        {
        enabled: false
        }
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      }
    };

  // Create the next button.
  createNavigationButtons('machine');
    
  drawChart('machine', options);
  }
  
// Draw a chart of models.
function drawModels()
  {
  var machineName = machineData["name"];
  var machineType = machineTypes[machineName];
  
  var options =
    {
    chart:
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'pie',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    labels: getRawLabels(machineType),
    series: getSeries(machineType),
    colors: getColors(machineType),
    tooltip:
      {
      y:
        {
        formatter: (value) => { return value + '%' }
        }
      },
    dataLabels:
      {
      dropShadow:
        {
        enabled: false
        }
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      }
    };

  // Create the next button.
  createNavigationButtons('models');
    
  drawChart('models', options);
  }

// Draw chart of operating system adoption over time.
function drawOS()
  {
  // Use numeric labels from the data. 
  
  // Setup a stacked area chart.
  var options = 
    {
    chart: 
      {
      type: 'area',
      stacked: true,
      animations: 
        {
        enabled: enableAnimations,
        speed: 800,
        animateGradually: 
          {
          enabled: false,
          delay: 150
          },
        dynamicAnimation: 
          {
          enabled: enableAnimations,
          speed: 350
          }
        },
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    labels: getRawLabels('os'),
    series: getOSSeries('os'),
    colors: getColors('os'),
    dataLabels: 
      {
      enabled: false
      },
    stroke: 
      {
      curve: 'smooth',
      width: 2
      },
    fill: 
      {
      type: 'gradient',
      gradient: 
        {
        opacityFrom: 0.6,
        opacityTo: 0.8,
        }
      },
    xaxis: 
      {
      type: 'datetime',
      labels:
        {
        format: 'MMM yyyy',
        style:
          {
          colors: textColor()
          }
        }
      },
    yaxis:
      {
      min: 0,
      max: 100,
      show: false
      },
    tooltip:
      {
      y: 
        {
        formatter: (value) => { return value.toFixed(1) + '%' },
        },
      x: 
        {
        format: 'MMM yyyy',
        },
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      },
    grid:
      {
      borderColor: gridColor()
      }
    };

  // Create the next button.
  createNavigationButtons('os');
      
  drawChart('os', options);
  }

// The operating system series needs funky sorting of version numbers.
function getOSSeries(datatype)
  {
  var versions = {};
  
  // Get a sorted array of timestamps.
  var timestamps = Object.keys(insights[datatype].data);
  
  var sortedTimestamps =
    timestamps.sort(
      function (a, b) 
        {
        return parseInt(a) - parseInt(b);
        });
        
  // Go through the timestamps.
  for(var i = 0; i < sortedTimestamps.length; ++i)
    {
    var timestamp = sortedTimestamps[i];
    
    var data = insights[datatype].data[timestamp];
    
    // Go through each OS version in each sample time.
    for(var osversion in data)
      {
      var version = versions[osversion];
      
      // Create a new version, if necessary.
      if(version == undefined)
        {
        version =
          {
          name: osversion,
          data: []
          };
        
        versions[osversion] = version;
        }
        
      // Add this timestamp value for this version.
      var time = parseInt(timestamp);
      
      if((time > minTimestamp) && (time < maxTimestamp))
        version.data.push([time, parseFloat(data[osversion])]);
      }
    }
    
  // Now sort the version numbers.
  var keys = Object.keys(versions);
  
  var sortedKeys =
    keys.sort(
      function (a, b)
        {
        if(a == "10.9")
          a = "10.09";
          
        if(b == "10.9")
          b = "10.09";
        
        return Number(a) - Number(b);
        });

  // Build the series.
  var series = new Array;
  
  for(var i = 0; i < sortedKeys.length; ++i)
    {
    var key = sortedKeys[i];
    series.push(versions[key]);
    }
    
  return series;
  }

// Draw chart of CPU usage over time.
function drawCPU()
  {
  // Use numeric labels from the data.
  
  // Setup a stacked area chart.
  var options =
    {
    chart:
      {
      type: 'area',
      stacked: true,
      animations:
        {
        enabled: enableAnimations,
        speed: 800,
        animateGradually:
          {
          enabled: false,
          delay: 150
          },
        dynamicAnimation:
          {
          enabled: enableAnimations,
          speed: 350
          }
        },
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    labels: getCPULabels('cpu'),
    series: getCPUSeries('cpu'),
    colors: getColors('cpu'),
    dataLabels:
      {
      enabled: false
      },
    stroke:
      {
      curve: 'smooth',
      width: 2
      },
    fill:
      {
      type: 'gradient',
      gradient:
        {
        opacityFrom: 0.6,
        opacityTo: 0.8,
        }
      },
    xaxis:
      {
      type: 'datetime',
      labels:
        {
        format: 'MMM yyyy',
        style:
          {
          colors: textColor()
          }
        }
      },
    yaxis:
      {
      min: 0,
      max: 100,
      show: false
      },
    tooltip:
      {
      y:
        {
        formatter: (value) => { return value.toFixed(1) + '%' },
        },
      x:
        {
        format: 'MMM yyyy',
        },
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      },
    grid:
      {
      borderColor: gridColor()
      }
    };

  // Create the next button.
  createNavigationButtons('cpu');
      
  drawChart('cpu', options);
  }

function getCPULabels(datatype)
  {
  var labels = new Array;
  
  for(var key in insights[datatype].data)
    if(key != "Other")
      labels.push(key);
    
  var sortedLabels =
    labels.sort(
      function (a, b)
        {
        return cpuCodes[a] - cpuCodes[b];
        });
    
  return sortedLabels;
  }

// Be careful about how these are displayed.
function getCPUSeries(datatype)
  {
  var cpus = {};
  
  // Get a sorted array of timestamps.
  var timestamps = Object.keys(insights[datatype].data);
  
  var sortedTimestamps =
    timestamps.sort(
      function (a, b)
        {
        return parseInt(a) - parseInt(b);
        });
        
  // Go through the timestamps.
  for(var i = 0; i < sortedTimestamps.length; ++i)
    {
    var timestamp = sortedTimestamps[i];
    
    var data = insights[datatype].data[timestamp];
    
    // Go through each CPU in each sample time.
    for(var cpucode in data)
      {
      if(cpucode == "Other")
        continue;
        
      var cpu = cpus[cpucode];
      
      // Create a new version, if necessary.
      if(cpu == undefined)
        {
        cpu =
          {
          name: cpucode,
          data: []
          };
        
        cpus[cpucode] = cpu;
        }
        
      // Add this timestamp value for this version.
      var time = parseInt(timestamp);
      
      if((time > minTimestamp) && (time < maxTimestamp))
        cpu.data.push([time, parseFloat(data[cpucode])]);
      }
    }
    
  // Now sort the cpu codes.
  var keys = Object.keys(cpus);
  
  var sortedKeys =
    keys.sort(
      function (a, b)
        {
        return cpuCodes[a] - cpuCodes[b];
        });

  // Build the series.
  var series = new Array;
  
  for(var i = 0; i < sortedKeys.length; ++i)
    {
    var key = sortedKeys[i];
    series.push(cpus[key]);
    }
    
  return series;
  }

// Draw chart of operating system changes over time.
function drawOSChanges()
  {
  var keys =
    [
    "systemtasks",
    "swapused",
    "freeram",
    "freestorage",
    "score"
    ];

  var OSChangeColors =
    {
    "systemtasks" : "#A5978B",
    "swapused" : "#4CAF50",
    "freeram" : "#00E396",
    "freestorage" : "#008FFB",
    "score" : "#FF4560"
    };
    
  var options = 
    {
    chart: 
      {
      type: 'line',
      animations:
        {
        enabled: enableAnimations,
        speed: 800,
        animateGradually: 
          {
          enabled: false,
          delay: 150
          },
        dynamicAnimation: 
          {
          enabled: enableAnimations,
          speed: 350
          }
        },
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    labels: getOrderedLabels('oschanges', keys),
    
    series:
      getOSChangesSeries(text['oschanges']['labels'], 'oschanges', keys),
    
    colors: getOrderedColors(OSChangeColors, keys),
    dataLabels: 
      {
      enabled: false
      },
    stroke: 
      {
      curve: 'smooth',
      width: 2
      },
    xaxis: 
      {
      type: 'category',
      categories: getOSChangesCategories('oschanges'),
      labels:
        {
        style:
          {
          colors: textColor()
          }
        }
      },
    yaxis:
      [
        {
        min: 0,
        max: 1000,
        axisTicks: 
          {
          show: false,
          },
        axisBorder: 
          {
          show: true,
          color: OSChangeColors['systemtasks']
          },
        labels: 
          {
          style: 
            {
            colors: OSChangeColors['systemtasks']
            },
          formatter: (value) => { return value.toFixed(0) }
          },
        title: 
          {
          text: text['oschanges']['yaxis']['systemtasks'],
          style: 
            {
            color: OSChangeColors['systemtasks']
            }
          }
        },
        {
        show: false,
        min: 0,
        max: 3,
        opposite: true,
        axisTicks: 
          {
          show: true,
          },
        axisBorder: 
          {
          show: true,
          color: OSChangeColors['swapused']
          },
        labels: 
          {
          style: 
            {
            colors: OSChangeColors['swapused']
            },
          formatter: 
            (value) => 
              { 
              var mb = value * 1024;
              
              return mb.toFixed(0) + ' ' + localization['MB'];
              }
          },
        title: 
          {
          style:
            {
            color: OSChangeColors['swapused']
            }
          }
        },
        {
        min: 0,
        max: 3,
        opposite: true,
        axisTicks: 
          {
          show: true,
          },
        axisBorder: 
          {
          show: true,
          color: OSChangeColors['swapused']
          },
        labels: 
          {
          style: 
            {
            colors: OSChangeColors['swapused']
            },
          formatter: 
            (value) => 
              { 
              var mb = value * 1024;
              
              return mb.toFixed(0) + ' ' + localization['MB'];
              }
          },
        title: 
          {
          text: text['oschanges']['yaxis']['swapused'],
          style: 
            {
            color: OSChangeColors['swapused']
            }
          }
        },
        {
        opposite: true,
        min: 0,
        max: 100,
        axisTicks: 
          {
          show: true,
          },
        axisBorder: 
          {
          show: true,
          color: OSChangeColors['freestorage']
          },
        labels: 
          {
          style: 
            {
            colors: OSChangeColors['freestorage']
            },
          formatter: (value) => { return value.toFixed(0) + '%' }
          },
        title: 
          {
          text: text['oschanges']['yaxis']['freestorage'],
          style: 
            {
            color: OSChangeColors['freestorage']
            }
          }
        },
        {
        axisTicks: 
          {
          show: true,
          },
        axisBorder: 
          {
          show: true,
          color: OSChangeColors['score']
          },
        labels: 
          {
          style: 
            {
            colors: OSChangeColors['score']
            },
          formatter: (value) => { return text['performancelabels'][value] }
          },
        title: 
          {
          text: text['oschanges']['yaxis']['score'],
          style: 
            {
            color: OSChangeColors['score']
            }
          }
        }
      ],
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      },
    grid:
      {
      borderColor: gridColor()
      }
    };

  // Create the next button.
  createNavigationButtons('oschanges');
      
  drawChart('oschanges', options);
  }

// Generate series data for operating system changes.
function getOSChangesSeries(labelValues, datatype, keys)
  {
  var metrics = {};
  
  // Sort version numbers.
  var versionNumbers = Object.keys(insights[datatype].data);
  
  var sortedVersionNumbers =
    versionNumbers.sort(
      function (a, b) 
        {
        if(a == "10.9")
          a = "10.09";
          
        if(b == "10.9")
          b = "10.09";
        
        return Number(a) - Number(b);
        });
       
  // Iterate through versions. 
  for(var i = 0; i < sortedVersionNumbers.length; ++i)
    {
    var versionNumber = sortedVersionNumbers[i];
    
    // Get data for this version.
    var data = insights[datatype].data[versionNumber];
    
    // Iterate through all metrics. 
    for(var metricname in data)
      {
      var metric = metrics[metricname];
      
      // Create the metric if it doesn't exist.
      if(metric == undefined)
        {
        metric =
          {
          name: labelValues[metricname],
          data: []
          };
        
        metrics[metricname] = metric;
        }
        
      metric.data.push(parseFloat(data[metricname]));
      }
    }
    
  // Convert the metrics to a series array.
  var series = new Array;
  
  for(var i = 0; i < keys.length; ++i)
    {
    var key = keys[i];
    series.push(metrics[key]);
    }
    
  return series;
  }

// Get operating system version categories for operating system changes.
function getOSChangesCategories(datatype)
  {
  var versions = {};
  
  // Get all version numbers.
  var versionNumbers = Object.keys(insights[datatype].data);
  
  for(var i = 0; i < versionNumbers.length; ++i)
    {
    var versionNumber = versionNumbers[i];
    
    versions[versionNumber] = 1;
    }
    
  // Sort the versions.
  var keys = Object.keys(versions);
  
  var sortedKeys =
    versionNumbers.sort(
      function (a, b) 
        {
        if(a == "10.9")
          a = "10.09";
          
        if(b == "10.9")
          b = "10.09";
        
        return Number(a) - Number(b);
        });
          
  // Return the versions with the full version number.
  var categories = new Array;
  
  for(var i = 0; i < sortedKeys.length; ++i)
    {
    var version = sortedKeys[i];
    
    categories.push(version);
    }
    
  return categories;
  }

// Draw antivirus data.
function drawAV()
  {
  // Setup the antivirus chart.
  var options = 
    {
    chart: 
      {
      type: 'line',
      animations:
        {
        enabled: enableAnimations,
        speed: 800,
        animateGradually: 
          {
          enabled: false,
          delay: 150
          },
        dynamicAnimation: 
          {
          enabled: enableAnimations,
          speed: 350
          }
        },
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
     },
    labels: getRawLabels('av'),
    series: getAVSeries(text['av']['labels'], 'av'),
    colors: getColors('av'),
    dataLabels: 
      {
      enabled: false
      },
    stroke: 
      {
      curve: 'smooth',
      width: 2
      },
    xaxis: 
      {
      type: 'datetime',
      labels:
        {
        style:
          {
          colors: textColor()
          }
        }
      },
    yaxis:
      {
      labels:
        {
        formatter: (value) => { return value.toFixed(0) + '%' },
        style:
          {
          colors: textColor()
          }
        }
      },
    tooltip:
      {
      y: 
        {
        formatter: (value) => { return value.toFixed(1) + '%' },
        },
      x: 
        {
        format: 'MMM yyyy',
        },
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      },
    grid:
      {
      borderColor: gridColor()
      }
    };

  // Create the next button.
  createNavigationButtons('av');
      
  drawChart('av', options);
  }

// Get series data for the antivirus data.
function getAVSeries(labelValues, datatype)
  {
  var series = {};
  
  // Sort the timestamps.
  var timestamps = Object.keys(insights[datatype].data);
  
  var sortedTimestamps =
    timestamps.sort(
      function (a, b) 
        {
        return parseInt(a) - parseInt(b);
        });
        
  // Iterate through the timestamps.
  for(var i = 0; i < sortedTimestamps.length; ++i)
    {
    var timestamp = sortedTimestamps[i];
    
    var data = insights[datatype].data[timestamp];
    
    for(var key in data)
      {
      var element = series[key];
      
      // Create an element if it doesn't already exist.
      if(element == undefined)
        {
        element =
          {
          name: labelValues[key],
          data: []
          };
        
        series[key] = element;
        }
        
      var time = parseInt(timestamp);
      
      if((time > minTimestamp) && (time < maxTimestamp))
        element.data.push([time, parseFloat(data[key])]);
      }
    }
    
  var keys = Object.keys(series);
  
  // Now return the series array.
  var seriesArray = new Array;
  
  for(var i = 0; i < keys.length; ++i)
    {
    var key = keys[i];
    seriesArray.push(series[key]);
    }
    
  return seriesArray;
  }

// Draw antivirus vendors.
function drawAntivirusVendors()
  {
  var options =
    {
    chart: 
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'pie',
      width: '80%',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    labels: getLabels(avvendors, 'avvendors'),
    series: getSeries('avvendors'),
    colors: getTopColors('avvendors'),
    tooltip:
      {
      y:
        {
        formatter: (value) => { return value + '%' }
        }
      },
    dataLabels:
      {
      dropShadow:
        {
        enabled: false
        }
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      }
    };

  // Create the next button.
  createNavigationButtons('avvendors');
  
  drawChart('avvendors', options);
  }

function drawTopAVFileSystemTime()
  {
  drawAntivirusPerformance('topav', 'filesystemtime');
  }

function drawTopAVFileScore()
  {
  drawAntivirusPerformance('topav', 'score');
  }
  
function drawTopAVFileHasAdware()
  {
  drawAntivirusPerformance('topav', 'hasadware');
  }

// Draw antivirus vendor performance.
function drawAntivirusPerformance(key, drawMetric)
  {
  var section = key + '_' + drawMetric;
  
  var options =
    {
    chart: 
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'bar',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    dataLabels:
      {
      enabled: true,
      style: 
        {
        fontSize: '12px',
        colors: ['#fff']
        },
      formatter: 
        (value) =>
          {
          if(drawMetric == 'hasadware')
            return value.toFixed(2) + "%";
            
          return value;
          },
      dropShadow:
        {
        enabled: false
        }
      },
    series: getTopAVSeries(text[key]['labels'], 'topav', drawMetric),
    colors: getColors(key),
    xaxis: 
      {
      categories: getTopAVCategories(avvendors, key),
      labels:
        {
        style:
          {
          colors: textColor()
          }
        }
      },
    yaxis:
      {
      title: 
        {
        text: text[section]['yaxis'],
        style:
          {
          color: textColor()
          }
        },
      labels:
        {
        formatter: 
          (value) =>
            {
            if(drawMetric == 'score')
              return text['performancelabels'][value];
            else if(drawMetric == 'hasadware')
              return value.toFixed(0) + "%";
            else
              return value;
            },
        style:
          {
          colors: textColor()
          }
        }
      },
    tooltip:
      {
      y: 
        {
        formatter: 
          (value) =>
            {
            if(drawMetric == 'filesystemtime')
              return value + ' secs';
              
            else if(drawMetric == 'hasadware')
              return value.toFixed(2) + "%";
            
            return value;
            }
        }
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      },
    grid:
      {
      borderColor: gridColor()
      }
    };

  // Create the next button.
  createNavigationButtons(section);
            
  drawChart(section, options);
  }

function getTopAVSeries(labelValues, datatype, drawMetric)
  {
  var metrics = {};
  
  // Get a sorted array of vendor names.
  var vendorNames = Object.keys(insights[datatype].data);
  
  var sortedVendorNames = vendorNames.sort();
        
  // Iterate through sorted vendor names.
  for(var i = 0; i < sortedVendorNames.length; ++i)
    {
    var vendor = sortedVendorNames[i];
    
    // Get data for this vendor.
    var vendorData = insights[datatype].data[vendor];
    
    // Iterate through all metrics. 
    for(var metricname in vendorData)
      if(metricname == drawMetric)
        {
        var metric = metrics[metricname];
      
        // Create the metric if it doesn't exist.
        if(metric == undefined)
          {
          metric =
            {
            name: labelValues[metricname],
            data: []
            };
        
          metrics[metricname] = metric;
          }
        
        metric.data.push(parseFloat(vendorData[metricname]));
        }
    }
    
  // Convert the metrics to a series array.
  var keys = Object.keys(metrics);
  
  var sortedKeys = keys.sort();

  var series = new Array;
  
  for(var i = 0; i < sortedKeys.length; ++i)
    {
    var key = sortedKeys[i];
    series.push(metrics[key]);
    }
    
  return series;
  }

function getTopAVCategories(vendorNames, datatype)
  {
  var vendors = {};
  
  // Iterate through vendors.
  for(var vendor in insights[datatype].data)
    {
    vendors[vendor] = 1;
    }
    
  // Convert the metrics to a series array.
  var keys = Object.keys(vendors);
  
  var sortedKeys = keys.sort();

  var categories = new Array;
  
  for(var i = 0; i < sortedKeys.length; ++i)
    {
    var key = sortedKeys[i];
    
    categories.push(vendorNames[key]);
    }
    
  return categories;
  }

// Draw Time Machine statistics.
function drawTimeMachine()
  {
  var options =
    {
    chart: 
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'pie',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    labels: getLabels(text['tm']['labels'], 'tm'),
    series: getSeries('tm'),
    colors: getColors('tm'),
    tooltip:
      {
      y:
        {
        formatter: (value) => { return value + '%' }
        }
      },
    dataLabels:
      {
      dropShadow:
        {
        enabled: false
        }
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      }
    };

  // Create the next button.
  createNavigationButtons('tm');
  
  drawChart('tm', options);
  }

// Draw hard drive speed.
function drawDiskSpeed()
  {
  var xaxisAnnotations =
    [
      {
      x: 20,
      x2: 200,
      borderColor: '#ef9a9a',
      opacity: 0.2,
      label:
        {
        borderColor: '#ef9a9a',
        style:
          {
          fontSize: '10px',
          color: '#fff',
          background: '#ef9a9a',
          },
        offsetY: -10,
        text: text['diskspeed']['xaxis']['mechanical'],
        }
      },
      {
      x: 400,
      x2: 1000,
      borderColor: '#ef9a9a',
      opacity: 0.2,
      label:
        {
        borderColor: '#ef9a9a',
        style:
          {
          fontSize: '10px',
          color: '#fff',
          background: '#ef9a9a',
          },
        offsetY: -10,
        text: text['diskspeed']['xaxis']['sata'],
        }
      },
      {
      x: 1200,
      x2: 3200,
      borderColor: '#ef9a9a',
      opacity: 0.2,
      label:
        {
        borderColor: '#ef9a9a',
        style:
          {
          fontSize: '10px',
          color: '#fff',
          background: '#ef9a9a',
          },
        offsetY: -10,
        text: text['diskspeed']['xaxis']['modern'],
        }
      }
    ];

  var readspeed = machineData["readspeed"];
  var writespeed = machineData["writespeed"];
  
  if(readspeed != undefined && writespeed != undefined)
    {
    var difference = Math.abs(readspeed - writespeed);
    
    if(difference < 100)
      {
      var speed = (parseInt(readspeed) + parseInt(writespeed)) / 2;
      
      xaxisAnnotations.push(
        {
        strokeDashArray: 0,
        x: speed,
        borderColor: "#555555",
        label:
          {
          borderColor: "#555555",
          style:
            {
            color: "#ffffff",
            background: "#555555"
            },
          text: text["yourdiskspeed"]
          }
        });
      }
    else
      {
      xaxisAnnotations.push(
        {
        strokeDashArray: 0,
        x: writespeed,
        borderColor: "#222222",
        label:
          {
          borderColor: "#222222",
          style:
            {
            color: "#ffffff",
            background: "#222222"
            },
          text: text["yourwritespeed"]
          }
        });

      xaxisAnnotations.push(
        {
        strokeDashArray: 0,
        x: readspeed,
        borderColor: "#777777",
        label:
          {
          borderColor: "#777777",
          style:
            {
            color: "#ffffff",
            background: "#777777"
            },
          text: text["yourreadspeed"]
          }
        });
      }
    }
    
  var speeds = getDiskSpeeds(text['diskspeed']['labels'], 'diskspeed');
  
  var options =
    {
    chart: 
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'line',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    stroke:
      {
      curve: 'smooth',
      width: 2
      },
    series: getDiskSpeedSeries(speeds),
    colors: diskColors,
    xaxis:
      {
      type: 'numeric',
      min: speeds['mechanical_readspeed'].min,
      max: speeds['mechanical_readspeed'].max,
      labels:
        {
        formatter:
          (value) =>
            {
            var result = value.toFixed(0) + ' ' + localization['MB/s'];
            
            return result;
            },
        style:
          {
          colors: textColor()
          }
        }
      },
    yaxis:
      {
      labels:
        {
        formatter: (value) => { return value.toFixed(0) + '%' },
        style:
          {
          colors: textColor()
          }
        }
      },
    tooltip:
      {
      y: 
        {
        formatter: (value) => { return value.toFixed(0) + '%' }
        }
      },
    annotations:
      {
      xaxis: xaxisAnnotations
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      },
    grid:
      {
      borderColor: gridColor()
      }
    };

  // Create the next button.
  createNavigationButtons('diskspeed');
  
  drawChart('diskspeed', options);
  }

// Disk speeds are reported by bucket.
function getDiskSpeeds(labelValues, datatype)
  {
  var counts = {};
  
  var speeds = {};
  
  // Get disk types
  var types = Object.keys(insights[datatype].data);
  
  // Go through the types.
  for(var i = 0; i < types.length; ++i)
    {
    var type = types[i];
    
    var metrics = insights[datatype].data[type];
    
    // Go through metrics.
    for(var metric in metrics)
      {
      // Go through speeds.
      var metricSpeeds = metrics[metric];
      
      for(var speed in metricSpeeds)
        speeds[speed] = 1;
      }
    }
    
  var keys = Object.keys(speeds);
  
  var sortedSpeeds =
    keys.sort(
      function(a,b)
        {
        return parseInt(a) - parseInt(b);
        });

  // Go through the types.
  for(var i = 0; i < types.length; ++i)
    {
    var type = types[i];
    
    var metrics = insights[datatype].data[type];
    
    // Go through metrics.
    for(var metric in metrics)
      {
      var key = type + '_' + metric;
      
      // Go through speeds.
      var metricSpeeds = metrics[metric];
      
      for(var j = 0; j < sortedSpeeds.length; ++j)
        {
        var speed = sortedSpeeds[j];
        
        var count = counts[key];
        
        // Create a new count, if necessary.
        if(count == undefined)
          {
          count =
            {
            name: labelValues[key],
            data: [],
            min: 100,
            max: 100
            };
        
          counts[key] = count;
          }
        
        // Add this timestamp value for this version.
        var value = metricSpeeds[speed];
        
        if(value == undefined)
          value = "0";
      
        var speedValue = parseInt(speed);
        
        if(speedValue < count.min)
          count.min = speedValue;
        else if(speedValue > count.max)
          count.max = speedValue;
          
        count.data.push([speedValue, parseFloat(value)]);
        }
      }
    }
    
  return counts;
  }
  
function getDiskSpeedSeries(speeds)
  {
  // Build the series.
  var series = new Array;
  
  series.push(speeds['mechanical_readspeed']);
  series.push(speeds['mechanical_writespeed']);
  series.push(speeds['solidstate_readspeed']);
  series.push(speeds['solidstate_writespeed']);
  series.push(speeds['fusion_readspeed']);
  series.push(speeds['fusion_writespeed']);
  
  return series;
  }

// Draw hard drive filesystem speed.
function drawFilesystemSpeed()
  {
  var xaxisAnnotations =
    [
      {
      x: 5,
      x2: 60,
      borderColor: '#ef9a9a',
      opacity: 0.2,
      label:
        {
        borderColor: '#ef9a9a',
        style:
          {
          fontSize: '10px',
          color: '#fff',
          background: '#ef9a9a',
          },
        offsetY: -10,
        text: text['filesystem']['xaxis']['normal'],
        }
      },
      {
      x: 100,
      x2: 130,
      borderColor: '#ef9a9a',
      opacity: 0.2,
      label:
        {
        borderColor: '#ef9a9a',
        style:
          {
          fontSize: '10px',
          color: '#fff',
          background: '#ef9a9a',
          },
        offsetY: -10,
        text: text['filesystem']['xaxis']['failure'],
        }
      }
    ];

  var filesystemtime = machineData["filesystemtime"];
  
  if(filesystemtime != undefined)
    xaxisAnnotations.push(
      {
      strokeDashArray: 0,
      x: filesystemtime,
      borderColor: "#555555",
      label:
        {
        borderColor: "#555555",
        style:
          {
          color: "#ffffff",
          background: "#555555"
          },
        text: text["yourfilesystemtime"]
        }
      });

  var speeds =
    getFilesystemSpeeds(text['filesystem']['labels'], 'filesystem');
  
  var options =
    {
    chart: 
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'line',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    stroke:
      {
      curve: 'smooth',
      width: 2
      },
    series: getFilesystemSpeedSeries(speeds),
    colors: filesystemColors,
    xaxis:
      {
      type: 'numeric',
      min: 0,
      max: speeds['mechanical'].max,
      labels:
        {
        formatter: (value) => { return value.toFixed(0) + ' secs' },
        style:
          {
          colors: textColor()
          }
        }
      },
    yaxis:
      {
      labels:
        {
        formatter: (value) => { return value.toFixed(0) + '%' },
        style:
          {
          colors: textColor()
          }
        }
      },
    tooltip:
      {
      y: 
        {
        formatter: (value) => { return value.toFixed(0) + '%' }
        }
      },
    annotations: 
      {
      xaxis: xaxisAnnotations
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      },
    grid:
      {
      borderColor: gridColor()
      }
    };

  // Create the next button.
  createNavigationButtons('filesystem');
  
  drawChart('filesystem', options);
  }

// Disk speeds are reported by bucket.
function getFilesystemSpeeds(labelValues, datatype)
  {
  var counts = {};
  
  var speeds = {};
  
  // Get disk types
  var types = Object.keys(insights[datatype].data);
  
  // Go through the types.
  for(var i = 0; i < types.length; ++i)
    {
    var type = types[i];
    
    var filesystemSpeeds = insights[datatype].data[type];
    
    for(var speed in filesystemSpeeds)
      speeds[speed] = 1;
    }
    
  var keys = Object.keys(speeds);
  
  var sortedSpeeds =
    keys.sort(
      function(a,b)
        {
        return parseInt(a) - parseInt(b);
        });

  // Go through the types.
  for(var i = 0; i < types.length; ++i)
    {
    var type = types[i];
    
    var filesystemSpeeds = insights[datatype].data[type];
         
    for(var j = 0; j < sortedSpeeds.length; ++j)
      {
      var speed = sortedSpeeds[j];
      
      var count = counts[type];
      
      // Create a new count, if necessary.
      if(count == undefined)
        {
        count =
          {
          name: labelValues[type],
          data: [],
          min: 100,
          max: 100
          };
      
        counts[type] = count;
        }
      
      // Add this timestamp value for this version.
      var value = filesystemSpeeds[speed];
      
      if(value == undefined)
        value = "0";
    
      var speedValue = parseInt(speed);
      
      if(speedValue < count.min)
        count.min = speedValue;
      else if(speedValue > count.max)
        count.max = speedValue;
        
      count.data.push([speedValue, parseFloat(value)]);
      }
    }

  return counts;
  }
  
function getFilesystemSpeedSeries(speeds)
  {
  // Build the series.
  var series = new Array;
  
  series.push(speeds['mechanical']);
  series.push(speeds['solidstate']);
  series.push(speeds['fusion']);
  
  return series;
  }

// Draw hard drive failure.
function drawHdFailure()
  {
  var annotations =
    {
    yaxis:
      [{
      strokeDashArray: 0,
      y: parseFloat(insights['hdfailure'].data['all'].rate),
      borderColor: "#E30000",
      label:
        {
        borderColor: "#E30000",
        style:
          {
          color: "#ffffff",
          background: "#E30000"
          },
        text: text["allmachines"]
        }
      }]
    };
    
  var options =
    {
    chart:
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'bar',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    labels: getHDFailureLabels('hdfailure'),
    series: getHDFailureSeries('hdfailure'),
    colors: getColors('hdfailure'),
    dataLabels:
      {
      formatter: (value) => { return value.toFixed(2) + ' %' }
      },
    tooltip:
      {
      y:
        {
        formatter: (value) => { return value.toFixed(2) + ' %' }
        }
      },
    yaxis:
      {
      max: 25,
      labels:
        {
        formatter: (value) => { return value.toFixed(0) + ' %' },
        style:
          {
          colors: textColor()
          }
        }
      },
    xaxis:
      {
      labels:
        {
        style:
          {
          colors: textColor()
          },
        rotate: -45,
        rotateAlways: true
        }
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      },
    grid:
      {
      borderColor: gridColor()
      },
    annotations: annotations
    };

  // Create the next button.
  createNavigationButtons('hdfailure');
    
  drawChart('hdfailure', options);
  }

function getHDFailureLabels(datatype)
  {
  var labels = new Array;
  
  for(var key in insights[datatype].data)
    if(key != 'all')
      labels.push(key);
    
  return labels;
  }

function getHDFailureSeries(datatype)
  {
  var data =
    {
    name: text['hdfailurerate'],
    data: []
    };
    
  for(var key in insights[datatype].data)
    if(key != 'all')
      data.data.push(parseFloat(insights[datatype].data[key].rate));
    
  var series = new Array;
  
  series.push(data);
    
  return series;
  }

// Draw top system modifications.
function drawTopMods()
  {
  var options =
    {
    chart:
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'pie',
      width: '100%',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    labels: getRawLabels('topmods'),
    series: getSeries('topmods'),
    colors: getTopColors('topmods'),
    tooltip:
      {
      y:
        {
        formatter: (value) => { return value + '%' }
        }
      },
    dataLabels:
      {
      dropShadow:
        {
        enabled: false
        }
      },
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      }
    };

  // Create the next button.
  createNavigationButtons('topmods');
  
  drawChart('topmods', options);
  }

// Draw system modifications.
function drawSystemMods()
  {
  var annotations = undefined;
  
  var performance = machineData["performance"];
  var seconds = machineData["seconds"];

  var systemextensions = machineData["systemextensions"];
  var kernelextensions = machineData["kernelextensions"];
  var launchagents = machineData["launchagents"];
  var launchdaemons = machineData["launchdaemons"];
  var userlaunchagents = machineData["userlaunchagents"];
  
  if(performance != undefined && seconds != undefined)
    if(systemextensions != undefined && kernelextensions != undefined)
      if(launchagents != undefined && launchdaemons != undefined)
        if(userlaunchagents != undefined)
          {
          var total =
            parseInt(systemextensions)
              + parseInt(kernelextensions)
              + parseInt(launchagents)
              + parseInt(launchdaemons)
              + parseInt(userlaunchagents);
      
          total = Math.round(total/10) * 10;
          
          annotations =
            {
            points:
              [
                {
                x: total.toString(10),
                y: performance,
                yAxisIndex: 1,
                seriesIndex: 1,
                marker:
                  {
                  size: 6,
                  fillColor: colors[1],
                  strokeWidth: 0
                  },
                label:
                  {
                  borderColor: colors[1],
                  style:
                    {
                    color: "#ffffff",
                    background: colors[1]
                    },
                  text: text["yourperformance"]
                  }
                },
                {
                x: total.toString(10),
                y: parseFloat(seconds) / 60,
                yAxisIndex: 2,
                seriesIndex: 2,
                marker:
                  {
                  size: 6,
                  fillColor: colors[2],
                  strokeWidth: 0
                  },
                label:
                  {
                  borderColor: colors[2],
                  style:
                    {
                    color: "#ffffff",
                    background: colors[2]
                    },
                  text: text["yourruntime"]
                  }
                }
              ]
            };
          }

  var mods = getSystemMods('systemmods');
  
  var options =
    {
    chart: 
      {
      animations:
        {
        enabled: enableAnimations
        },
      type: 'line',
      toolbar:
        {
        show: false
        },
      events:
        {
        mounted:
          function(chartContext, config)
            {
            showText();
            }
        }
      },
    stroke:
      {
      curve: 'smooth',
      width: 2
      },
    series: getSystemModsSeries(text['systemmods']['labels'], mods),
    colors: colors,
    xaxis:
      {
      type: 'category',
      categories: getSystemModsCategories(mods),
      title:
        {
        text: text['systemmods']['xaxis'],
        style:
          {
          color: textColor()
          }
        },
      labels:
        {
        style:
          {
          colors: textColor()
          }
        }
      },
    yaxis:
      [
        {
        min: 0,
        max: 100,
        axisTicks: 
          {
          show: false,
          },
        axisBorder: 
          {
          show: true,
          color: colors[0]
          },
        labels: 
          {
          style: 
            {
            colors: colors[0]
            },
          formatter: (value) => { return value.toFixed(2) + '%' }
          },
        title: 
          {
          text: text['systemmods']['labels']['count'],
          style: 
            {
            color: colors[0]
            }
          }
        },
        {
        min: 0,
        max: 3,
        tickAmount: 3,
        opposite: true,
        axisTicks: 
          {
          show: true,
          },
        axisBorder: 
          {
          show: true,
          color: colors[1]
          },
        labels: 
          {
          style: 
            {
            colors: colors[1]
            },
          formatter:
            (value) =>
              {
              var result = text['performancelabels'][value];
              
              if(result == undefined)
                result = value;
              
              return result;
              }
          },
        title: 
          {
          text: text['systemmods']['labels']['score'],
          style: 
            {
            color: colors[1]
            }
          }
        },
        {
        min: 2,
        max: 10,
        axisTicks: 
          {
          show: true,
          },
        axisBorder: 
          {
          show: true,
          color: colors[2]
          },
        labels: 
          {
          style: 
            {
            colors: colors[2]
            },
          formatter: 
            (value) => 
              { 
              return value.toFixed(2) + ' min';
              }
          },
        title: 
          {
          text: text['systemmods']['labels']['runtime'],
          style: 
            {
            color: colors[2]
            }
          }
        },
      ],
    legend:
      {
      labels:
        {
        colors: textColor()
        }
      },
    grid:
      {
      borderColor: gridColor()
      },
    annotations: annotations
    };

  // Create the next button.
  createNavigationButtons('systemmods');
  
  drawChart('systemmods', options);
  }

// System modifications are reported by bucket.
function getSystemMods(datatype)
  {
  var mods = {};
  
  // Go through the mods.
  for(var key in insights[datatype].data)
    {
    var info = insights[datatype].data[key];
         
    var count = mods[key];
      
    // Create a new count, if necessary.
    if(count == undefined)
      {
      count =
        {
        name: key,
        data: info
        };
    
      mods[key] = count;
      }
    }

  return mods;
  }
  
function getSystemModsSeries(labelValues, mods)
  {
  var count = 
    {
    name: labelValues["count"],
    data: new Array
    };
    
  var score =
    {
    name: labelValues["score"],
    data: new Array
    };
    
  var runtime = 
    {
    name: labelValues["runtime"],
    data: new Array
    };
    
  var sortedCounts = Object.keys(mods).sort(
    function(a,b)
      {
      return parseInt(a) > parseInt(b)
      });
      
  for(var i = 0; i < sortedCounts.length; ++i)
    {
    var key = sortedCounts[i];
    
    var info = mods[key];
    
    count.data.push(info.data["count"]);
    score.data.push(info.data["score"]);
    runtime.data.push(info.data["runtime"]);
    }
    
  // Build the series.
  var series = new Array;
  
  series.push(count);
  series.push(score);
  series.push(runtime);
  
  return series;
  }

function getSystemModsCategories(mods)
  {
  // Build the categories.
  var categories = new Array;
  
  var sortedCounts = Object.keys(mods).sort(
    function(a,b)
      {
      return parseInt(a) > parseInt(b)
      });
      
  for(var i = 0; i < sortedCounts.length; ++i)
    {
    var key = sortedCounts[i];
    
    categories.push(key);
    }
      
  return categories;
  }

function createNavigationButtons(key)
  {
  var previousKey = getPreviousKey(key);
  
  if(previousKey != undefined)
    {
    var previousTitle = text['previous'] + text[previousKey]['title'];

    createNavigationButton('previous', previousTitle, next[previousKey]);
    }
  else
    removeNavigationButton('previous');
    
  var nextKey = getNextKey(key);
  
  if(nextKey != undefined)
    {
    var nextTitle = text['next'] + text[nextKey]['title'];

    createNavigationButton('next', nextTitle, next[nextKey]);
    }
  else
    removeNavigationButton('next');
  }
  
function getPreviousKey(key)
  {
  var previousKey = text[key]['previous'];
  
  if(previousKey == 'intro')
    return previousKey;
    
  var insightsKey = previousKey;
  
  if(text[previousKey] != undefined)
    if(text[previousKey]['key'] != undefined)
      insightsKey = text[previousKey]['key'];

  if(insightsKey == 'models')
    if(machineData["name"] != undefined)
      if(machineTypes[machineData["name"]] != undefined)
        insightsKey = 'machine';
    
  while(insights[insightsKey] == undefined)
    {
    key = previousKey;
    
    if(text[key] == undefined)
      return undefined;
      
    previousKey = text[key]['previous'];

    insightsKey = previousKey;

    if(text[previousKey] != undefined)
      if(text[previousKey]['key'] != undefined)
        insightsKey = text[previousKey]['key'];
  
    if(insightsKey == 'models')
      if(machineData["name"] != undefined)
        if(machineTypes[machineData["name"]] != undefined)
          insightsKey = 'machine';
    }
    
  return previousKey;
  }
  
function getNextKey(key)
  {
  var nextKey = text[key]['next'];
  
  if(nextKey == 'more')
    return nextKey;
    
  var insightsKey = nextKey;
  
  if(text[nextKey] != undefined)
    if(text[nextKey]['key'] != undefined)
      insightsKey = text[nextKey]['key'];
    
  if(insightsKey == 'models')
    if(machineData["name"] != undefined)
      if(machineTypes[machineData["name"]] != undefined)
        insightsKey = 'os';
    
  while(insights[insightsKey] == undefined)
    {
    key = nextKey;
    
    if(text[key] == undefined)
      return undefined;
      
    nextKey = text[key]['next'];

    insightsKey = nextKey;

    if(text[nextKey] != undefined)
      if(text[nextKey]['key'] != undefined)
        insightsKey = text[nextKey]['key'];

    if(insightsKey == 'models')
      if(machineData["name"] != undefined)
        if(machineTypes[machineData["name"]] != undefined)
          insightsKey = 'os';
    }
    
  return nextKey;
  }
  
function removeNavigationButton(key)
  {
  var button = document.getElementById(key)
  
  if(button != undefined)
    button.remove();
  }
  
function createNavigationButton(key, title, fn)
  {
  removeNavigationButton(key);
  
  button = document.createElement("button");
  
  button.setAttribute("id", key);
  button.setAttribute("class", "navigation");
  
  button.addEventListener("click", fn);

  var titleNode = document.createTextNode(title);
  button.appendChild(titleNode);

  document.body.appendChild(button);
  }

function getSeries(datatype)
  {
  var series = new Array;
  
  for(var key in insights[datatype].data)
    series.push(parseFloat(insights[datatype].data[key]));
    
  return series;
  }
  
function getLabels(labelValues, datatype)
  {
  var labels = new Array;

  for(var key in insights[datatype].data)
    labels.push(labelValues[key]);
    
  return labels;
  }

function getRawLabels(datatype)
  {
  var labels = new Array;
  
  for(var key in insights[datatype].data)
    if(key == "other")
      labels.push(localization["other"]);
    else
      labels.push(key);
    
  return labels;
  }

function getOrderedLabels(datatype, keys)
  {
  var labels = new Array;
  
  for(var i = 0; i < keys.length; ++i)
    {
    var key = keys[i];
    
    labels.push(insights[datatype].data[key]);
    }
    
  return labels;
  }

function getColors(datatype)
  {
  var labelColors = new Array;
  
  var index = 0;
  
  for(var key in insights[datatype].data)
    labelColors.push(colors[index++]);
    
  return labelColors;
  }

function getOrderedColors(orderedColors, keys)
  {
  var labelColors = new Array;
  
  for(var i = 0; i < keys.length; ++i)
    {
    var key = keys[i];
    
    labelColors.push(orderedColors[key]);
    }
    
  return labelColors;
  }

function getTopColors(datatype)
  {
  var labelColors = new Array;
  
  var index = 0;
  
  for(var key in insights[datatype].data)
    {
    if(index >= colors.length)
      index = 0;
      
    var value = parseFloat(insights[datatype].data[key]);
    
    if(value > 1)
      labelColors.push(colors[index++]);
    else
      labelColors.push(grey);
    }
    
  return labelColors;
  }
  
function drawChart(section, options)
  {
  updateCurrentSection(section);

  drawHTML(section);

  // Create the chart element.
  createChart();
  
  // Draw the chart.
  var chart = new ApexCharts(document.querySelector("#chart"), options);

  chart.render();
  
  if(enableAnimations)
    hideTextElements();
  }
  
function updateCurrentSection(section)
  {
  currentSection = section;

  if(window.webkit != undefined)
    window.webkit.messageHandlers.updateCurrentSection.postMessage(section);
    
  else if(window.updateCurrentSection != undefined)
    window.updateCurrentSection.postMessage(section);
  }

function closeWindow()
  {
  if(window.webkit != undefined)
    window.webkit.messageHandlers.closeInsights.postMessage("close");
    
  else if(window.closeInsights != undefined)
    window.closeInsights.postMessage("close");
  }

function drawHTML(section)
  {
  var title = document.getElementById("title")

  title.innerHTML = text[section]['title'];
  
  var commentarySection = document.getElementById("commentary")

  commentarySection.innerHTML = commentary[section];
  
  deleteChart();
  }

function deleteChart()
  {
  var chartDiv = document.getElementById("chart")
  
  if(chartDiv != undefined)
    chartDiv.remove();
  }
  
function createChart()
  {
  var chartWrapper = document.getElementById("chartwrapper")

  chartDiv = document.createElement("div");
  
  chartDiv.setAttribute("id", "chart");
  
  var commentary = document.getElementById("commentary")

  chartWrapper.insertBefore(chartDiv, commentary);
  }

function showText()
  {
  if(enableAnimations)
    setTimeout(
      function()
        {
        showTextElements();
        },
      800);
    
  enableAnimations = true;
  }

function hideTextElements()
  {
  var elements = document.querySelectorAll('.apexcharts-data-labels');
  
  elements.forEach(
    function(element)
      {
      element.style.display = 'none';
      });

  elements = document.querySelectorAll('.apexcharts-pie-label');
  
  elements.forEach(
    function(element)
      {
      element.style.display = 'none';
      });
  }

function showTextElements()
  {
  var elements = document.querySelectorAll('.apexcharts-data-labels');
  
  elements.forEach(
    function(element)
      {
      element.style.display = 'block';
      });

  elements = document.querySelectorAll('.apexcharts-pie-label');
  
  elements.forEach(
    function(element)
      {
      element.style.display = 'block';
      });
  }
  
function textColor()
  {
  var appearance = document.body.className;

  if(appearance.startsWith("DarkAqua"))
    return 'rgba(224,255,219,1)';
    
  return 'rgba(10,51,10,1)';
  }

function gridColor()
  {
  var appearance = document.body.className;
  
  if(appearance.startsWith("DarkAqua"))
    return 'rgba(100,100,100,1)';
    
  return 'rgba(224,224,224,1)';
  }

function buyPowerUser()
  {
  if(window.webkit != undefined)
    window.webkit.messageHandlers.buyPowerUser.postMessage("buy");
    
  else if(window.buyPowerUser != undefined)
    window.buyPowerUser.postMessage("buy");
  }
