<?xml version="1.0"?>
<xsl:stylesheet 
  version='1.0' 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  extension-element-prefixes="exslt">

  <!-- Convert sections of an EtreCheck report into HTML 
       representations. -->
  <xsl:output method="text" encoding="UTF-8"/> 
 
  <xsl:variable name="fulldriveaccess" select="/etrecheck/header/fulldriveaccess"/>

  <xsl:template match='/etrecheck'>
  
    <xsl:call-template name="header"/>
    <xsl:call-template name="hardware"/>
    <xsl:call-template name="video"/>
    <xsl:call-template name="disk"/>
    <xsl:call-template name="virtualvolume"/>
    <xsl:call-template name="network"/>
    <xsl:call-template name="systemsoftware"/>
    <xsl:call-template name="configurationfiles"/>
    <xsl:call-template name="notifications"/>
    <xsl:call-template name="security"/>
    <xsl:call-template name="adware"/>
    <xsl:call-template name="unsigned"/>
    <xsl:call-template name="cleanup"/>
    <xsl:call-template name="applications"/>
    <xsl:call-template name="systemextensions"/>
    <xsl:call-template name="kernelextensions"/>
    <xsl:call-template name="startupitems"/>
    <xsl:call-template name="systemlaunchagents"/>
    <xsl:call-template name="systemlaunchdaemons"/>
    <xsl:call-template name="launchagents"/>
    <xsl:call-template name="launchdaemons"/>
    <xsl:call-template name="userlaunchagents"/>
    <xsl:call-template name="loginitems"/>
    <xsl:call-template name="internetplugins"/>
    <xsl:call-template name="userinternetplugins"/>
    <xsl:call-template name="audioplugins"/>
    <xsl:call-template name="useraudioplugins"/>
    <xsl:call-template name="itunesplugins"/>
    <xsl:call-template name="useritunesplugins"/>
    <xsl:call-template name="safariextensions"/>
    <xsl:call-template name="preferencepanes"/>
    <xsl:call-template name="timemachine"/>
    <xsl:call-template name="performance"/>
    <xsl:call-template name="cpuusage"/>
    <xsl:call-template name="cpu"/>
    <xsl:call-template name="memory"/>
    <xsl:call-template name="networkusage"/>
    <xsl:call-template name="energy"/>
    <xsl:call-template name="vm"/>
    <xsl:call-template name="install"/>
    <xsl:call-template name="diagnostics"/>
    
  </xsl:template>

  <!-- EtreCheck stats. -->
  <xsl:template name="header">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">version</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/header/version"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">build</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/header/build"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">date</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/header/date"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">yearmonth</xsl:with-param>
      <xsl:with-param name="value" select="concat(substring(/etrecheck/header/date, 1, 4), substring(/etrecheck/header/date,6,2))"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">apptype</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/header/apptype"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">runtime</xsl:with-param>
      <xsl:with-param
        name="value" select="/etrecheck/header/runtime[position()=1]"/>
    </xsl:call-template>

    <xsl:apply-templates select="/etrecheck/header/runtime[1]" mode="seconds"/>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">performance</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="/etrecheck/performance/score = 'excellentperformance'">
            <xsl:text>3</xsl:text>
          </xsl:when>
          <xsl:when test="/etrecheck/performance/score = 'goodperformance'">
            <xsl:text>2</xsl:text>
          </xsl:when>
          <xsl:when test="/etrecheck/performance/score = 'belowaverageperformance'">
            <xsl:text>1</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>0</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param> 
    </xsl:call-template>
      
    <xsl:call-template name="emit">
      <xsl:with-param name="name">problem</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/header/problem"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">hasproblemdescription</xsl:with-param>
      <xsl:with-param 
        name="value" select="string-length(/etrecheck/header/problemdescription)"/>
    </xsl:call-template>
    
  </xsl:template>

  <!-- Hardware information. -->
  <xsl:template name="hardware">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">vintage</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/vintage"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">name</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/name"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">model</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/model"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">cpucount</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/cpucount"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">cpucode</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/cpucode"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">speed</xsl:with-param>
      <xsl:with-param 
        name="value" 
        select="translate(/etrecheck/hardware/speed,',٫٬٠١٢٣٤٥٦٧٨٩','...0123456789')"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">ram</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/ram"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">upgradeable</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/upgradeable"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">applemaximummemory</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/applemaximummemory"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">userupgradeable</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/userupgradeable"/>
    </xsl:call-template>
       
    <xsl:variable
      name="failingram"
      select="count(memorybanks/memorybank[status != 'ok']) > 0"/>
          
    <xsl:call-template name="emit">
      <xsl:with-param name="name">failingram</xsl:with-param>
      <xsl:with-param name="value" select="$failingram"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">batteryhealth</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/batteryhealth"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">batterycyclecount</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/batterycyclecount"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">batterypercent</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/batterypercent"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">marketingname</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/marketingname[lang='en']"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">technicalspecificationsurl</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/technicalspecificationsurl"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">userguideurl</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/hardware/userguideurl"/>
    </xsl:call-template>

  </xsl:template>
 
  <!-- Video information. -->
  <xsl:template name="video">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">resolution</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/video//display[position()=1]/resolution"/>
    </xsl:call-template>
    
  </xsl:template>
  
  <!-- Drive information. -->
  <xsl:template name="disk">
  
    <!-- Only look at the boot disk. For that, we need to start with volumes. -->
    
  </xsl:template>

  <!-- Other volumes information. -->
  <xsl:template name="virtualvolume">
  
    <xsl:choose>
      <xsl:when test="/etrecheck/systemsoftware/majorversion &gt;= 15">
        <xsl:apply-templates select="/etrecheck/virtualvolume" mode="catalina"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:for-each select="/etrecheck/virtualvolume/volume">
          <xsl:variable name="device" select="device"/>
          <xsl:choose>
            <xsl:when test="mountpoint = '/'">
              <xsl:apply-templates select="."/>
            </xsl:when>
            <xsl:otherwise/>
          </xsl:choose>
        </xsl:for-each>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Other volumes information (Catalina). -->
  <xsl:template match="virtualvolume" mode="catalina">
  
    <xsl:variable name="boot" select="exslt:node-set(volume[mountpoint = '/'])"/>
    
    <xsl:variable 
      name="data" select="exslt:node-set(volume[mountpoint = '/System/Volumes/Data'])"/>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">disktype</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:call-template name="disktype">
          <xsl:with-param name="volume" select="volume[mountpoint = '/']"/>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">filesystem</xsl:with-param>
      <xsl:with-param name="value">APFS</xsl:with-param>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">size</xsl:with-param>
      <xsl:with-param name="value" select="$data/size"/>
    </xsl:call-template>

    <xsl:variable name="bootused">
      <xsl:apply-templates select="$boot/used"/>
    </xsl:variable>
    
    <xsl:variable name="dataused">
      <xsl:apply-templates select="$data/used"/>
    </xsl:variable>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">used</xsl:with-param>
      <xsl:with-param name="value" select="number($bootused) + number($dataused)"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">free</xsl:with-param>
      <xsl:with-param name="value" select="$data/free"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">available</xsl:with-param>
      <xsl:with-param name="value" select="$data/available"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">encryption</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="$data/encryption">
            <xsl:value-of select="$data/encryption/status"/>        
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>notencrypted</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
    
  </xsl:template>

  <!-- Storage volume. -->
  <xsl:template match="volume">
    
    <xsl:call-template name="emit">
      <xsl:with-param name="name">disktype</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:call-template name="disktype">
          <xsl:with-param name="volume" select="."/>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">filesystem</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="starts-with(filesystem, 'Mac OS Extended')">
            <xsl:text>HFS+</xsl:text>
          </xsl:when>
          <xsl:when test="starts-with(filesystem, 'Journaled HFS+')">
            <xsl:text>HFS+</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="filesystem"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
    
    <xsl:call-template name="emit">
      <xsl:with-param name="name">size</xsl:with-param>
      <xsl:with-param name="value" select="size"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">used</xsl:with-param>
      <xsl:with-param name="value" select="used"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">free</xsl:with-param>
      <xsl:with-param name="value" select="free"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">available</xsl:with-param>
      <xsl:with-param name="value" select="available"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">encryption</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="encryption">
            <xsl:value-of select="encryption/status"/>        
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>notencrypted</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
    
  </xsl:template>

  <!-- Get the disk type. -->
  <xsl:template name="disktype">
    <xsl:param name="volume"/>
    
    <!-- Get the number of containers. -->
    <xsl:variable name="containercount" select="count($volume/containers/device)"/>

    <xsl:variable name="containerdevice" select="$volume/containers/device"/>
    
    <xsl:choose>
      <xsl:when test="$containercount &gt; 1">
        <xsl:text>fusion</xsl:text>
      </xsl:when>
      <xsl:when test="/etrecheck/disk/drive[device = $containerdevice]/mechanism">
        <xsl:value-of select="/etrecheck/disk/drive[device = $containerdevice]/mechanism"/>
      </xsl:when>
      <xsl:when test="/etrecheck/virtualvolume/volume[device = $containerdevice]">
        <xsl:call-template name="disktype">
          <xsl:with-param name="volume" select="/etrecheck/virtualvolume/volume[device = $containerdevice]"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:text>unknown</xsl:text>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>
  
  <!-- Network information. -->
  <xsl:template name="network">
      
    <!-- Not using this. -->
        
  </xsl:template>
  
  <!-- System software. -->
  <xsl:template name="systemsoftware">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">osversion</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/systemsoftware/version"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">osmajorversion</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/systemsoftware/majorversion"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">uptime</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/systemsoftware/uptime"/>
    </xsl:call-template>

  </xsl:template>

  <!-- Configuration files. -->
  <xsl:template name="configurationfiles">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">hostcount</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/configurationfiles/hostscount"/>
    </xsl:call-template>
            
  </xsl:template>

  <!-- Notification information. -->
  <xsl:template name="notifications">

    <!-- I don't care about this. --> 
        
  </xsl:template>

  <!-- Security information. -->
  <xsl:template name="security">
  
    <xsl:variable name="sip">
      <xsl:value-of select="/etrecheck/security/SIP"/>
    </xsl:variable>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">gatekeeper</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/security/gatekeeper/status"/>
    </xsl:call-template>
    
    <xsl:call-template name="emit">
      <xsl:with-param name="name">sip</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="/etrecheck/systemsoftware/majorversion &lt; 11">
            <xsl:text>notsupported</xsl:text>
          </xsl:when>
          <xsl:when test="$sip = 'enabled'">
            <xsl:text>enabled</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>disabled</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
    
    <xsl:call-template name="emit">
      <xsl:with-param name="name">automaticupdatesdisabled</xsl:with-param>
      <xsl:with-param 
        name="value" 
        select="/etrecheck/security/softwareupdate/automaticupdatesdisabled"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">criticalupdatesdisabled</xsl:with-param>
      <xsl:with-param 
        name="value" select="/etrecheck/security/softwareupdate/criticalupdatesdisabled"/>
    </xsl:call-template>
    
    <xsl:call-template name="emit">
      <xsl:with-param name="name">avcount3rdparty</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="/etrecheck/adware/avcount3rdparty">
            <xsl:value-of select="/etrecheck/adware/avcount3rdparty"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="count(/etrecheck/adware//avtype)"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
    
    <xsl:call-template name="emit">
      <xsl:with-param name="name">av3rdparty</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="/etrecheck/adware/avcount3rdparty = 0">
            <xsl:text>none</xsl:text>
          </xsl:when>
          <xsl:when test="/etrecheck/adware/avcount3rdparty = 1">
            <xsl:value-of select="/etrecheck/adware//avtype[. != 'apple']"/>
          </xsl:when>
          <xsl:when test="not(/etrecheck/adware/avcount3rdparty)">
            <xsl:choose>
              <xsl:when test="count(/etrecheck/adware//avtype[. != 'apple']) &gt; 1">
                <xsl:text>multiple</xsl:text>
              </xsl:when>
              <xsl:when test="count(/etrecheck/adware//avtype[. != 'apple']) = 1">
                <xsl:value-of select="/etrecheck/adware//avtype[. != 'apple']"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>none</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>multiple</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>

  </xsl:template>

  <!-- Adware information. -->
  <xsl:template name="adware">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">adware</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="count(/etrecheck/adware/launchdfiles/launchdfile) &gt; 0 or count(/etrecheck/adware/safariextensions/extension) &gt; 0 or count(/etrecheck/adware/loginitems/loginitem) &gt; 0">
            <xsl:text>1</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>0</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
    
  </xsl:template>

  <!-- Unsigned files information. -->
  <xsl:template name="unsigned">
  
    <xsl:variable
      name="unsafecount"
      select="count(/etrecheck/unsigned//launchdfile[apple != 'true' and (not(details) or (details != 'legitimate' and details != 'legitimate_prefix' and details != 'whitelist' and details != 'whitelistprefix' and details != 'executablemaspermissions'))])"/>
    
    <xsl:call-template name="emit">
      <xsl:with-param name="name">likelyadware</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="$unsafecount &gt; 0">
            <xsl:text>1</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>0</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
        
  </xsl:template>

  <!-- Clean up information. -->
  <xsl:template name="cleanup">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">cleanup</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="count(/etrecheck/cleanup//launchdfile) &gt; 0">
            <xsl:text>1</xsl:text>
          </xsl:when>
          <xsl:when test="count(/etrecheck/cleanup//extension) &gt; 0">
            <xsl:text>1</xsl:text>
          </xsl:when>
          <xsl:when test="count(/etrecheck/cleanup//systemextension[status = 'terminated_waiting_to_uninstall_on_reboot']) &gt; 0">
            <xsl:text>1</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>0</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
        
  </xsl:template>

  <!-- 32-bit Apps. -->
  <xsl:template name="applications">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">count32bitapps</xsl:with-param>
      <xsl:with-param 
        name="value" 
        select="count(/etrecheck/applications/application[has64bit = 'false'])"/>
    </xsl:call-template>
        
  </xsl:template>

  <!-- System extensions. -->
  <xsl:template name="systemextensions">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">systemextensions</xsl:with-param>
      <xsl:with-param 
        name="value" select="count(/etrecheck/systemextensions/systemextension)"/>
    </xsl:call-template>
    
  </xsl:template>

  <!-- Kernel extensions. -->
  <xsl:template name="kernelextensions">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">kernelextensions</xsl:with-param>
      <xsl:with-param 
        name="value" select="count(/etrecheck/kernelextensions//extension)"/>
    </xsl:call-template>
        
  </xsl:template>

  <!-- Print startup items. -->
  <xsl:template name="startupitems">
  
    <!-- I don't care. -->
    
  </xsl:template>

  <!-- Print system launch agents. -->
  <xsl:template name="systemlaunchagents">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">systemlaunchagents</xsl:with-param>
      <xsl:with-param 
        name="value" select="count(/etrecheck/systemlaunchagents/launchdfile)"/>
    </xsl:call-template>

  </xsl:template>
  
  <!-- Print system launch daemons. -->
  <xsl:template name="systemlaunchdaemons">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">systemlaunchdaemons</xsl:with-param>
      <xsl:with-param 
        name="value" select="count(/etrecheck/systemlaunchdaemons/launchdfile)"/>
    </xsl:call-template>

  </xsl:template>

  <!-- Print launch agents. -->
  <xsl:template name="launchagents">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">launchagents</xsl:with-param>
      <xsl:with-param 
        name="value" select="count(/etrecheck/launchagents/launchdfile)"/>
    </xsl:call-template>
        
  </xsl:template>

  <!-- Print launch daemons. -->
  <xsl:template name="launchdaemons">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">launchdaemons</xsl:with-param>
      <xsl:with-param 
        name="value" select="count(/etrecheck/launchdaemons/launchdfile)"/>
    </xsl:call-template>

  </xsl:template>

  <!-- Print user launch agents. -->
  <xsl:template name="userlaunchagents">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">userlaunchagents</xsl:with-param>
      <xsl:with-param 
        name="value" select="count(/etrecheck/userlaunchagents/launchdfile)"/>
    </xsl:call-template>
        
  </xsl:template>

  <!-- Print login items. -->
  <xsl:template name="loginitems">
  
    <!-- I don't care. -->
        
  </xsl:template>

  <!-- Print internet plugins. -->
  <xsl:template name="internetplugins">
  
    <!-- I don't care. -->
        
  </xsl:template>

  <!-- Print user internet plugins. -->
  <xsl:template name="userinternetplugins">
  
    <!-- I don't care. -->
        
  </xsl:template>

  <!-- Print audio plugins. -->
  <xsl:template name="audioplugins">
  
    <!-- I don't care. -->
        
  </xsl:template>

  <!-- Print user audio plugins. -->
  <xsl:template name="useraudioplugins">
  
    <!-- I don't care. -->
        
  </xsl:template>

  <!-- Print iTunes plugins. -->
  <xsl:template name="itunesplugins">
  
    <!-- I don't care. -->
        
  </xsl:template>

  <!-- Print user iTunes plugins. -->
  <xsl:template name="useritunesplugins">
  
    <!-- I don't care. -->
        
  </xsl:template>

  <!-- Print Safari extensions. -->
  <xsl:template name="safariextensions">
  
    <!-- I don't care. -->
        
  </xsl:template>
  
  <!-- Print Preference panes. -->
  <xsl:template name="preferencepanes">
  
    <!-- I don't care. -->
        
  </xsl:template>
  
  <!-- Print Fonts. -->
  <xsl:template name="fonts">
  
    <!-- I don't care. -->
        
  </xsl:template>
  
  <!-- Print Time Machine information. -->
  <xsl:template name="timemachine">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">tmconfigured</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="$fulldriveaccess != 'true' and /etrecheck/timemachine/notconfigured = 'true'">    
            <xsl:text>notconfigured</xsl:text>
          </xsl:when>
          <xsl:when test="$fulldriveaccess != 'true'">
            <xsl:text>unknown</xsl:text>
          </xsl:when>
          <xsl:when test="/etrecheck/timemachine/notconfigured = 'true'">
            <xsl:text>notconfigured</xsl:text>
          </xsl:when>
          <xsl:when test="sum(/etrecheck/timemachine/destinations//count) = 0">
            <xsl:text>notconfigured</xsl:text>
          </xsl:when>
          <xsl:when
            test="count(/etrecheck/timemachine//destination[lastbackupold = 'true']) > 0 and count(/etrecheck/timemachine//destination[lastbackupold = 'true']) = count(/etrecheck/timemachine//destination)">
            <xsl:text>old</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>configured</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">otherbackup</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when 
            test="count(/etrecheck//launchdfile[contains(path, 'bombich')]) &gt; 0">
            <xsl:text>1</xsl:text>
          </xsl:when>
          <xsl:when 
            test="count(/etrecheck//launchdfile[contains(path, 'shirtpocket')]) &gt; 0">
            <xsl:text>1</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>0</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>

  </xsl:template>

  <!-- Performance. -->
  <xsl:template name="performance">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">nominalio</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/performance/nominalio"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">filesystemtime</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/performance/filesystemtime"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">filesystemtimeout</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="/etrecheck/performance/filesystemtimeout = 'true'">
            <xsl:text>true</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>false</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
    
    <xsl:call-template name="emit">
      <xsl:with-param name="name">filesystemfailure</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="/etrecheck/performance/filesystemfailure = 'true'">
            <xsl:text>true</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>false</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">writespeed</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/performance/writespeed"/>
    </xsl:call-template>
    
    <xsl:call-template name="emit">
      <xsl:with-param name="name">writetimeout</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="/etrecheck/performance/writetimeout = 'true'">
            <xsl:text>true</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>false</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">readspeed</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/performance/readspeed"/>
    </xsl:call-template>
    
    <xsl:call-template name="emit">
      <xsl:with-param name="name">readtimeout</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="/etrecheck/performance/readtimeout = 'true'">
            <xsl:text>true</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>false</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>

  </xsl:template>

  <!-- CPU usage. -->
  <xsl:template name="cpuusage">
      
    <xsl:call-template name="emit">
      <xsl:with-param name="name">user</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/cpu/usage/user"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">system</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/cpu/usage/system"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">idle</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/cpu/usage/idle"/>
    </xsl:call-template>
        
  </xsl:template>

  <!-- Print CPU usage information. -->
  <xsl:template name="cpu">
  
    <!-- I don't care. -->
              
  </xsl:template>

  <!-- Print a memory usage information. -->
  <xsl:template name="memory">
  
    <!-- I don't care. -->
              
  </xsl:template>

  <!-- Print a network usage information. -->
  <xsl:template name="networkusage">
  
    <!-- I don't care. -->
        
  </xsl:template>

  <!-- Print a energy usage information. -->
  <xsl:template name="energy">
  
    <!-- I don't care. -->
        
  </xsl:template>

  <!-- Print a virtual memory usage information. -->
  <xsl:template name="vm">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">freeram</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/vm/freeram"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">usedram</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/vm/usedram"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">filecache</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/vm/filecache"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">availableram</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/vm/availableram"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">swapused</xsl:with-param>
      <xsl:with-param name="value" select="/etrecheck/vm/swapused"/>
    </xsl:call-template>

  </xsl:template>

  <!-- Print software install information. -->
  <xsl:template name="install">
  
    <!-- I don't care. -->
           
  </xsl:template>

  <!-- Print diagnostics information. -->
  <xsl:template name="diagnostics">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">kernelpanics</xsl:with-param>
      <xsl:with-param 
        name="value" 
        select="count(/etrecheck/diagnostics/event[type = 6 or type = 14])"/>
    </xsl:call-template>
    
    <xsl:call-template name="emit">
      <xsl:with-param name="name">bridgekernelpanics</xsl:with-param>
      <xsl:with-param 
        name="value" 
        select="count(/etrecheck/diagnostics/event[type = 12])"/>
    </xsl:call-template>

    <xsl:call-template name="emit">
      <xsl:with-param name="name">diagnostics</xsl:with-param>
      <xsl:with-param name="value">
        <xsl:choose>
          <xsl:when test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas' and /etrecheck/diagnostics/insufficientpermissions = 'true'">
            <xsl:text>insufficientpermissions</xsl:text>
          </xsl:when>
          <xsl:when test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
            <xsl:text>needsfulldriveaccess</xsl:text>
          </xsl:when>
          <xsl:when test="/etrecheck/diagnostics/insufficientpermissions = 'true'">
            <xsl:text>insufficientpermissions</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>ok</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:with-param>
    </xsl:call-template>
      
  </xsl:template>
  
  <xsl:template match="runtime[1]" mode="seconds">
  
    <xsl:call-template name="emit">
      <xsl:with-param name="name">seconds</xsl:with-param>
      <xsl:with-param name="value" select="."/>
    </xsl:call-template>
  
  </xsl:template>
  
  <xsl:template name="emit">
    <xsl:param name="name"/>
    <xsl:param name="value"/>
    
    <xsl:variable name="node" select="exslt:node-set($value)"/>
    
    <xsl:text>"</xsl:text>
    <xsl:choose>
      <xsl:when test="$name">
        <xsl:value-of select="$name"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="local-name($node)"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>"</xsl:text>
    
    <xsl:text>:</xsl:text>
    <xsl:text>"</xsl:text>
    <xsl:apply-templates select="$node"/>
    <xsl:text>"</xsl:text>
    <xsl:text>,</xsl:text>
    
  </xsl:template>
  
  <xsl:template match="*[@units]">
  
    <xsl:choose>
      <xsl:when test="@units = 'B'">
        <xsl:value-of select="number(.) div 1024 div 1024 div 1024"/>
      </xsl:when>
      <xsl:when test="@units = 'KB'">
        <xsl:value-of select="number(.) div 1024 div 1024"/>
      </xsl:when>
      <xsl:when test="@units = 'MB'">
        <xsl:value-of select="number(.) div 1024"/>
      </xsl:when>
      <xsl:when test="@units = 'GB'">
        <xsl:value-of select="number(.)"/>
      </xsl:when>
      <xsl:when test="@units = 'TB'">
        <xsl:value-of select="number(.) * 1024"/>
      </xsl:when>
      <xsl:when test="@units = 'PB'">
        <xsl:value-of select="number(.) * 1024 * 1024"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="."/>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>
  
</xsl:stylesheet>
