<?xml version="1.0"?>
<xsl:stylesheet
  version='1.0'
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  xmlns:math="http://exslt.org/math"
  extension-element-prefixes="exslt math">

  <!-- Convert sections of an EtreCheck report into HTML
       representations. -->
  <xsl:output method="html" indent="yes" encoding="UTF-8"/>
 
  <!-- Am I printing? -->
  <xsl:param name="printing">true</xsl:param>

  <!-- Set the language. -->
  <xsl:param name="language">en</xsl:param>

  <!-- Set the appearance. -->
  <xsl:param name="appearance">Aqua</xsl:param>

  <!-- Set the section. -->
  <xsl:param name="section"/>

  <!-- Power user. -->
  <xsl:param name="poweruser">false</xsl:param>
  
  <!-- Is the store available? -->
  <xsl:param name="storeavailable">true</xsl:param>
  
  <!-- Power user product. -->
  <xsl:param name="powerusertaxincluded">false</xsl:param>
  <xsl:param name="powerusernet">US$17.99</xsl:param>
  <xsl:param name="powerusertax">US$0.00</xsl:param>
  <xsl:param name="poweruserprice">US$17.99</xsl:param>

  <!-- Upgrade available. -->
  <xsl:param name="masupgrade">false</xsl:param>

  <!-- Set the solution code. -->
  <xsl:param name="solutioncode"/>

  <!-- Set the solution transaction. -->
  <xsl:param name="solutiontransaction"/>

  <!-- Set the solution transaction date. -->
  <xsl:param name="solutiontransactiondate"/>

  <!-- MAS or Pro? -->
  <xsl:param name="apptype"/>

  <!-- URL for user interface data. -->
  <xsl:param name="strings_url" select="concat('UI_', $language, '.xml')"/>
  
  <!-- URL for issues data. -->
  <xsl:param
    name="issues_url" select="concat('Issues_', $language, '.xml')"/>

  <!-- Serial MD5 of running program.. -->
  <xsl:param name="serial">NONE</xsl:param>

  <!-- Hide unsigned files. -->
  <xsl:param name="hideunsignedfiles">true</xsl:param>

  <!-- The Ventura preferences version. -->
  <xsl:param name="prefversion">pre13</xsl:param>
  
  <!-- Do I have full drive access? -->
  <xsl:variable
    name="fulldriveaccess" select="/etrecheck/header/fulldriveaccess"/>
  
  <!-- Issue cross references. -->
  <xsl:variable
    name="issuesxref"
    select="document('Issues.xml')/issues"/>

  <!-- Language-specific data. -->
  <xsl:variable name="strings" select="document($strings_url)/strings"/>
  
  <!-- Language-specific issues. -->
  <xsl:variable name="issues" select="document($issues_url)/issues"/>

  <!-- Get rid of these. -->
  <xsl:variable name="performance" select="$strings/performance_rating"/>

  <xsl:variable name="statusattributes" select="$strings/status"/>
  
  <!-- This is for the user interface. -->
  <xsl:template match="/etrecheck">
  
    <xsl:text disable-output-escaping='yes'>&lt;!DOCTYPE html&gt;
</xsl:text>
    <html>
      <xsl:attribute name="lang">
        <xsl:value-of select="$language"/>
      </xsl:attribute>
      
      <head>

        <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>

        <title><xsl:value-of select="$strings/etrecheck/title"/></title>

      </head>
      
      <body>
      
        <xsl:apply-templates select="header" mode="top"/>

        <xsl:call-template name="major"/>

        <xsl:call-template name="minor"/>

        <xsl:apply-templates select="hardware" mode="top"/>
        <xsl:apply-templates select="usb" mode="top"/>
        <xsl:apply-templates select="video" mode="top"/>

        <xsl:apply-templates select="systemsoftware" mode="top"/>
        <xsl:apply-templates select="configurationfiles" mode="top"/>
        <xsl:apply-templates select="configurationprofiles" mode="top"/>
        <xsl:call-template name="chromedefaults"/>
        <xsl:apply-templates select="notifications" mode="top"/>

        <xsl:apply-templates select="startupitems" mode="top"/>
        <xsl:apply-templates select="fonts" mode="top"/>
        <xsl:apply-templates select="vm" mode="top"/>
        <xsl:call-template name="diagnostics"/>

        <!-- Storage is special because it pulls from a number of
             different interfaces and sources and renames them. -->
        <xsl:apply-templates select="disk" mode="top"/>
        <xsl:apply-templates select="virtualvolume" mode="top"/>
        <xsl:apply-templates select="timemachine" mode="top"/>

        <xsl:apply-templates select="network" mode="top"/>
        <xsl:apply-templates select="networkusage" mode="top"/>

        <xsl:apply-templates select="security" mode="top"/>
        <xsl:apply-templates select="adware" mode="top"/>
        <xsl:apply-templates select="unsigned" mode="top"/>
        <xsl:apply-templates select="configurationprofiles" mode="top"/>
        <xsl:call-template name="chromedefaults"/>
        <xsl:apply-templates select="notifications" mode="top"/>

        <xsl:call-template name="performance"/>
        <xsl:call-template name="cpu"/>
        <xsl:apply-templates select="cpu" mode="top"/>
        <xsl:apply-templates select="memory" mode="top"/>
        
        <xsl:if test="/etrecheck/header/apptype = 'pro'">
          <xsl:apply-templates select="energy" mode="top"/>
        </xsl:if>
        
        <xsl:apply-templates select="systemextensions" mode="top"/>
        <xsl:apply-templates select="kernelextensions" mode="top"/>
        <xsl:apply-templates select="systemlaunchdaemons" mode="top"/>
        <xsl:apply-templates select="systemlaunchagents" mode="top"/>
        <xsl:apply-templates select="launchdaemons" mode="top"/>
        <xsl:apply-templates select="launchagents" mode="top"/>
        <xsl:apply-templates select="userlaunchagents" mode="top"/>
        <xsl:apply-templates select="loginitems" mode="top"/>
        <xsl:apply-templates select="applications" mode="top"/>
        <xsl:apply-templates select="modernplugins" mode="top"/>
        <xsl:apply-templates select="internetplugins" mode="top"/>
        <xsl:call-template name="userinternetplugins"/>
        <xsl:apply-templates select="audioplugins" mode="top"/>
        <xsl:apply-templates select="coremediaioplugins" mode="top"/>
        <xsl:call-template name="useraudioplugins"/>
        <xsl:call-template name="usercoremediaioplugins"/>
        <xsl:apply-templates select="itunesplugins" mode="top"/>
        <xsl:call-template name="useritunesplugins"/>
        <xsl:apply-templates select="safariextensions" mode="top"/>
        <xsl:apply-templates select="preferencepanes" mode="top"/>
        <xsl:apply-templates select="install" mode="top"/>
        <xsl:call-template name="cleanup"/>

      </body>

    </html>

  </xsl:template>

  <xsl:include href="modern.xsl" />

</xsl:stylesheet>
