<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet
  version='1.0'
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  xmlns:math="http://exslt.org/math"
  extension-element-prefixes="exslt math">

  <xsl:output method="text" indent="no" encoding="UTF-8"/>

  <!-- MAS or Pro? -->
  <xsl:param name="identifier">none</xsl:param>

  <!-- Newlines are special in XSL. -->
  <xsl:variable name="newline">
    <xsl:text>&#xa;</xsl:text>
  </xsl:variable>

  <!-- This is just for issues. -->
  <xsl:template match="/etrecheck">
  
    <xsl:for-each select=".//identifier[. = $identifier]/..">
      <xsl:if test="position() = 1">
        <xsl:call-template name="node">
          <xsl:with-param name="node" select="."/>
        </xsl:call-template>
      </xsl:if>
    </xsl:for-each>
    
  </xsl:template>

  <xsl:template name="node">
    <xsl:param name="node"/>
    <xsl:param name="indent">  </xsl:param>

    <xsl:if test="string-length($node/text()) &gt; 0">
      <xsl:value-of select="concat($indent, local-name($node))"/>
      <xsl:text>: </xsl:text>
      <xsl:value-of select="normalize-space($node/text())"/>
      <xsl:value-of select="$newline"/>
    </xsl:if>
    
    <xsl:for-each select="$node/*">
      <xsl:variable name="name" select="local-name()"/>
      
      <xsl:choose>
        <xsl:when test="$name = 'serial_unsafe'"/>
        <xsl:when test="$name = 'path_unsafe'"/>
        <xsl:when test="$name = 'app_path_unsafe'"/>
        <xsl:when test="$name = 'filename_unsafe'"/>
        <xsl:when test="$name = 'executable_unsafe'"/>
        <xsl:when test="$name = 'arguments_unsafe'"/>
        <xsl:when test="$name = 'name_unsafe'"/>
        <xsl:when test="$name = 'displayname_unsafe'"/>
        <xsl:when test="$name = 'mountpoint_unsafe'"/>
        <xsl:when test="$name = 'address_unsafe'"/>
        <xsl:when test="$name = 'user_unsafe'"/>
        <xsl:when test="$name = 'bundle_unsafe'"/>
        <xsl:when test="$name = 'location_unsafe'"/>
        <xsl:when test="$name = 'title_unsafe'"/>
        <xsl:when test="$name = 'body_unsafe'"/>
        <xsl:when test="$name = 'identifier'"/>
        <xsl:otherwise>
          <xsl:value-of select="concat($indent, '  ', $name)"/>
          <xsl:text>: </xsl:text>
          <xsl:value-of select="normalize-space(text())"/>
          <xsl:value-of select="$newline"/>
          <xsl:for-each select="*">
            <xsl:call-template name="node">
              <xsl:with-param name="node" select="."/>
              <xsl:with-param name="indent" select="concat($indent,'    ')"/>
            </xsl:call-template>
          </xsl:for-each>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    
  </xsl:template>
  
  <xsl:template match="@* | node()">
    <xsl:apply-templates select="@* | node()"/>
  </xsl:template>

</xsl:stylesheet>
