<?xml version="1.0"?>
<xsl:stylesheet
  version='1.0'
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:exslt="http://exslt.org/common"
  xmlns:math="http://exslt.org/math"
  extension-element-prefixes="exslt math">

  <!-- Header section. -->
  <xsl:template match="header" mode="top">
  
    <!-- These are not divided in the original XML. -->
    
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">header</xsl:with-param>
      <xsl:with-param name="contents">
        <xsl:apply-templates select="."/>
      </xsl:with-param>
    </xsl:call-template>
  
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">instructions</xsl:with-param>
      <xsl:with-param name="contents">
      
        <xsl:apply-templates
          select="$strings/etrecheck/instructions" mode="copy"/>

      </xsl:with-param>
    </xsl:call-template>

    <xsl:variable name="purchasesolution">
      <xsl:choose>
        <xsl:when test="/etrecheck/header/problem = 'justchecking'"/>
        
        <xsl:when test="$poweruser = 'true'">
          <xsl:call-template name="get_solution"/>
        </xsl:when>
        
        <xsl:otherwise>
          <xsl:call-template name="purchase_solution"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:if test="$printing = 'false'">
      <xsl:if test="string-length($purchasesolution) &gt; 0">
        <xsl:call-template name="section-wrapper">
          <xsl:with-param name="name">purchasesolution</xsl:with-param>
          <xsl:with-param name="contents">
            <xsl:copy-of select="$purchasesolution"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:if>
    </xsl:if>

  </xsl:template>

  <xsl:template name="purchase_solution">

    <div class="upgrade">
  
      <xsl:choose>
        <xsl:when test="$masupgrade = 'true'">
          <xsl:apply-templates
            select="$strings/upgrade/description_solution_upgrade"
            mode="copy"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates
            select="$strings/upgrade/description_solution" mode="copy"/>
        </xsl:otherwise>
      </xsl:choose>

      <xsl:call-template name="purchase_solution_table"/>
      
    </div>
  
  </xsl:template>
  
  <xsl:template name="purchase_solution_table">

    <table class="tablelist">
      <tr>
        <td>
          <strong>
            <xsl:value-of select="$strings/upgrade/poweruser_name"/>
          </strong>
          <br/>
          <xsl:value-of select="$strings/upgrade/poweruser_description"/>
          <xsl:if test="$masupgrade = 'true' and $storeavailable = 'false'">
            <br/>
            <div class="fail">
              <xsl:copy-of select="$strings/store/notavailable/node()"/>
            </div>
          </xsl:if>
        </td>
        <td/>
        <td class="actioncolumn">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/help/poweruserinapppurchase')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/upgrade/help_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/upgrade/help"/>
          </button>
          <xsl:if test="$masupgrade = 'true'">
            <button>
              <xsl:choose>
                <xsl:when test="$storeavailable = 'true'">
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/buy/masupgrade')</xsl:text>
                  </xsl:attribute>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:attribute name="disabled">
                    <xsl:text>disabled</xsl:text>
                  </xsl:attribute>
                </xsl:otherwise>
              </xsl:choose>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/upgrade/upgrade_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/upgrade/upgrade"/>
            </button>
          </xsl:if>
        </td>
      </tr>
      <xsl:if test="$masupgrade = 'false'">
        <tr class="horizontalborder">
          <td>
            <strong>
              <xsl:value-of select="$strings/upgrade/price"/>
            </strong>
          </td>
          <td class="pricecolumn">
            <xsl:value-of select="$poweruserprice"/>
            <xsl:value-of select="$strings/upgrade/plustax"/>
          </td>
          <td class="actioncolumn">
            <button>
              <xsl:choose>
                <xsl:when test="$storeavailable = 'true'">
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/buy/poweruser')</xsl:text>
                  </xsl:attribute>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:attribute name="disabled">
                    <xsl:text>disabled</xsl:text>
                  </xsl:attribute>
                </xsl:otherwise>
              </xsl:choose>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/upgrade/buynow_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/upgrade/buynow"/>
            </button>
          </td>
        </tr>
        <tr>
          <td colspan="3">
            <xsl:choose>
              <xsl:when test="$powerusertaxincluded = 'true'">
                <em>
                  <xsl:value-of select="$strings/upgrade/taxesincluded"/>
                </em>
              </xsl:when>
              <xsl:when test="$powerusertaxincluded = 'false'">
                <em>
                  <xsl:value-of select="$strings/upgrade/taxesextra"/>
                </em>
              </xsl:when>
              <xsl:otherwise/>
            </xsl:choose>
          </td>
        </tr>
        <xsl:if test="$storeavailable != 'true'">
          <tr>
            <td colspan="3">
              <div class="fail">
                <xsl:copy-of select="$strings/store/notavailable/node()"/>
              </div>
            </td>
          </tr>
        </xsl:if>
      </xsl:if>
    </table>

    <p class="buynow">
      <button>
        <xsl:attribute name="onclick">
          <xsl:text>GET('/buy/contact')</xsl:text>
        </xsl:attribute>
        <xsl:attribute name="title">
          <xsl:value-of select="$strings/upgrade/contact_title"/>
        </xsl:attribute>
        <xsl:value-of select="$strings/upgrade/contact"/>
      </button>
    </p>

    <xsl:choose>
      <xsl:when test="$masupgrade = 'true'"/>
      <xsl:otherwise>
        <xsl:copy-of
          select="$strings/upgrade/privacy_description_pro/node()"/>
      </xsl:otherwise>
    </xsl:choose>
      
  </xsl:template>

  <xsl:template name="get_solution">

    <div class="upgrade">
  
      <xsl:apply-templates
        select="$strings/upgrade/description_get_solution" mode="copy"/>
      
      <p class="buynow">
        <button>
          <xsl:attribute name="onclick">
            <xsl:text>GET('/get/solution')</xsl:text>
          </xsl:attribute>
          <xsl:attribute name="title">
            <xsl:value-of select="$strings/upgrade/get_solution_title"/>
          </xsl:attribute>
          <xsl:value-of select="$strings/upgrade/get_solution"/>
        </button>
      </p>
    </div>
  
  </xsl:template>
  
  <xsl:include href="Issues.xsl" />

  <!-- Major issues. -->
  <xsl:template name="major">
  
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">major</xsl:with-param>
      <xsl:with-param name="contents">
      
        <xsl:variable name="content">
          <xsl:call-template name="majorissues"/>
        </xsl:variable>

        <xsl:choose>
        
          <xsl:when test="string-length($content) &gt; 0">
          
            <div class="upgrade">
              <p>
                <xsl:apply-templates
                  select="$issues/major/description" mode="copy"/>
                
                <xsl:apply-templates
                  select="$issues/major/problemsfound" mode="copy"/>
                
                <xsl:apply-templates
                  select="$issues/major/problemsfound_fix" mode="copy"/>
              </p>
            </div>
            
            <table class="tablelist">
              <xsl:copy-of select="$content"/>
            </table>

            <xsl:if test="$printing = 'false'">
              <xsl:choose>
                <xsl:when
                  test="/etrecheck/header/problem = 'justchecking'"/>
                  
                <xsl:when test="$poweruser = 'true'">
                  <h1>
                    <xsl:value-of
                      select="$strings/upgrade/major_issues_solutions"/>
                  </h1>
                  <xsl:call-template name="get_issues_solution"/>
                </xsl:when>
                
                <xsl:otherwise>
                  <h1>
                    <xsl:value-of
                      select="$strings/upgrade/major_issues_solutions"/>
                  </h1>
                  <xsl:call-template name="purchase_issues_solution"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:if>
            
          </xsl:when>
          <xsl:otherwise>
            <table class="tablelist">
              <xsl:call-template name="nomajorissues"/>
            </table>
          </xsl:otherwise>
        </xsl:choose>
        
      </xsl:with-param>
    </xsl:call-template>
    
  </xsl:template>
  
  <!-- No Major issues. -->
  <xsl:template name="nomajorissues">
  
    <tr class="horizontalborder">
      <td class="noissues">
      
        <xsl:choose>
          
          <xsl:when
            test="/etrecheck/header/problem = 'justchecking'">
            <xsl:apply-templates
              select="$issues/major/noproblem_justchecking" mode="copy"/>
          </xsl:when>
          
          <xsl:otherwise>
          
            <xsl:apply-templates
              select="$issues/major/noproblem" mode="copy"/>
          
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>jump("report")</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$issues/major/review_title"/>
              </xsl:attribute>
              <xsl:value-of select="$issues/major/review"/>
            </button>
          
          </xsl:otherwise>
          
        </xsl:choose>
        
      </td>
    </tr>

  </xsl:template>
  
  <!-- Minor issues. -->
  <xsl:template name="minor">
  
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">minor</xsl:with-param>
      <xsl:with-param name="contents">
      
        <xsl:variable name="content">
          <xsl:call-template name="minorissues"/>
        </xsl:variable>

        <xsl:choose>
          <xsl:when test="string-length($content) &gt; 0">
          
            <div class="upgrade">
              <p>
                <xsl:apply-templates
                  select="$issues/minor/description" mode="copy"/>
      
                <xsl:apply-templates
                  select="$issues/minor/problemsfound" mode="copy"/>
                
                <xsl:apply-templates
                  select="$issues/minor/problemsfound_fix" mode="copy"/>
              </p>
            </div>
            
            <table class="tablelist">
              <xsl:copy-of select="$content"/>
            </table>

            <xsl:if test="$printing = 'false'">
              <xsl:choose>
                <xsl:when
                  test="/etrecheck/header/problem = 'justchecking'"/>
                  
                <xsl:when test="$poweruser = 'true'">
                  <h1>
                    <xsl:value-of
                      select="$strings/upgrade/minor_issues_solutions"/>
                  </h1>
                  <xsl:call-template name="get_issues_solution"/>
                </xsl:when>
                
                <xsl:otherwise>
                  <h1>
                    <xsl:value-of
                      select="$strings/upgrade/minor_issues_solutions"/>
                  </h1>
                  <xsl:call-template name="purchase_issues_solution"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:if>

          </xsl:when>
          <xsl:otherwise>
            <table class="tablelist">
              <xsl:call-template name="nominorissues"/>
            </table>
          </xsl:otherwise>
        </xsl:choose>

      </xsl:with-param>
    </xsl:call-template>

  </xsl:template>

  <!-- No Minor issues. -->
  <xsl:template name="nominorissues">
  
    <tr class="horizontalborder">
      <td class="noissues">
        <xsl:choose>
        
          <xsl:when
            test="/etrecheck/header/problem = 'justchecking'">
            <xsl:apply-templates
              select="$issues/minor/noproblem_justchecking" mode="copy"/>
          </xsl:when>
          
          <xsl:otherwise>
          
            <xsl:apply-templates
              select="$issues/minor/noproblem" mode="copy"/>
        
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>jump("report")</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$issues/minor/review_title"/>
              </xsl:attribute>
              <xsl:value-of select="$issues/minor/review"/>
            </button>
          
          </xsl:otherwise>
          
        </xsl:choose>
      </td>
    </tr>

  </xsl:template>
  
  <!-- Emit a purchase opportunity. -->
  <xsl:template name="purchase_issues_solution">
  
    <div class="upgrade">
  
      <xsl:choose>
        <xsl:when test="$masupgrade = 'true'">
          <xsl:apply-templates
            select="$strings/upgrade/issues_solution_upgrade" mode="copy"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates
            select="$strings/upgrade/issues_solution" mode="copy"/>
        </xsl:otherwise>
      </xsl:choose>

      <xsl:call-template name="purchase_solution_table"/>

    </div>

  </xsl:template>
  
  <!-- Just give it away. -->
  <xsl:template name="get_issues_solution">
  
    <div class="upgrade">
  
      <xsl:apply-templates
        select="$strings/upgrade/issues_get_solution" mode="copy"/>
      
      <p class="buynow">
        <button>
          <xsl:attribute name="onclick">
            <xsl:text>GET('/get/solution')</xsl:text>
          </xsl:attribute>
          <xsl:attribute name="title">
            <xsl:value-of select="$strings/upgrade/get_solution_title"/>
          </xsl:attribute>
          <xsl:value-of select="$strings/upgrade/get_solution"/>
        </button>
      </p>
    </div>

  </xsl:template>
  
  <!-- Emit an issue. -->
  <xsl:template name="issue">
    <xsl:param name="type"/>
    <xsl:param name="name"/>
    
    <xsl:variable
      name="issuenodes" select="$issues/*[local-name() = $type]"/>
    
    <xsl:variable
      name="issuexref"
      select="$issuesxref/*[local-name() = $type]/issue[@id = $name]"/>

    <xsl:variable
      name="issue" select="$issuenodes/*[@id = $name]"/>
    
    <tr class="horizontalborder">
      <td class="header">
        <xsl:value-of select="$issue/title"/>
      </td>
      <td>
        <xsl:value-of select="$issue/description"/>
      </td>
      <td>
        <button>
          <xsl:attribute name="onclick">
            <xsl:value-of
              select="concat('jump(&quot;',$issuexref/section,'&quot;,&quot;',$issuexref/issue,'&quot;)')"/>
          </xsl:attribute>
          <xsl:attribute name="title">
            <xsl:value-of select="$issuenodes/review_title"/>
          </xsl:attribute>
          <xsl:value-of select="$issuenodes/review"/>
        </button>
      </td>
    </tr>
    
  </xsl:template>
  
  <!-- A menu for the hardware page. -->
  <xsl:template name="hardwaremenu">
  
    <xsl:variable name="machineid">
      <xsl:text>machine</xsl:text>
    </xsl:variable>
    
    <xsl:variable name="processorid">
      <xsl:text>processor</xsl:text>
    </xsl:variable>

    <xsl:variable name="ramid">
      <xsl:text>ram</xsl:text>
    </xsl:variable>

    <xsl:variable name="batteryid">
      <xsl:choose>
        <xsl:when test="/etrecheck/hardware/batteryhealth">
          <xsl:text>battery</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>omit</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="usbid">
      <xsl:choose>
        <xsl:when test="count(/etrecheck/usb/node/node) &gt; 0">
          <xsl:text>usb</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>omit</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="videoid">
      <xsl:text>video</xsl:text>
    </xsl:variable>

    <div class="menu">
      <table class="menu">
        <tr>
          <td>
            <a href="#">
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('menuJumpToId(&quot;',$machineid,'&quot;)')"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/hardwaremenu/machine"/>
            </a>
          </td>
          <td>
            <a href="#">
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('menuJumpToId(&quot;',$processorid,'&quot;)')"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/hardwaremenu/processor"/>
            </a>
          </td>
          <td>
            <a href="#">
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('menuJumpToId(&quot;',$ramid,'&quot;)')"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/hardwaremenu/ram"/>
            </a>
          </td>
          <xsl:if test="$batteryid != 'omit'">
            <td>
              <a href="#">
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('menuJumpToId(&quot;',$batteryid,'&quot;)')"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/hardwaremenu/battery"/>
              </a>
            </td>
          </xsl:if>
          <xsl:if test="$usbid != 'omit'">
            <td>
              <a href="#">
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('menuJumpToId(&quot;',$usbid,'&quot;)')"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/hardwaremenu/usb"/>
              </a>
            </td>
          </xsl:if>
          <td>
            <a href="#">
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('menuJumpToId(&quot;',$videoid,'&quot;)')"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/hardwaremenu/video"/>
            </a>
          </td>
        </tr>
      </table>
    </div>
    
    <div class="menuspacer"/>
    
  </xsl:template>

  <!-- Hardware information. -->
  <xsl:template match="hardware" mode="top">
  
    <!-- These are not divided in the original XML. -->
    
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">machine</xsl:with-param>
      <xsl:with-param name="contents">
        <xsl:call-template name="machine"/>
      </xsl:with-param>
    </xsl:call-template>
    
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">processor</xsl:with-param>
      <xsl:with-param name="contents">
        <xsl:call-template name="processor"/>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">ram</xsl:with-param>
      <xsl:with-param name="contents">
        <xsl:call-template name="ram"/>
      </xsl:with-param>
    </xsl:call-template>

    <xsl:if test="batteryhealth and batterycyclecount and batterypercent">
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">battery</xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:call-template name="battery"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    
  </xsl:template>
 
  <!-- Machine information. -->
  <xsl:template name="machine">
  
    <xsl:variable name="warrantyandserviceurl">
      <xsl:call-template name="format">
        <xsl:with-param
          name="format"
          select="concat('https://support.apple.com/',$language,'-ca/',warrantyandserviceurl)"/>
        <xsl:with-param name="value">
          <xsl:value-of select="$language"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="hardwaresupporturl">
      <xsl:call-template name="format">
        <xsl:with-param
          name="format"
          select="concat('https://support.apple.com/',$language,'-ca/',hardwaresupporturl)"/>
        <xsl:with-param name="value">
          <xsl:value-of select="$language"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="skip">
      <row/>
      <row/>
      <row/>
      <row/>
      <row/>
      <xsl:if test="marketingname[@lang=$language]">
        <row/>
      </xsl:if>
      <xsl:if test="serial_unsafe and serial != serial_unsafe">
        <row/>
      </xsl:if>
      <xsl:if test="coveredbywarranty">
        <row/>
        <row/>
      </xsl:if>
    </xsl:variable>
    
    <table>
      <tr>
        <td>
          <xsl:attribute name="rowspan">
            <xsl:value-of select="count(exslt:node-set($skip)/row)"/>
          </xsl:attribute>
          
          <img src="image/computer.png">
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/machine/icon"/>
            </xsl:attribute>
          </img>
        </td>
        
        <!-- Add some blank space to accomodate the header. -->
        <td>
          <xsl:text>&#160;</xsl:text>
          <br/>
          <xsl:text>&#160;</xsl:text>
          <br/>
          <xsl:text>&#160;</xsl:text>
          <br/>
        </td>
        <td>
          <xsl:text>&#160;</xsl:text>
          <br/>
          <xsl:text>&#160;</xsl:text>
          <br/>
          <xsl:text>&#160;</xsl:text>
          <br/>
        </td>
        <td>
          <xsl:text>&#160;</xsl:text>
          <br/>
          <xsl:text>&#160;</xsl:text>
          <br/>
          <xsl:text>&#160;</xsl:text>
          <br/>
        </td>
      </tr>
      <xsl:if test="marketingname[@lang=$language]">
        <tr>
          <th><xsl:value-of select="$strings/machine/name"/></th>
          <td colspan="2"><xsl:value-of select="marketingname[@lang=$language]"/></td>
        </tr>
      </xsl:if>
      <tr>
        <th><xsl:value-of select="$strings/machine/model"/></th>
        <td>
          <xsl:value-of select="model"/>
        </td>
        <td class="identify">
          <xsl:call-template name="identify">
            <xsl:with-param name="modeltype" select="modeltype"/>
          </xsl:call-template>
        </td>
      </tr>
      
      <xsl:if test="serial_unsafe and serial != serial_unsafe">
        <tr>
          <th><xsl:value-of select="$strings/machine/serial"/></th>
          <td colspan="2"><xsl:value-of select="serial_unsafe"/></td>
        </tr>
      </xsl:if>
      
      <tr>
        <th><xsl:value-of select="$strings/machine/status"/></th>
        <xsl:choose>
          <xsl:when test="vintage = 'vintage'">
            <td>
              <xsl:copy-of select="$strings/machine/vintage/node()"/>
            </td>
            <td class="vintage">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET("/link/https://support.apple.com/HT201624");</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/machine/vintage_button_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/machine/vintage_button"/>
                <img class="buttonicon" src="image/webbrowserbutton.png"/>
              </button>
            </td>
          </xsl:when>
          
          <xsl:when test="vintage = 'obsolete'">
            <td>
              <xsl:copy-of select="$strings/machine/obsolete/node()"/>
            </td>
            <td class="vintage">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET("/link/https://support.apple.com/HT201624");</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/machine/obsolete_button_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/machine/obsolete_button"/>
                <img class="buttonicon" src="image/webbrowserbutton.png"/>
              </button>
            </td>
          </xsl:when>

          <xsl:otherwise>
            <td colspan="2">
              <xsl:copy-of select="$strings/machine/supported/node()"/>
            </td>
          </xsl:otherwise>
          
        </xsl:choose>
      </tr>

      <xsl:if test="coveredbywarranty">
        <tr>
          <th><xsl:value-of select="$strings/machine/coveredbywarranty"/></th>
          <td colspan="2"><xsl:apply-templates select="coveredbywarranty"/></td>
        </tr>
        <tr>
          <th><xsl:value-of select="$strings/machine/warrantyexpiry"/></th>
          <td colspan="2"><xsl:apply-templates select="warrantyexpiry"/></td>
        </tr>
      </xsl:if>

      <tr>
        <td colspan="3">
          <fieldset class="appleresources">
            <legend>
              <xsl:value-of select="$strings/machine/support_resources"/>
            </legend>
            <xsl:if test="technicalspecificationsurl">
              <button class="test">
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('GET(&quot;/techspeclink/',technicalspecificationsurl,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/machine/technical_specifications_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/machine/technical_specifications"/>
                <img class="buttonicon" src="image/webbrowserbutton.png"/>
              </button>
              <br/>
            </xsl:if>
            <xsl:if test="userguideurl">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('GET(&quot;/link/',userguideurl,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/machine/user_guide_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/machine/user_guide"/>
                <img class="buttonicon" src="image/webbrowserbutton.png"/>
              </button>
              <br/>
            </xsl:if>
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('GET(&quot;/link/',$warrantyandserviceurl,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of
                  select="$strings/machine/warranty_service_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/machine/warranty_service"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
            <br/>
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('GET(&quot;/link/',$hardwaresupporturl,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of
                  select="$strings/machine/hardware_support_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/machine/hardware_support"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </fieldset>
        </td>
      </tr>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/machine/description/node()"/>
      
      <xsl:if test="vintage = 'vintage' or vintage = 'obsolete'">
        <xsl:copy-of select="$strings/machine/description_vintage/node()"/>
      </xsl:if>
    </p>
    
  </xsl:template>

  <!-- Identification button. -->
  <xsl:template name="identify">
    <xsl:param name="modeltype"/>
    
    <xsl:variable name="url">
      <xsl:choose>
        <xsl:when test="$modeltype = 'MacPro'">
          <xsl:text>https://support.apple.com/HT202888</xsl:text>
        </xsl:when>
        <xsl:when test="$modeltype = 'Macmini'">
          <xsl:text>https://support.apple.com/HT201894</xsl:text>
        </xsl:when>
        <xsl:when test="$modeltype = 'iMac'">
          <xsl:text>https://support.apple.com/HT201634</xsl:text>
        </xsl:when>
        <xsl:when test="$modeltype = 'MacBookPro'">
          <xsl:text>https://support.apple.com/HT201300</xsl:text>
        </xsl:when>
        <xsl:when test="$modeltype = 'MacBook'">
          <xsl:text>https://support.apple.com/HT201608</xsl:text>
        </xsl:when>
        <xsl:when test="$modeltype = 'MacBookAir'">
          <xsl:text>https://support.apple.com/HT201862</xsl:text>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:if test="string-length($url) &gt; 0">
      <button>
        <xsl:attribute name="onclick">
          <xsl:value-of
            select="concat('GET(&quot;/link/',$url,'&quot;)')"/>
        </xsl:attribute>
        <xsl:attribute name="title">
          <xsl:value-of select="$strings/machine/identify_title"/>
        </xsl:attribute>
        <xsl:value-of select="$strings/machine/identify"/>
        <img class="buttonicon" src="image/webbrowserbutton.png"/>
      </button>
    </xsl:if>
    
  </xsl:template>
  
  <!-- Processor information. -->
  <xsl:template name="processor">

    <table>
      <tr>
        <th><xsl:value-of select="$strings/processor/count"/></th>
        <td><xsl:value-of select="cpucount"/></td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/processor/cores"/></th>
        <td><xsl:value-of select="corecount"/></td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/processor/model"/></th>
        <td>
          <xsl:apply-templates select="speed" mode="units"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="cpu_type"/>
        </td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/processor/code"/></th>
        
        <td>
          <xsl:value-of select="cpucode"/>
          <xsl:if test="wikichip">
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('GET(&quot;/link/https://en.wikichip.org/w/index.php?title=',wikichip, '&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/processor/wikichip_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/processor/wikichip"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </xsl:if>
        </td>
      </tr>
    </table>

    <p class="description">
      <xsl:copy-of select="$strings/processor/description/node()"/>

      <xsl:if test="wikichip">
        <xsl:copy-of select="$strings/processor/description_wikichip/node()"/>
      </xsl:if>
    </p>
    
  </xsl:template>
  
  <!-- RAM information. -->
  <xsl:template name="ram">

    <xsl:variable
      name="failingram"
      select="count(memorybanks/memorybank[status != 'ok']) > 0"/>
      
    <table>
      <tr>
        <th><xsl:value-of select="$strings/ram/total"/></th>
        <td>
          <xsl:apply-templates select="ram" mode="units"/>
      
          <div class="appleresources">
            <p>
              <xsl:choose>
              
                <!-- If memory is upgradeable, say so and provide a link
                     to instructions from Apple. -->
                <xsl:when
                  test="upgradeable = 'true' and applemaximummemory and ram &gt;= applemaximummemory">
                  <strong>
                    <xsl:value-of select="$strings/ram/maxed"/>
                  </strong>
                </xsl:when>
                
                <xsl:when test="upgradeable = 'true'">
                  
                  <strong>
                    <xsl:choose>
                      <!-- If memory is upgradeable, but on by Apple, say show
                           and provide a link to instructions from Apple. -->
                      <xsl:when test="userupgradeable = 'false'">
                        <xsl:value-of select="$strings/ram/upgradeablebyapple"/>
                      </xsl:when>
                      <xsl:otherwise>
                        <xsl:value-of select="$strings/ram/upgradeable"/>
                      </xsl:otherwise>
                    </xsl:choose>
                  </strong>

                  <button>
                    <xsl:attribute name="onclick">
                      <xsl:value-of
                        select="concat('GET(&quot;/link/',memoryupgradeinstructionsurl,'&quot;)')"/>
                    </xsl:attribute>
                    <xsl:attribute name="title">
                      <xsl:value-of select="$strings/ram/upgradeable_instructions_title"/>
                    </xsl:attribute>
                    <xsl:value-of select="$strings/ram/upgradeable_instructions"/>
                    <img class="buttonicon" src="image/webbrowserbutton.png"/>
                  </button>

                </xsl:when>
                
                <!-- Better hope you bought enough RAM! -->
                <xsl:otherwise>
                  <strong>
                    <xsl:value-of select="$strings/ram/not_upgradeable"/>
                  </strong>
                </xsl:otherwise>
                
              </xsl:choose>

            </p>
          </div>
          
        </td>
      </tr>
      <xsl:if test="count(memorybanks/memorybank) &gt; 0">
        <tr>
          <th colspan="2">
            <xsl:value-of select="$strings/ram/memorybanks"/>
          </th>
        </tr>
        <tr>
          <td colspan="2">
            <ul>
              <xsl:for-each select="memorybanks/memorybank">
              
                <!-- Add a list item for each DIMM. -->
                <li>

                  <div class="DIMMWrapper">

                    <!-- Make it kinda look like a DIMM. -->
                    <img src="image/DIMM.png">
                      <xsl:attribute name="alt">
                        <xsl:value-of select="$strings/ram/dimm_image"/>
                      </xsl:attribute>
                    </img>

                    <!-- Put the specs on top of the image. -->
                    <div class="dimm">
                      <p>
                        <xsl:value-of select="identifier"/>
                      </p>
                      <p>
                        <xsl:if
                          test="string-length(concat(size, type, speed)) = 0">
                          <xsl:value-of select="$strings/ram/emptyslot"/>
                        </xsl:if>
                        
                        <xsl:apply-templates select="size" mode="units"/>
                        <xsl:text> </xsl:text>
                        <xsl:value-of select="type"/>
                        <xsl:text> </xsl:text>
                        <xsl:apply-templates select="speed" mode="units"/>
                        <xsl:if test="$failingram">
                          <xsl:text> </xsl:text>
                          <xsl:value-of select="status"/>
                        </xsl:if>
                      </p>
                    </div>
                  </div>
                </li>
              </xsl:for-each>
            </ul>
          </td>
        </tr>
      </xsl:if>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/ram/description/node()"/>
      
      <xsl:choose>
      
        <xsl:when
          test="upgradeable = 'true' and applemaximummemory and ram &gt;= applemaximummemory">
          <xsl:copy-of select="$strings/ram/description_upgraded/node()"/>
        </xsl:when>
        
        <xsl:when test="upgradeable = 'true'">
          <xsl:copy-of
            select="$strings/ram/description_upgradeable/node()"/>
          <xsl:if test="userupgradeable = 'false'">
            <xsl:copy-of
              select="$strings/ram/description_upgradeablebyapple/node()"/>
          </xsl:if>
        </xsl:when>
        
        <xsl:otherwise/>
        
      </xsl:choose>
      
      <xsl:if test="count(memorybanks/memorybank) &gt; 0">
        <xsl:copy-of select="$strings/ram/description_memorybanks/node()"/>
      </xsl:if>

    </p>
    
  </xsl:template>
  
  <!-- Battery information. -->
  <xsl:template name="battery">

    <xsl:variable name="batteryhealth" select="$strings/batteryhealth"/>

    <xsl:variable name="healthvalue" select="batteryhealth"/>
    
    <table>
      <tr>
        <th><xsl:value-of select="$strings/battery/health"/></th>
        <td>
          <xsl:value-of select="$batteryhealth/value[@key = $healthvalue]"/>
          
          <!-- Provide a button to Apple resources about battery health. -->
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/link/https://support.apple.com/HT204054')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/battery/more_information_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/battery/more_information"/>
            <img class="buttonicon" src="image/webbrowserbutton.png"/>
          </button>
        </td>
      </tr>
      <tr>
        <th><xsl:value-of select="$strings/battery/cycle_count"/></th>
        <td>
          <xsl:value-of select="batterycyclecount"/>

          <div class="meter">
            <xsl:call-template name="meter">
              <xsl:with-param name="percentage" select="batterypercent"/>
              <xsl:with-param
                name="units" select="$strings/battery/percent_used"/>
            </xsl:call-template>
          </div>
          
          <!-- Provide a button to Apple resources about battery cycles. -->
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/link/https://support.apple.com/HT201585')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/battery/about_cycles_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/battery/about_cycles"/>
            <img class="buttonicon" src="image/webbrowserbutton.png"/>
          </button>
        </td>
      </tr>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/battery/description/node()"/>
    </p>

  </xsl:template>
  
  <!-- Video information. -->
  <xsl:template match="video">
  
    <!-- Make a little table and show all GPUs and displays associated with
         each GPU. -->
    <table class="tablelist">
      <tr>
        <!-- Need to manually add a vertical border. -->
        <th class="verticalborder">
          <xsl:call-template name="plural">
            <xsl:with-param name="keys" select="$strings/video"/>
            <xsl:with-param name="key">
              <xsl:text>gpu</xsl:text>
            </xsl:with-param>
            <xsl:with-param name="value" select="exslt:node-set(count(gpu))"/>
          </xsl:call-template>
        </th>
        <th><xsl:value-of select="$strings/video/displays"/></th>
      </tr>
      
      <xsl:for-each select="gpu[bus = 'spdisplays_builtin']">
      
        <xsl:call-template name="gpu">
          <xsl:with-param name="show_integrated_button">
            <xsl:choose>
              <xsl:when test="/etrecheck/hardware/cpucode = 'a12z'">
                <xsl:value-of select="false()"/>
              </xsl:when>
              <xsl:when test="/etrecheck/hardware/cpucode = 'm1'">
                <xsl:value-of select="false()"/>
              </xsl:when>
              <xsl:when test="/etrecheck/hardware/cpucode = 'm2'">
                <xsl:value-of select="false()"/>
              </xsl:when>
              <xsl:when test="position() = 1">
                <xsl:value-of select="true()"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="false()"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:with-param>
        </xsl:call-template>
        
      </xsl:for-each>

      <xsl:for-each select="gpu[bus != 'spdisplays_builtin']">

        <xsl:call-template name="gpu">
          <xsl:with-param name="show_discrete_button">
            <xsl:choose>
              <xsl:when test="/etrecheck/hardware/cpucode = 'a12z'">
                <xsl:value-of select="false()"/>
              </xsl:when>
              <xsl:when test="/etrecheck/hardware/cpucode = 'm1'">
                <xsl:value-of select="false()"/>
              </xsl:when>
              <xsl:when test="/etrecheck/hardware/cpucode = 'm2'">
                <xsl:value-of select="false()"/>
              </xsl:when>
              <xsl:when test="/etrecheck/hardware/modeltype = 'MacBookPro' and position() = 1">
                <xsl:value-of select="true()"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="false()"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:with-param>
        </xsl:call-template>

      </xsl:for-each>

      <xsl:for-each select="gpu[not(bus)]">

        <xsl:call-template name="gpu">
          <xsl:with-param name="show_discrete_button" select="false()"/>
          <xsl:with-param name="show_integrated_button" select="false()"/>
        </xsl:call-template>

      </xsl:for-each>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/video/description/node()"/>
      
      <xsl:if test="count(gpu) &gt; 1">
        <xsl:copy-of select="$strings/video/description_multiple/node()"/>
      </xsl:if>
      
      <xsl:choose>
        <xsl:when test="/etrecheck/hardware/cpucode = 'a12z'"/>
        <xsl:when test="/etrecheck/hardware/cpucode = 'm1'"/>
        <xsl:when test="/etrecheck/hardware/cpucode = 'm2'"/>
        <xsl:when test="count(gpu[bus = 'spdisplays_builtin']) &gt; 0">
          <xsl:copy-of select="$strings/video/description_integrated/node()"/>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
    </p>

  </xsl:template>

  <xsl:template name="gpu">
    <xsl:param name="show_integrated_button"/>
    <xsl:param name="show_discrete_button"/>
    
    <!-- Need to manually add borders. -->
    <tr class="horizontalborder">
      
      <td class="verticalborder">
        <strong>
          <xsl:choose>
            <xsl:when test="name">
              <xsl:value-of select="name"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>GPU</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </strong>
        <br/>
        <xsl:choose>
          <xsl:when test="bus = 'spdisplays_builtin' and $show_integrated_button = 'true'">
            <xsl:value-of select="$strings/video/integrated"/>
            <br/>
          </xsl:when>
          <xsl:when test="$show_discrete_button = 'true'">
            <xsl:value-of select="$strings/video/discrete"/>
            <br/>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>
        <xsl:if test="vram">
          <xsl:apply-templates select="vram" mode="units"/>
          <xsl:text> VRAM</xsl:text>
        </xsl:if>
        <xsl:choose>
          <xsl:when test="$show_integrated_button = 'true'">
            <br/>
            
            <!-- Show a button to get more information from Apple about
                 integrated video. -->
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/link/https://support.apple.com/HT204349')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/video/about_integrated_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/video/about_integrated"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </xsl:when>
          <xsl:when test="$show_discrete_button = 'true'">
            <br/>
            
            <!-- Show a button to get more information from Apple about
                 discrete video. -->
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/link/https://support.apple.com/HT202053')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/video/about_discrete_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/video/about_discrete"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>
      </td>
      <td>
        <ul>
      
          <!-- Report displays, if any. -->
          <xsl:choose>
          
            <xsl:when test="count(displays/display) &gt; 0">
              
              <xsl:for-each select="displays/display">
                <li>
                  <xsl:choose>
                    <xsl:when test="builtin = 'true'">
                      <img src="image/computer.png">
                        <xsl:attribute name="alt">
                          <xsl:value-of
                            select="$strings/video/display_icon"/>
                        </xsl:attribute>
                      </img>
                    </xsl:when>
                    <xsl:otherwise>
                      <img>
                        <xsl:attribute name="src">
                          <xsl:value-of
                            select="concat('image/display/', name)"/>
                        </xsl:attribute>
                        <xsl:attribute name="alt">
                          <xsl:value-of
                            select="$strings/video/display_icon"/>
                        </xsl:attribute>
                      </img>
                    </xsl:otherwise>
                  </xsl:choose>
                  
                  <p>
                    <xsl:choose>
                      <xsl:when test="name = 'spdisplays_display'">
                        <xsl:value-of select="$strings/video/display"/>
                      </xsl:when>
                      <xsl:otherwise>
                        <xsl:value-of select="name"/>
                      </xsl:otherwise>
                    </xsl:choose>
                    
                    <xsl:if test="builtin = 'true'">
                      <xsl:text> </xsl:text>
                      <xsl:value-of select="$strings/video/builtin"/>
                    </xsl:if>
                  </p>
                  <p>
                    <xsl:value-of select="resolution"/>
                  </p>
                </li>
              </xsl:for-each>

            </xsl:when>
            
            <xsl:otherwise>
              <li>
                <p>
                  <xsl:value-of select="$strings/video/no_displays"/>
                </p>
              </li>
            </xsl:otherwise>
            
          </xsl:choose>
        </ul>
      </td>
    </tr>

  </xsl:template>
  
  <!-- System software. -->
  <xsl:template match="systemsoftware">
  
    <table>
      <tr>
        <td>
          <img src="image/os.png">
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/system_software/icon"/>
            </xsl:attribute>
          </img>
        </td>
        <td>
          <table>
            <tr>
              <th>
                <xsl:value-of select="$strings/system_software/name"/>
              </th>
              <td>
                <xsl:value-of select="name"/>

                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/link/https://support.apple.com/guide/mac-help/welcome/mac/',/etrecheck/systemsoftware/version,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/system_software/user_guide_button_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/system_software/user_guide_button"/>
                  <img class="buttonicon" src="image/webbrowserbutton.png"/>
                </button>
              </td>
            </tr>
            <tr>
              <th>
                <xsl:value-of select="$strings/system_software/version"/>
              </th>
              <td><xsl:value-of select="version"/></td>
            </tr>
            <tr>
              <th>
                <xsl:value-of select="$strings/system_software/build"/>
              </th>
              <td><xsl:value-of select="build"/></td>
            </tr>
            <tr>
              <th>
                <xsl:value-of select="$strings/system_software/uptime"/>
              </th>
              <td><xsl:apply-templates select="uptime"/></td>
            </tr>
            <xsl:if test="bootmode = 'safe_boot'">
              <tr>
                <th>
                  <xsl:value-of select="$strings/system_software/safemode"/>
                </th>
                <td>
                  <xsl:value-of
                    select="$strings/system_software/safemode_enabled"/>
                </td>
              </tr>
            </xsl:if>
            <xsl:if test="bootargs">
              <tr>
                <th>
                  <xsl:value-of select="$strings/system_software/bootargs"/>
                </th>
                <td>
                  <xsl:value-of select="bootargs"/>
                </td>
              </tr>
            </xsl:if>
          </table>
        </td>
      </tr>
    </table>

    <p class="description">
      <xsl:copy-of select="$strings/system_software/description/node()"/>
    </p>

  </xsl:template>

  <!-- Uptime. -->
  <xsl:template match="uptime">

    <xsl:variable name="days" select=". div 24"/>
    <xsl:variable name="hours" select=". mod 24"/>
    
    <xsl:choose>
    
      <xsl:when test="$days &gt;= 1">
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/uptime_human"/>
          <xsl:with-param name="key" select="'day'"/>
          <xsl:with-param name="value" select="$days"/>
        </xsl:call-template>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/uptime_human"/>
          <xsl:with-param name="key" select="'hour'"/>
          <xsl:with-param name="value" select="$hours"/>
        </xsl:call-template>
      </xsl:otherwise>
      
    </xsl:choose>
    
  </xsl:template>
  
  <!-- Configuration files. -->
  <xsl:template match="configurationfiles">
  
    <xsl:if test="count(*)">
    
      <table class="tablelist">
      
        <xsl:if test="etcsysctlconfexists">
          <tr>
            <th>
              <xsl:value-of
                select="$strings/configuration/unexpected_file"/>
            </th>
            <td>/etc/sysctl.conf</td>
            <td class="buttoncolumn">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET('/reveal//etc/sysctl.conf')</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/configuration/reveal_in_finder_title"/>
                </xsl:attribute>
                <xsl:value-of
                  select="$strings/configuration/reveal_in_finder"/>
                <img class="buttonicon" src="image/finderbutton.png"/>
              </button>
            </td>
          </tr>
        </xsl:if>
        
        <xsl:if test="etclaunchdconfexists">
          <tr>
            <th>
              <xsl:value-of
                select="$strings/configuration/unexpected_file"/>
            </th>
            <td>/etc/launchd.conf</td>
            <td class="buttoncolumn">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET('/reveal//etc/launchd.conf')</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/configuration/reveal_in_finder_title"/>
                </xsl:attribute>
                <xsl:value-of
                  select="$strings/configuration/reveal_in_finder"/>
                <img class="buttonicon" src="image/finderbutton.png"/>
              </button>
            </td>
          </tr>
        </xsl:if>

        <xsl:choose>
          <xsl:when test="hostsmissing = 'true'">
            <tr>
              <th colspan="2">
                <xsl:value-of select="$strings/configuration/hosts_missing"/>
              </th>
            </tr>
          </xsl:when>
          
          <xsl:when test="hostscount or hostscorrupt = 'true'">
            <tr>
              <th>
                <xsl:value-of select="$strings/configuration/hosts_count"/>
              </th>
              <td>
                <xsl:value-of select="hostscount"/>
                
                <xsl:if test="hostscorrupt = 'true'">
                  <br/><br/>
                  <xsl:value-of
                    select="$strings/configuration/hosts_corrupt"/>
                </xsl:if>
              </td>
              <td class="buttoncolumn">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/reveal//etc/hosts')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/configuration/reveal_in_finder_title"/>
                  </xsl:attribute>
                  <xsl:value-of
                    select="$strings/configuration/reveal_in_finder"/>
                  <img class="buttonicon" src="image/finderbutton.png"/>
                </button>
              </td>
            </tr>
            <xsl:for-each select="hosts/host">
              <tr>
                <th/>
                <td>
                  <xsl:value-of select="name"/>
                </td>
                <td class="buttoncolumn">
                  <xsl:value-of select="address"/>
                </td>
              </tr>
            </xsl:for-each>
            
            <xsl:variable name="count">
              <xsl:value-of select="count(hosts/host)"/>
            </xsl:variable>
            
            <xsl:if test="$count &gt; 0 and hostscount &gt; $count">
              <xsl:variable name="more">
                <xsl:value-of select="hostscount - $count"/>
              </xsl:variable>
              
              <tr>
                <th/>
                <td colspan="2">
                  <xsl:call-template name="format">
                    <xsl:with-param
                      name="format"
                      select="$strings/configuration/hosts_more"/>
                    <xsl:with-param name="value" select="$more"/>
                  </xsl:call-template>
                </td>
              </tr>
            </xsl:if>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>
  
        <!-- Use a for-each just to shorten the context. -->
        <xsl:for-each select="unexpectedsudoerssize">
        
          <xsl:variable
            name="format"
            select="$strings/configuration/unexpected_sudoers_size_format"/>
            
          <tr>
            <th>
              <xsl:value-of
                select="$strings/configuration/unexpected_sudoers_size"/>
            </th>
            <td>
              <xsl:call-template name="format">
                <xsl:with-param name="format" select="$format"/>
                <xsl:with-param name="value" select="size|expectedsize"/>
              </xsl:call-template>
            </td>
            <td class="buttoncolumn">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET('/reveal//etc/sudoers')</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/configuration/reveal_in_finder_title"/>
                </xsl:attribute>
                <xsl:value-of
                  select="$strings/configuration/reveal_in_finder"/>
                <img class="buttonicon" src="image/finderbutton.png"/>
              </button>
            </td>
          </tr>
        </xsl:for-each>

        <xsl:if test="corecount &gt; 0">
          <xsl:variable name="coresize">
            <xsl:apply-templates select="coresize" mode="units"/>
          </xsl:variable>

          <th>
            <xsl:call-template name="plural">
              <xsl:with-param name="keys" select="$strings/configuration"/>
              <xsl:with-param name="key">
                <xsl:text>corecount</xsl:text>
              </xsl:with-param>
              <xsl:with-param name="value" select="corecount"/>
            </xsl:call-template>
          </th>
          <td>
            <xsl:call-template name="format">
              <xsl:with-param name="format"
                select="$strings/configuration/coresize"/>
              <xsl:with-param name="value" select="coresize"/>
            </xsl:call-template>
          </td>
          <td class="buttoncolumn">
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/openfolder//cores')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/configuration/show_in_finder_title"/>
              </xsl:attribute>
              <xsl:value-of
                select="$strings/configuration/show_in_finder"/>
              <img class="buttonicon" src="image/finderbutton.png"/>
            </button>
          </td>
        </xsl:if>
      </table>
      
      <xsl:if
        test="count(/etrecheck/configurationfiles/featureflags//category) &gt; 0">
        <table class="tablelist">
          <tr>
            <th colspan="2">
              <xsl:value-of
                select="$strings/configuration/featureflags"/>
            </th>
          </tr>
          <xsl:for-each
            select="/etrecheck/configurationfiles/featureflags/domain">
          
            <xsl:if test="count(category) &gt; 0">
        
              <tr class="horizontalborder">
                <td class="header" colspan="2">
                  <xsl:value-of select="name"/>
                </td>
              </tr>
              
              <xsl:for-each select="category">
                <tr class="horizontalborder">
                  <td colspan="2">
                    <xsl:value-of select="name"/>
                  </td>
                </tr>
                <tr>
                  <td/>
                  <td>
                    <xsl:value-of select="key"/>
                    <xsl:text> = </xsl:text>
                    <xsl:value-of select="value"/>
                  </td>
                </tr>
              </xsl:for-each>
            </xsl:if>

          </xsl:for-each>
        </table>
      </xsl:if>

    </xsl:if>
    
    <p class="description">
      <xsl:copy-of select="$strings/configuration/description/node()"/>

      <xsl:if test="etcsysctlconfexists or etclaunchdconfexists or hostscount or unexpectedsudoerssize">
        <xsl:copy-of select="$strings/configuration/description_hasfiles/node()"/>
      </xsl:if>
      
      <xsl:if test="corecount &gt; 0">
        <xsl:copy-of select="$strings/configuration/description_corefiles/node()"/>
      </xsl:if>
    </p>

  </xsl:template>
  
  <!-- Go ahead and assume size and expected size should always have
       units. -->
  <xsl:template match="size|expectedsize">
  
    <xsl:apply-templates select="." mode="units"/>
    
  </xsl:template>
  
  <!-- Print startup items. -->
  <xsl:template match="startupitems">
  
    <xsl:if test="count(startupitem) &gt; 0">

      <table class="tablelist">

        <xsl:for-each select="startupitem">

          <tr class="horizontalborder">
          
            <td class="header">
              <xsl:value-of select="$strings/startup_items/name"/>
              <br/>
              <xsl:value-of select="$strings/startup_items/path"/>
              <xsl:if test="version">
                <br/>
                <xsl:value-of select="$strings/startup_items/version"/>
              </xsl:if>
            </td>
            <td>
              <xsl:value-of select="name"/>
              <br/>
              <xsl:value-of select="path"/>
              <xsl:if test="version">
                <xsl:value-of select="$strings/startup_items/version"/>
                <br/>
                <xsl:value-of select="version"/>
              </xsl:if>
            </td>
          </tr>
        
        </xsl:for-each>
      
      </table>
      
    </xsl:if>
    
  </xsl:template>

  <!-- Print Fonts. -->
  <xsl:template match="fonts">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/fonts/name"/></th>
        <th><xsl:value-of select="$strings/fonts/path"/></th>
      </tr>
      <xsl:for-each select="font">
        <tr>
          <td><xsl:value-of select="name"/></td>
          <td><xsl:value-of select="path"/></td>
        </tr>
      </xsl:for-each>
    </table>
    
    <button>
      <xsl:attribute name="onclick">
        <xsl:text>GET('/openapplication//Applications/Font Book.app')</xsl:text>
      </xsl:attribute>
      <xsl:attribute name="title">
        <xsl:value-of select="$strings/fonts/open_fontbook_title"/>
      </xsl:attribute>
      <xsl:value-of
        select="$strings/fonts/open_fontbook"/>
      <img class="buttonicon" src="image/fontbookbutton.png"/>
    </button>

    <p class="description">
      <xsl:copy-of select="$strings/fonts/description/node()"/>
    </p>

  </xsl:template>
  
  <!-- Print a virtual memory usage information. -->
  <xsl:template match="vm">
  
    <table class="tablelist">
      <tr>
        <td class="header"><xsl:value-of select="$strings/vm/physical_ram"/></td>
        <td><xsl:apply-templates select="/etrecheck/hardware/ram" mode="units"/></td>
      </tr>
    </table>
    
    <table class="tablelist">
      <tr>
        <td class="header"><xsl:value-of select="$strings/vm/free_ram"/></td>
        <td><xsl:apply-templates select="freeram" mode="units"/></td>
      </tr>
      <tr>
        <td class="header"><xsl:value-of select="$strings/vm/used_ram"/></td>
        <td><xsl:apply-templates select="usedram" mode="units"/></td>
      </tr>
      <tr>
        <td class="header"><xsl:value-of select="$strings/vm/cached_files"/></td>
        <td><xsl:apply-templates select="filecache" mode="units"/></td>
      </tr>
    </table>
    
    <table class="tablelist">
      <tr>
        <td class="header"><xsl:value-of select="$strings/vm/available_ram"/></td>
        <td><xsl:apply-templates select="availableram" mode="units"/></td>
      </tr>
      <tr>
        <td class="header"><xsl:value-of select="$strings/vm/swap_used"/></td>
        <td><xsl:apply-templates select="swapused" mode="units"/></td>
      </tr>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/vm/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print diagnostics information. -->
  <xsl:template name="diagnostics">
  
    <xsl:variable name="try">
      <xsl:choose>
        <xsl:when
          test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when test="insufficientpermissions = 'true'">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of
            select="count(/etrecheck/diagnostics/event[type != 10]) &gt; 0"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$try = 'true'">
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">diagnostics</xsl:with-param>
        <xsl:with-param name="contents">
        
          <xsl:if test="/etrecheck/diagnostics/event">
            <table class="tablelist">
            
              <xsl:apply-templates
                select="/etrecheck/diagnostics/event[type != 10]"/>
              
            </table>
          </xsl:if>
          
          <xsl:choose>
            <xsl:when
              test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas' and /etrecheck/diagnostics/insufficientpermissions = 'true'">
              
              <p>
                <button class="mas">
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/help/fulldriveaccess')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of
                      select="$strings/diagnostics/fulldriveaccess_title"/>
                  </xsl:attribute>
                  <xsl:value-of
                    select="$strings/diagnostics/fulldriveaccess"/>
                </button>
                <strong>
                  <xsl:value-of
                    select="$strings/diagnostics/diagnosticsunavailable"/>
                </strong>
              </p>

              <p>
                <strong>
                  <xsl:value-of
                    select="$strings/diagnostics/needsfulldriveaccess"/>
                </strong>
              </p>

              <p>
                <strong>
                  <xsl:value-of
                    select="$strings/diagnostics/insufficientpermissions"/>
                </strong>
              </p>

            </xsl:when>
            <xsl:when
              test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
              
              <p>
                <button class="mas">
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/help/fulldriveaccess')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of
                      select="$strings/diagnostics/fulldriveaccess_title"/>
                  </xsl:attribute>
                  <xsl:value-of
                    select="$strings/diagnostics/fulldriveaccess"/>
                </button>
                <strong>
                  <xsl:value-of
                    select="$strings/diagnostics/diagnosticsunavailable"/>
                </strong>
              </p>

              <p>
                <strong>
                  <xsl:value-of
                    select="$strings/diagnostics/needsfulldriveaccess"/>
                </strong>
              </p>

            </xsl:when>
            <xsl:when
              test="/etrecheck/diagnostics/insufficientpermissions = 'true'">
            
              <p>
                <strong>
                  <xsl:value-of
                    select="$strings/diagnostics/insufficientpermissions"/>
                </strong>
              </p>
              
            </xsl:when>
            <xsl:otherwise/>
          </xsl:choose>

          <p class="description">
            <xsl:copy-of select="$strings/diagnostics/description/node()"/>
            
            <xsl:if test="count(/etrecheck/diagnostics/event/event[type &lt; 4]) &gt; 0">
              <xsl:copy-of
                select="$strings/diagnostics/description_reports/node()"/>
            </xsl:if>
          </p>

        </xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    
  </xsl:template>

  <!-- Print a diagnostics event. -->
  <xsl:template match="event">
  
    <xsl:variable name="type" select="type"/>
    <xsl:variable name="name" select="name"/>
    <xsl:variable
      name="eventtype"
      select="$strings/diagnostics/types/value[@key = $type]"/>
    
    <tr class="horizontalborder">
      <td class="header">
        <xsl:value-of select="$strings/diagnostics/type"/>
      </td>
      <td class="value">
        <xsl:value-of select="$eventtype"/>
        <xsl:choose>
        
          <xsl:when test="type = 4"/>
          <xsl:when test="type = 5"/>
          <xsl:when test="type = 7"/>
          <xsl:when test="type = 8"/>
          <xsl:when test="type = 9"/>
          <xsl:when test="type = 10"/>
          
          <xsl:when test="path">
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('GET(&quot;/consoleopen/',path_unsafe,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/diagnostics/open_report_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/diagnostics/open_report"/>
              <img class="buttonicon" src="image/consolebutton.png"/>
            </button>
          </xsl:when>
          
          <xsl:otherwise/>
          
        </xsl:choose>
      </td>
    </tr>
    
    <tr>
      <td class="header">
        <xsl:choose>
        
          <xsl:when test="type = 5"/>
          <xsl:when test="type = 6 and string-length(details) &gt; 0">
            <xsl:value-of select="$strings/diagnostics/details"/>
          </xsl:when>
          <xsl:when test="type = 6"/>
          <xsl:when test="type = 14 and string-length(details) &gt; 0">
            <xsl:value-of select="$strings/diagnostics/details"/>
          </xsl:when>
          <xsl:when test="type = 14"/>
          <xsl:when test="type = 12 and string-length(details) &gt; 0">
            <xsl:value-of select="$strings/diagnostics/details"/>
          </xsl:when>
          <xsl:when test="type = 12"/>
          <xsl:when test="type = 8">
            <xsl:value-of select="$strings/diagnostics/details"/>
          </xsl:when>
          
          <xsl:when test="type = 10">
            <xsl:value-of select="$strings/diagnostics/reason"/>
          </xsl:when>

          <xsl:otherwise>
            <xsl:value-of select="$strings/diagnostics/application"/>
          </xsl:otherwise>
          
        </xsl:choose>
      </td>
      <td class="value">
        <xsl:choose>
        
          <xsl:when test="type = 5"/>
          <xsl:when test="type = 6 and string-length(details) &gt; 0">
            <xsl:value-of select="details"/>
          </xsl:when>
          <xsl:when test="type = 6"/>
          <xsl:when test="type = 14 and string-length(details) &gt; 0">
            <xsl:value-of select="details"/>
          </xsl:when>
          <xsl:when test="type = 14"/>
          <xsl:when test="type = 12 and string-length(details) &gt; 0">
            <xsl:value-of select="details"/>
          </xsl:when>
          <xsl:when test="type = 12"/>

          <xsl:when test="type = 10">
            <xsl:value-of select="code"/>
            <xsl:text> - </xsl:text>
            <xsl:value-of
              select="$strings/diagnostics/shutdown/value[@key = $name]"/>
          </xsl:when>

          <xsl:when test="bundle_unsafe">
            <xsl:apply-templates select="name"/>
            
            <xsl:if test="executable_unsafe">
              <xsl:value-of select="concat(' (', executable_unsafe, ')')"/>
            </xsl:if>
            
            <xsl:choose>
              <xsl:when test="man">
                <!-- Provide a button to an Apple man page. -->
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/man/',man,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/diagnostics/about_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/diagnostics/about"/>
                  <img class="buttonicon" src="image/man.png"/>
                </button>
              </xsl:when>
              <xsl:when test="apple = 'true'"/>
              <xsl:otherwise>
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/reveal/',bundle_unsafe,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of
                      select="$strings/diagnostics/reveal_in_finder_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/diagnostics/reveal_in_finder"/>
                  <img class="buttonicon" src="image/finderbutton.png"/>
                </button>
              </xsl:otherwise>
            </xsl:choose>
            
          </xsl:when>
          
          <xsl:when test="executable_unsafe">
            <xsl:apply-templates select="name"/>
            
            <xsl:choose>
              <xsl:when test="man">
                <!-- Provide a button to an Apple man page. -->
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/man/',man,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/diagnostics/about_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/diagnostics/about"/>
                  <img class="buttonicon" src="image/man.png"/>
                </button>
              </xsl:when>
              <xsl:when test="apple = 'true'"/>
              <xsl:otherwise>
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/reveal/',executable_unsafe,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of
                      select="$strings/diagnostics/reveal_in_finder_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/diagnostics/reveal_in_finder"/>
                  <img class="buttonicon" src="image/finderbutton.png"/>
                </button>
              </xsl:otherwise>
            </xsl:choose>
            
          </xsl:when>

          <xsl:otherwise>
            <xsl:apply-templates select="name"/>
          </xsl:otherwise>
          
        </xsl:choose>
      </td>
    </tr>

    <xsl:choose>
    
      <xsl:when test="type = 5"/>
      <xsl:when test="type = 6"/>
      <xsl:when test="type = 14"/>
      <xsl:when test="type = 12"/>
      <xsl:when test="type = 8"/>
      <xsl:when test="type = 10"/>

      <xsl:when test="bundle_unsafe or executable_unsafe">
        <tr>
          <td class="header">
            <xsl:value-of select="$strings/diagnostics/executable"/>
          </td>
          <td class="value">
            <xsl:choose>
            
              <xsl:when test="bundle_unsafe">
                <xsl:apply-templates select="bundle_unsafe"/>
              </xsl:when>
              
              <xsl:when test="executable_unsafe">
                <xsl:apply-templates select="executable_unsafe"/>
              </xsl:when>

              <xsl:otherwise/>
              
            </xsl:choose>
          </td>
        </tr>
      </xsl:when>
      
      <xsl:otherwise/>
      
    </xsl:choose>

    <xsl:choose>
    
      <xsl:when test="count &gt; 1">
        <tr>
          <td class="header">
            <xsl:value-of select="$strings/diagnostics/count"/>
          </td>
          <td class="value">
            <xsl:apply-templates select="count"/>
          </td>
        </tr>
        <xsl:if test="firstdate and firstdate != date">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/diagnostics/firstdate"/>
            </td>
            <td class="value">
              <xsl:apply-templates select="firstdate"/>
            </td>
          </tr>
        </xsl:if>
        <tr>
          <td class="header">
            <xsl:value-of select="$strings/diagnostics/lastdate"/>
          </td>
          <td class="value">
            <xsl:apply-templates select="date"/>
          </td>
        </tr>
      </xsl:when>

      <xsl:otherwise>
        <tr>
          <td class="header">
            <xsl:value-of select="$strings/diagnostics/date"/>
          </td>
          <td class="value">
            <xsl:apply-templates select="date"/>
          </td>
        </tr>
      </xsl:otherwise>
      
    </xsl:choose>
    
    <!-- If I have information, add it. -->
    <xsl:if test="information">
      <tr>
      
        <!-- Make sure to use a description type to vertically line up
             the text properly. -->
        <td class="description header">
          <xsl:choose>
            <xsl:when test="type = 6">
              <xsl:value-of select="$strings/diagnostics/kernelextensions"/>
            </xsl:when>
            <xsl:when test="type = 14">
              <xsl:value-of select="$strings/diagnostics/kernelextensions"/>
            </xsl:when>
            <xsl:when test="type = 12">
              <xsl:value-of select="$strings/diagnostics/kernelextensions"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="$strings/diagnostics/details"/>
            </xsl:otherwise>
          </xsl:choose>
        </td>
        <td class="description">
          <xsl:apply-templates select="information"/>
        </td>
      </tr>
    </xsl:if>
    
    <xsl:if test="type = 10 and code = -60">
      <tr>
        <td class="disk_repair" colspan="2">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET("/link/https://support.apple.com/guide/disk-utility/repair-a-disk-dskutl1040/mac");</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/diagnostics/disk_repair_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/diagnostics/disk_repair"/>
            <img class="buttonicon" src="image/webbrowserbutton.png"/>
          </button>
        </td>
      </tr>
    </xsl:if>
    
  </xsl:template>

  <!-- Print additional information about an event. -->
  <xsl:template match="information">
  
    <xsl:if test="../type = 6 or ../type = 14">
      <xsl:choose>
        <xsl:when test="string-length(.) = 0">
          <xsl:value-of select="$strings/diagnostics/none"/>
        </xsl:when>
      
        <xsl:otherwise/>
      </xsl:choose>
    </xsl:if>
    
    <xsl:value-of select="."/>
    
  </xsl:template>
  
  <!-- Physical drive information. -->
  <xsl:template match="disk">
  
    <xsl:apply-templates select="drive"/>
    
    <p class="description">
      <xsl:copy-of select="$strings/drive/description/node()"/>
    </p>

  </xsl:template>

  <!-- Drive information. -->
  <xsl:template match="drive">
  
    <xsl:variable name="type" select="type"/>
    <xsl:variable name="mechanism" select="mechanism"/>
    <xsl:variable name="speed" select="busspeed"/>
    <xsl:variable name="smart" select="smartstatus"/>
    
    <!-- Get the localized bus speed name. -->
    <xsl:variable name="localizedbusspeed">
      <xsl:value-of
        select="$strings/drive/speeds/value[@key = $speed]"/>
    </xsl:variable>

    <!-- If the localized bus speed name is empty, just use what I have. -->
    <xsl:variable name="busspeed">
      <xsl:choose>
        <xsl:when test="string-length($localizedbusspeed)">
          <xsl:value-of select="$localizedbusspeed"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$speed"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!-- Get the localized smartstatus. -->
    <xsl:variable name="localizedsmartstatus">
      <xsl:value-of
        select="$strings/drive/smart/value[@key = $smart]"/>
    </xsl:variable>

    <!-- If the localized smart status is empty, just use what I have. -->
    <xsl:variable name="smartstatus">
      <xsl:choose>
        <xsl:when test="string-length($localizedsmartstatus)">
          <xsl:value-of select="$localizedsmartstatus"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$smart"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <table class="tablelist">
      <tr>
        <th colspan="3">
          <xsl:value-of select="model"/>
        </th>
      </tr>
      
      <tr class="horizontalborder">
   
        <td>
          <img>
            <xsl:attribute name="src">
              <xsl:value-of select="concat('image/device/', device)"/>
            </xsl:attribute>
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/drive/icon"/>
            </xsl:attribute>
          </img>
        </td>
        <td class="header">
          <xsl:value-of select="$strings/drive/device"/>
          
          <br/>
          <xsl:value-of select="$strings/drive/size"/>
          
          <xsl:if test="bus">
            <br/>
            <xsl:value-of select="$strings/drive/bus"/>
          </xsl:if>
          
          <xsl:if test="string-length($busspeed) &gt; 0">
            <br/>
            <xsl:value-of select="$strings/drive/busspeed"/>
          </xsl:if>

          <xsl:if test="type">
            <br/>
            <xsl:value-of select="$strings/drive/type"/>
          </xsl:if>
          
          <br/>
          <xsl:value-of select="$strings/drive/connection"/>

          <xsl:if test="string-length($mechanism) &gt; 0">
            <br/>
            <xsl:value-of select="$strings/drive/medium"/>
          </xsl:if>
          
          <xsl:if test="TRIM">
            <br/>
            <xsl:value-of select="$strings/drive/TRIM"/>
          </xsl:if>
          
          <xsl:if test="rpm">
            <br/>
            <xsl:value-of select="$strings/drive/rpm"/>
          </xsl:if>
          
          <xsl:choose>
            <xsl:when test="not(smartstatus)"/>
            <xsl:when test="smartstatus = 'Verified'"/>
            <xsl:when test="smartstatus = 'Not Supported'"/>
            <xsl:otherwise>
              <br/>
              <xsl:value-of select="$strings/drive/smart_status"/>
            </xsl:otherwise>
          </xsl:choose>

        </td>
        <td>
          <xsl:value-of select="device"/>
          
          <br/>
          <xsl:apply-templates select="size" mode="units"/>
          
          <xsl:if test="bus">
            <br/>
            <xsl:value-of select="bus"/>
          </xsl:if>
          
          <xsl:if
            test="string-length($busspeed) &gt; 0">
            <br/>
            <xsl:value-of select="$busspeed"/>
          </xsl:if>

          <xsl:if test="type">
            <br/>
            <xsl:value-of select="$strings/drive/types/value[@key = $type]"/>
          </xsl:if>
          
          <br/>
          <xsl:choose>
            <xsl:when test="internal = 'true'">
              <xsl:value-of select="$strings/drive/internal"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="$strings/drive/external"/>
            </xsl:otherwise>
          </xsl:choose>
    
          <xsl:if test="string-length($mechanism) &gt; 0">
            <br/>
            <xsl:value-of
              select="$strings/drive/mechanisms/value[@key = $mechanism]"/>
          </xsl:if>
          
          <xsl:if test="TRIM">
            <br/>
            <xsl:apply-templates select="TRIM"/>
          </xsl:if>
          
          <xsl:if test="rpm">
            <br/>
            <xsl:apply-templates select="rpm"/>
          </xsl:if>

          <xsl:choose>
            <xsl:when test="not(smartstatus)"/>
            <xsl:when test="smartstatus = 'Verified'"/>
            <xsl:when test="smartstatus = 'Not Supported'"/>
            <xsl:otherwise>
              <br/>
              <xsl:value-of select="$smartstatus"/>
            </xsl:otherwise>
          </xsl:choose>

        </td>
      </tr>
      
      <xsl:if test="volumes/device">
        <tr class="horizontalborder">
          <td class="partitions" colspan="3">
            <p><xsl:value-of select="$strings/drive/partitions"/></p>
            <xsl:apply-templates select="volumes" mode="partition"/>
          </td>
        </tr>
      </xsl:if>
    </table>
    
  </xsl:template>

  <!-- A mini-list of devices. -->
  <xsl:template match="volumes" mode="partition">
  
    <ul>
      <xsl:apply-templates select="device" mode="partition"/>
    </ul>
    
  </xsl:template>
  
  <!-- A mini-list of devices. -->
  <xsl:template match="device" mode="partition">
  
    <xsl:variable name="device" select="."/>
    
    <!-- Get the volume. -->
    <xsl:variable
      name="volume"
      select="/etrecheck/virtualvolume/volume[device = $device]"/>
    
    <!-- Get the localized filesystem name. -->
    <xsl:variable name="localizedfilesystem">
      <xsl:value-of
        select="$strings/volume/filesystems/value[@key = $volume/filesystem]"/>
    </xsl:variable>

    <!-- If the localized filesystem name is empty, just use what I have. -->
    <xsl:variable name="filesystem">
      <xsl:choose>
        <xsl:when test="string-length($localizedfilesystem)">
          <xsl:value-of select="$localizedfilesystem"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$volume/filesystem"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <li>
      <xsl:choose>
        <xsl:when test="$printing = 'false'">
          <a href="#">
            <xsl:attribute name="onclick">
              <xsl:value-of select="concat('showPartition(event, &quot;',$device, '&quot;)')"/>
            </xsl:attribute>
            
            <xsl:value-of select="$device"/>
          </a>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$device"/>
        </xsl:otherwise>
      </xsl:choose>
      
      <xsl:text> - </xsl:text>

      <xsl:choose>
        <xsl:when test="$volume/name_unsafe">
          <xsl:value-of select="$volume/name_unsafe"/>
          <xsl:text> </xsl:text>
        </xsl:when>
        
        <xsl:when test="$volume/type">
          <xsl:text>[</xsl:text>
          <xsl:value-of
            select="$strings/volume/types/value[@key = $volume/type]"/>
          <xsl:text>] </xsl:text>
        </xsl:when>
        
        <xsl:otherwise/>
      </xsl:choose>
      
      <xsl:choose>
        <xsl:when test="$volume/used &gt; 0">
          <xsl:text>(</xsl:text>
          <xsl:call-template name="format">
            <xsl:with-param
              name="format" select="$strings/drive/used"/>
            <xsl:with-param name="value">
              <xsl:apply-templates select="$volume/used" mode="units"/>
            </xsl:with-param>
          </xsl:call-template>
          <xsl:text>)</xsl:text>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
      
      <xsl:if test="string-length($filesystem) &gt; 0">
        <xsl:text> </xsl:text>
        <xsl:value-of select="$filesystem"/>
      </xsl:if>
      
      <xsl:if test="$volume/volumes/device">
        <br/>
        <p><xsl:value-of select="$strings/volume/partitions"/></p>
        <xsl:apply-templates select="$volume/volumes" mode="partition"/>
      </xsl:if>
    </li>
    
  </xsl:template>

  <!-- Volume information. -->
  <xsl:template match="virtualvolume" mode="top">
  
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">virtualvolume</xsl:with-param>
      <xsl:with-param name="control">
      
        <button id="show_unmounted_volumes">
          <xsl:attribute name="onclick">
            <xsl:text>showUnmountedVolumes()</xsl:text>
          </xsl:attribute>
          <xsl:value-of select="$strings/volume/show_unmounted_volumes"/>
        </button>

        <button id="hide_unmounted_volumes">
          <xsl:attribute name="onclick">
            <xsl:text>hideUnmountedVolumes()</xsl:text>
          </xsl:attribute>
          <xsl:value-of select="$strings/volume/hide_unmounted_volumes"/>
        </button>
      </xsl:with-param>
      
      <xsl:with-param name="contents">
        <xsl:call-template name="volumes"/>
      </xsl:with-param>
    </xsl:call-template>

  </xsl:template>
  
  <!-- Volume information. -->
  <xsl:template name="volumes">
  
    <xsl:apply-templates select="volume"/>

    <p class="description">
      <xsl:copy-of select="$strings/volume/description/node()"/>
    </p>

  </xsl:template>

  <!-- Virtual volume information. -->
  <xsl:template match="volume">
  
    <xsl:variable name="type" select="type"/>

    <!-- Get the filesystem. -->
    <xsl:variable name="rawfilesystem" select="filesystem"/>

    <!-- Get the localized filesystem name. -->
    <xsl:variable name="localizedfilesystem">
      <xsl:value-of
        select="$strings/volume/filesystems/value[@key = $rawfilesystem]"/>
    </xsl:variable>

    <!-- If the localized filesystem name is empty, just use what I have. -->
    <xsl:variable name="filesystem">
      <xsl:choose>
        <xsl:when test="string-length($localizedfilesystem)">
          <xsl:value-of select="$localizedfilesystem"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="filesystem"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!-- Is this a shared volume? -->
    <xsl:variable name="shared">
      <xsl:call-template name="isshared">
        <xsl:with-param name="volume" select="."/>
        <xsl:with-param name="filesystem" select="$filesystem"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:variable name="isfusionvolume">
      <xsl:call-template name="isfusionvolume">
        <xsl:with-param name="volume" select="."/>
        <xsl:with-param name="root" select="$report"/>
      </xsl:call-template>
    </xsl:variable>

    <table class="tablelist">
      <xsl:attribute name="id">
        <xsl:value-of select="device"/>
      </xsl:attribute>
    
      <xsl:choose>
    
        <!-- Assign a notmounted class so that these volumes can be shown
             or hidden as requested. -->
        <xsl:when test="mountpoint">
          <xsl:attribute name="class">
            <xsl:text>tablelist</xsl:text>
          </xsl:attribute>
        </xsl:when>
        
        <xsl:otherwise>
          <xsl:attribute name="class">
            <xsl:text>tablelist notmounted</xsl:text>
          </xsl:attribute>
        </xsl:otherwise>
      </xsl:choose>
      
      <tr>
        <th colspan="3">
          
          <xsl:choose>
            <xsl:when test="name">
              <xsl:value-of select="name_unsafe"/>
            </xsl:when>
            
            <xsl:otherwise>
              <xsl:value-of select="device"/>
            </xsl:otherwise>
            
          </xsl:choose>
          
        </th>
      </tr>
      <tr class="horizontalborder">
        <td>
          <img>
            <xsl:attribute name="src">
              <xsl:choose>
                <xsl:when test="diskimage = 'true'">
                  <xsl:text>image/diskimage</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="concat('image/device/', device)"/>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:attribute>
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/volume/icon"/>
            </xsl:attribute>
          </img>
        </td>
        <td class="header">
          <xsl:value-of select="$strings/volume/device"/>
          
          <xsl:if test="string-length($filesystem) &gt; 0">
            <br/>
            <xsl:value-of select="$strings/volume/filesystem"/>
          </xsl:if>
          
          <br/>
          <xsl:value-of select="$strings/volume/mount_point"/>
          
          <xsl:if test="type">
            <br/>
            <xsl:value-of select="$strings/volume/type"/>
          </xsl:if>
          
          <xsl:if test="diskimage = 'true'">
            <br/>
            <xsl:value-of select="$strings/volume/type"/>
          </xsl:if>

          <xsl:if test="$isfusionvolume = 'true'">
            <br/>
            <xsl:value-of select="$strings/volume/fusiondrive"/>
          </xsl:if>

          <xsl:if test="noowners = 'true'">
            <br/>
            <xsl:value-of select="$strings/volume/owners"/>
          </xsl:if>

          <xsl:if test="readonly = 'true'">
            <br/>
            <xsl:value-of select="$strings/volume/readonly"/>
          </xsl:if>

          <xsl:if test="encryption">
            <br/>
            <xsl:value-of select="$strings/volume/encryption_status"/>
          </xsl:if>

          <xsl:if test="errors">
            <br/>
            <xsl:value-of select="$strings/volume/errors"/>
          </xsl:if>

          <xsl:if test="used &gt; 0">
            <br/>
            <xsl:value-of select="$strings/volume/used"/>
          </xsl:if>

          <br/>
          <xsl:value-of select="$strings/volume/kind"/>

          <xsl:choose>
            <xsl:when test="type = 'apfscontainervolume'"/>
            <xsl:when test="type = 'apfsvirtualdrivevolume'"/>
            <xsl:when test="type = 'apfsprebootvolume'"/>
            <xsl:when test="name = 'xART'"/>
            <xsl:when test="name = 'Hardware' and mountpoint = '/System/Volumes/Hardware'"/>
            <xsl:when test="type = 'apfsrecovery'"/>
            <xsl:when test="type = 'apfssnapshot'"/>
            <xsl:when test="type = 'efivolume'"/>
            <xsl:when test="type = 'apfsvm'"/>
            <xsl:when test="name = 'Preboot'"/>
            <xsl:when test="name = 'Update'"/>
            <xsl:otherwise>

              <xsl:if test="$shared = 'true'">
                <br/>
                <xsl:value-of select="$strings/volume/shared"/>
                <br/>
              </xsl:if>

              <br/>
              <xsl:value-of select="$strings/volume/size"/>

              <xsl:if test="type = 'apfsvirtualdrivevolume'">
                <br/>
                <xsl:value-of select="$strings/volume/sharing"/>
              </xsl:if>
              
              <xsl:if test="available and available &gt; 0 and free &gt; 0 and mountpoint and free != available">
                <br/>
                <xsl:value-of select="$strings/volume/available"/>
              </xsl:if>

              <xsl:if test="free &gt; 0 and mountpoint">
                <br/>
                <xsl:value-of select="$strings/volume/free"/>
              </xsl:if>

            </xsl:otherwise>
          </xsl:choose>
          
        </td>
        <td>
          <xsl:value-of select="device"/>
          
          <xsl:if test="string-length($filesystem) &gt; 0">
            <br/>
            <xsl:value-of select="$filesystem"/>
          </xsl:if>
          
          <br/>
          <xsl:choose>
            <xsl:when test="mountpoint_unsafe">
              <xsl:value-of select="mountpoint_unsafe"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="$strings/volume/not_mounted"/>
            </xsl:otherwise>
          </xsl:choose>
          
          <xsl:if test="type">
            <br/>
            <xsl:value-of
              select="$strings/volume/types/value[@key = $type]"/>
          </xsl:if>
          
          <xsl:if test="diskimage = 'true'">
            <br/>
            <xsl:value-of
              select="$strings/volume/types/value[@key = 'diskimage']"/>
          </xsl:if>

          <xsl:if test="$isfusionvolume = 'true'">
            <br/>
            <xsl:value-of select="$strings/volume/yes"/>
          </xsl:if>

          <xsl:if test="noowners = 'true'">
            <br/>
            <xsl:value-of select="$strings/volume/no"/>
          </xsl:if>

          <xsl:if test="readonly = 'true'">
            <br/>
            <xsl:value-of select="$strings/volume/yes"/>
          </xsl:if>

          <xsl:if test="encryption">
            <xsl:variable
              name="encryptionstatus" select="encryption/status"/>
            
            <br/>
            <xsl:value-of
              select="$strings/volume/encryption/value[@key = $encryptionstatus]"/>
              
            <xsl:if test="substring(encryption/status, 3, 8) = 'crypting'">
              <xsl:if test="$encryptionstatus != 'encrypted'">
                <div class="progress">
                  <xsl:call-template name="progress">
                    <xsl:with-param
                      name="percentage" select="encryption/progress"/>
                    <xsl:with-param
                      name="units" select="$strings/volume/percent_done"/>
                  </xsl:call-template>
                </div>
              </xsl:if>
            </xsl:if>
          </xsl:if>

          <xsl:if test="errors">
   
            <br/>
            <xsl:call-template name="plural">
              <xsl:with-param name="keys" select="$strings/volume"/>
              <xsl:with-param name="key">
                <xsl:text>errors</xsl:text>
              </xsl:with-param>
              <xsl:with-param name="value" select="count(errors/error)"/>
            </xsl:call-template>

          </xsl:if>

          <xsl:if test="used &gt; 0">
            <br/>
            <xsl:apply-templates select="used" mode="units"/>
          </xsl:if>

          <br/>
          
          <xsl:choose>
            <xsl:when test="type = 'apfscontainervolume'">
              <xsl:value-of select="$strings/volume/system"/>
            </xsl:when>
            <xsl:when test="type = 'apfsvirtualdrivevolume'">
              <xsl:value-of select="$strings/volume/system"/>
            </xsl:when>
            <xsl:when test="type = 'apfsprebootvolume'">
              <xsl:value-of select="$strings/volume/system"/>
            </xsl:when>
            <xsl:when test="name = 'xART'">
              <xsl:value-of select="$strings/volume/system"/>
            </xsl:when>
            <xsl:when
              test="name = 'Hardware' and mountpoint = '/System/Volumes/Hardware'">
              <xsl:value-of select="$strings/volume/system"/>
            </xsl:when>
            <xsl:when test="type = 'apfsrecovery'">
              <xsl:value-of select="$strings/volume/system"/>
            </xsl:when>
            <xsl:when test="type = 'apfssnapshot'">
              <xsl:value-of select="$strings/volume/system"/>
            </xsl:when>
            <xsl:when test="type = 'efivolume'">
              <xsl:value-of select="$strings/volume/system"/>
            </xsl:when>
            <xsl:when test="type = 'apfsvm'">
              <xsl:value-of select="$strings/volume/system"/>
            </xsl:when>
            <xsl:when test="name = 'Preboot'">
              <xsl:value-of select="$strings/volume/system"/>
            </xsl:when>
            <xsl:when test="name = 'Update'">
              <xsl:value-of select="$strings/volume/system"/>
            </xsl:when>
            <xsl:otherwise>

              <xsl:value-of select="$strings/volume/data"/>

              <xsl:if test="$shared = 'true'">
                <br/>
                <xsl:value-of select="$strings/volume/yes"/>
                <br/>
                <span class="sharedvalues">
                  <xsl:value-of select="$strings/volume/sharedvalues"/>
                </span>
              </xsl:if>

              <br/>
              <xsl:apply-templates select="size" mode="units"/>
              
              <xsl:if test="type = 'apfsvirtualdrivevolume'">
                <br/>
                <xsl:call-template name="format">
                  <xsl:with-param
                    name="format" select="$strings/volume/sharing_format"/>
                  <xsl:with-param name="value">
                    <xsl:value-of select="count(volumes/device)"/>
                  </xsl:with-param>
                </xsl:call-template>
              </xsl:if>

              <xsl:if test="available and available &gt; 0 and free &gt; 0 and mountpoint and free != available">
                <br/>
                <xsl:apply-templates select="available" mode="units"/>
                
                <xsl:if test="istimemachine = 'false'">
                  <xsl:call-template name="freespace_gauge">
                    <xsl:with-param name="size" select="size"/>
                    <xsl:with-param name="free" select="available"/>
                  </xsl:call-template>
                </xsl:if>
              </xsl:if>
              
              <xsl:if test="free &gt; 0 and mountpoint">

                <br/>
                <xsl:apply-templates select="free" mode="units"/>
                
                <xsl:if test="istimemachine = 'false'">
                  <xsl:call-template name="freespace_gauge">
                    <xsl:with-param name="size" select="size"/>
                    <xsl:with-param name="free" select="free"/>
                  </xsl:call-template>
                </xsl:if>
              </xsl:if>
              
            </xsl:otherwise>
          </xsl:choose>

        </td>
      </tr>
      
      <xsl:if test="volumes/device">
        <tr>
          <td class="partitions" colspan="3">
            <p><xsl:value-of select="$strings/volume/partitions"/></p>
            <xsl:apply-templates select="volumes" mode="partition"/>
          </td>
        </tr>
      </xsl:if>

    </table>
    
  </xsl:template>

  <!-- Print Time Machine information. -->
  <xsl:template match="timemachine">
  
    <xsl:variable name="showdeletebutton">
      <xsl:choose>
        <xsl:when test="localsnapshotcount = 0">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="true()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <table class="tablelist">
      <tr>
        <th colspan="4">
          <xsl:value-of select="$strings/tm/configuration"/>
        </th>
      </tr>
      
      <xsl:choose>
        <xsl:when
          test="$fulldriveaccess != 'true' and notconfigured = 'true'">

          <tr class="horizontalborder">
            <td>
              <img src="image/timemachine/icon">
                <xsl:attribute name="alt">
                  <xsl:value-of select="$strings/tm/tm_icon"/>
                </xsl:attribute>
              </img>
            </td>
            <td colspan="2">
              <xsl:value-of select="$strings/tm/notconfigured"/>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET('/openapplication//System/Library/PreferencePanes/TimeMachine.prefPane')</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/tm/fix_title[@version = $prefversion or not(@version)]"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/tm/fix"/>
                <img class="buttonicon" src="image/timemachineprefpanebutton.png"/>
              </button>
            </td>
          </tr>

        </xsl:when>
        <xsl:when test="$fulldriveaccess != 'true'">

          <tr class="horizontalborder">
            <td>
              <img src="image/timemachine/icon">
                <xsl:attribute name="alt">
                  <xsl:value-of select="$strings/tm/tm_icon"/>
                </xsl:attribute>
              </img>
            </td>
            <td colspan="2">
              <xsl:value-of select="$strings/tm/fulldriveaccessdisabled"/>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET('/help/fulldriveaccess')</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/tm/fulldriveaccess_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/tm/fulldriveaccess"/>
              </button>
            </td>
          </tr>

        </xsl:when>
        <xsl:when
          test="notconfigured = 'true' and count(destinations/destination) = 0">

          <tr class="horizontalborder">
            <td>
              <img src="image/timemachine/icon">
                <xsl:attribute name="alt">
                  <xsl:value-of select="$strings/tm/tm_icon"/>
                </xsl:attribute>
              </img>
            </td>
            <td colspan="2">
              <xsl:value-of select="$strings/tm/notconfigured"/>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET('/openapplication//System/Library/PreferencePanes/TimeMachine.prefPane')</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/tm/fix_title[@version = $prefversion or not(@version)]"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/tm/fix"/>
                <img class="buttonicon" src="image/timemachineprefpanebutton.png"/>
              </button>
            </td>
          </tr>

        </xsl:when>
        
        <xsl:otherwise>
        
          <tr class="horizontalborder">
            <td>
              <img src="image/timemachine/icon">
                <xsl:attribute name="alt">
                  <xsl:value-of select="$strings/tm/tm_icon"/>
                </xsl:attribute>
              </img>
            </td>
            <td class="header">
              <xsl:if test="notconfigured = 'true'">
                <xsl:value-of select="$strings/tm/notconfigured"/>
                <br/>
              </xsl:if>
              <xsl:if test="skipsystemfiles">
                <xsl:value-of select="$strings/tm/skip_system_files"/>
                <br/>
              </xsl:if>
              <xsl:if test="mobilebackups">
                <xsl:value-of select="$strings/tm/mobile_backups"/>
                <br/>
              </xsl:if>
              <xsl:value-of select="$strings/tm/auto_backup"/>
            </td>
            <td>
              <xsl:if test="notconfigured = 'true'">
                <br/>
              </xsl:if>
              <xsl:if test="skipsystemfiles">
                <xsl:apply-templates select="skipsystemfiles"/>
                <br/>
              </xsl:if>
              <xsl:if test="mobilebackups">
                <xsl:apply-templates select="mobilebackups"/>
                <br/>
              </xsl:if>
              <xsl:apply-templates select="autobackup"/>
              
              <xsl:if test="autobackup != 'true'">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/openapplication//System/Library/PreferencePanes/TimeMachine.prefPane')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/tm/fix_title[@version = $prefversion or not(@version)]"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/tm/fix"/>
                  <img class="buttonicon" src="image/timemachineprefpanebutton.png"/>
                </button>
              </xsl:if>
            </td>
            <td/>
          </tr>
          
          <xsl:if test="volumesbeingbackedup/volume">
        
            <tr>
              <th colspan="4">
                <xsl:value-of select="$strings/tm/drives_backed_up"/>
              </th>
            </tr>
            
            <xsl:apply-templates
              select="volumesbeingbackedup/volume" mode="timemachine"/>
      
          </xsl:if>
          
        </xsl:otherwise>
      </xsl:choose>
      
      <xsl:if test="count(destinations/destination) &gt; 0">
        <tr>
          <th colspan="4">
            <xsl:value-of select="$strings/tm/destinations"/>
          </th>
        </tr>
      
        <xsl:apply-templates select="destinations/destination"/>
      </xsl:if>

      <xsl:if test="localsnapshotcount &gt; 0">
        <tr>
          <th colspan="4">
            <xsl:value-of select="$strings/tm/localsnapshots"/>
          </th>
        </tr>
        <tr class="horizontalborder">
          <td colspan="4" class="header">
            <xsl:call-template name="plural">
              <xsl:with-param name="keys" select="$strings/tm"/>
              <xsl:with-param name="key">
                <xsl:text>localsnapshots</xsl:text>
              </xsl:with-param>
              <xsl:with-param name="value" select="localsnapshotcount"/>
            </xsl:call-template>
          </td>
        </tr>
        <tr>
          <td colspan="4">
            <span class="localsnapshotdates">
              <xsl:value-of select="$strings/tm/oldest_localsnapshot"/>
            </span>
            <xsl:value-of select="oldestlocalsnapshot"/>
            <br/>
            <span class="localsnapshotdates">
              <xsl:value-of select="$strings/tm/last_localsnapshot"/>
            </span>
            <xsl:value-of select="lastlocalsnapshot"/>
          </td>
        </tr>

      </xsl:if>
      
      <xsl:if
        test="localsnapshotcount = 0 and otherlocalsnapshotcount &gt; 0">
        <tr>
          <th colspan="4">
            <xsl:value-of select="$strings/tm/localsnapshots"/>
          </th>
        </tr>
      </xsl:if>

      <xsl:if test="otherlocalsnapshotcount &gt; 0">
        <tr>
          <xsl:if test="localsnapshotcount = 0">
            <xsl:attribute name="class">
              <xsl:text>horizontalborder</xsl:text>
            </xsl:attribute>
          </xsl:if>
          <td colspan="4" class="header">
            <xsl:call-template name="plural">
              <xsl:with-param name="keys" select="$strings/tm"/>
              <xsl:with-param name="key">
                <xsl:text>otherlocalsnapshots</xsl:text>
              </xsl:with-param>
              <xsl:with-param
                name="value" select="otherlocalsnapshotcount"/>
            </xsl:call-template>
          </td>
        </tr>
      </xsl:if>

      <xsl:choose>
        <xsl:when
          test="count(/etrecheck//launchdfile[contains(path, 'bombich')]) &gt; 0">
          <tr>
            <th colspan="4">
              <xsl:value-of select="$strings/tm/otherbackup"/>
            </th>
          </tr>

          <tr class="horizontalborder">
            <td colspan="4" class="header">
              <xsl:value-of select="$strings/tm/other_ccc"/>
            </td>
          </tr>
        </xsl:when>
        <xsl:when
          test="count(/etrecheck//launchdfile[contains(path, 'shirtpocket')]) &gt; 0">
          <tr>
            <th colspan="4">
              <xsl:value-of select="$strings/tm/otherbackup"/>
            </th>
          </tr>

          <tr class="horizontalborder">
            <td colspan="4" class="header">
              <xsl:value-of select="$strings/tm/other_sd"/>
            </td>
          </tr>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>

    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/tm/description/node()"/>
      
      <xsl:choose>
        <xsl:when test="notconfigured = 'true'">
          <xsl:copy-of select="$strings/tm/description_tmno[@version = $prefversion or not(@version)]/node()"/>
        </xsl:when>
        <xsl:when test="autobackup != 'true'">
          <xsl:copy-of select="$strings/tm/description_noauto[@version = $prefversion or not(@version)]/node()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="$strings/tm/description_tmyes/node()"/>
        </xsl:otherwise>
      </xsl:choose>

      <xsl:if
        test="localsnapshotcount &gt; 0 or otherlocalsnapshotcount &gt; 0">
        <xsl:copy-of
          select="$strings/tm/description_localsnapshots/node()"/>
          
        <xsl:if test="$showdeletebutton = 'true'">
          <xsl:copy-of
            select="$strings/tm/description_deletelocalsnapshots/node()"/>
        </xsl:if>
        
        <xsl:if test="otherlocalsnapshotcount &gt; 0">
          <xsl:copy-of
            select="$strings/tm/description_otherlocalsnapshots/node()"/>
        </xsl:if>
      </xsl:if>

    </p>

  </xsl:template>

  <!-- Print a Time Machine volume. -->
  <xsl:template match="volume" mode="timemachine">
  
    <!-- This should always have a bottom border. -->
    <tr class="horizontalborder">
      <td>
        <img>
          <xsl:attribute name="src">
            <xsl:value-of select="concat('image/timemachine/', name_unsafe)"/>
          </xsl:attribute>
          <xsl:attribute name="alt">
            <xsl:value-of select="$strings/volume/icon"/>
          </xsl:attribute>
        </img>
      </td>
      <td class="header">
        <xsl:value-of select="$strings/tm/drive_name"/>
        <br/>
        <xsl:value-of select="$strings/tm/drive_size"/>
        <br/>
        <xsl:value-of select="$strings/tm/space_used"/>
      </td>
      <td>
        <xsl:value-of select="name_unsafe"/>
        <br/>
        <xsl:apply-templates select="size" mode="units"/>
        <br/>
        <xsl:apply-templates select="used" mode="units"/>
      </td>
      <td/>
    </tr>

  </xsl:template>

  <!-- Print a Time Machine destination. -->
  <xsl:template match="destination">
  
    <!-- Get the type. -->
    <xsl:variable name="rawtype" select="type"/>
    
    <!-- Get the localized type. -->
    <xsl:variable name="localizedtype">
      <xsl:value-of
        select="$strings/tm/types/value[@key = $rawtype]"/>
    </xsl:variable>

    <!-- If the localized smart status is empty, just use what I have. -->
    <xsl:variable name="type">
      <xsl:choose>
        <xsl:when test="string-length($localizedtype)">
          <xsl:value-of select="$localizedtype"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$rawtype"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <!-- This should always have a bottom border. -->
    <tr class="horizontalborder">
      <td>
        <xsl:if test="position() = last()">
          <xsl:attribute name="class">
            <xsl:text>bottom_left</xsl:text>
          </xsl:attribute>
        </xsl:if>
        <img>
          <xsl:attribute name="src">
            <xsl:value-of select="concat('image/timemachine/', name_unsafe)"/>
          </xsl:attribute>
          <xsl:attribute name="alt">
            <xsl:value-of select="$strings/volume/icon"/>
          </xsl:attribute>
        </img>
      </td>
      <td colspan="3">
        <span class="destinations">
          <xsl:value-of select="$strings/tm/drive_name"/>
        </span>
        <xsl:value-of select="name_unsafe"/>
        <br/>
        
        <span class="destinations">
          <xsl:value-of select="$strings/tm/type"/>
        </span>
        <xsl:value-of select="$type"/>
        <br/>
        
        <span class="destinations">
          <xsl:value-of select="$strings/tm/last_used"/>
        </span>
        <xsl:apply-templates select="lastused"/>
        
        <xsl:if test="$fulldriveaccess = 'true'">
          <br/>
          <span class="destinations">
            <xsl:value-of select="$strings/tm/total_size"/>
          </span>
          <xsl:apply-templates select="size" mode="units"/>
          <br/>
          
          <span class="destinations">
            <xsl:value-of select="$strings/tm/number_of_backups"/>
          </span>
          <xsl:if test="oldestbackup and lastbackup">
            <br/>
            <span class="destinations">
              <xsl:value-of select="$strings/tm/oldest_backup"/>
            </span>
            <xsl:value-of select="oldestbackup"/>
            <br/>
            
            <span class="destinations">
              <xsl:value-of select="$strings/tm/last_backup"/>
            </span>
            <xsl:value-of select="lastbackup"/>
          </xsl:if>
        </xsl:if>
      </td>
    </tr>

  </xsl:template>

  <!-- USB information. -->
  <xsl:template match="usb" mode="top">
  
    <xsl:if test="count(node//node) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:text>usb</xsl:text>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="usb">
  
    <xsl:for-each select="node">
      <xsl:if test="count(node) &gt; 0">
        <table class="tablelist">
          <tr>
            <th>
              <xsl:value-of select="normalize-space(name)"/>
            </th>
          </tr>
          <xsl:apply-templates select="node"/>
        </table>
      </xsl:if>
    </xsl:for-each>

  </xsl:template>

  <!-- USB node information. -->
  <xsl:template match="node">
    
    <tr class="horizontalborder">
      <td>
        <xsl:if test="manufacturer">
          <xsl:value-of select="normalize-space(manufacturer)"/>
          <xsl:text> - </xsl:text>
        </xsl:if>
        
        <xsl:value-of select="normalize-space(name)"/>
        
        <xsl:if test="count(node) &gt; 0">
          <ul>
            <xsl:apply-templates select="node" mode="chained"/>
          </ul>
        </xsl:if>
      </td>
    </tr>
      
  </xsl:template>

  <!-- USB node information. -->
  <xsl:template match="node" mode="chained">
    
    <li class="chained">
      <xsl:if test="manufacturer">
        <xsl:value-of select="normalize-space(manufacturer)"/>
        <xsl:text> - </xsl:text>
      </xsl:if>
      
      <xsl:value-of select="normalize-space(name)"/>
      
      <xsl:if test="count(node) &gt; 0">
        <ul>
          <xsl:apply-templates select="node" mode="chained"/>
        </ul>
      </xsl:if>
    </li>
      
  </xsl:template>

  <!-- Print a network information. -->
  <xsl:template match="network">
  
    <table class="tablelist">
      <tr>
        <th colspan="4">
          <xsl:value-of select="$strings/network/interfaces"/>
        </th>
      </tr>

      <xsl:for-each select="interfaces/networkinterface">

        <xsl:variable name="interface" select="interface"/>
        <xsl:variable
          name="modes"
          select="/etrecheck/hardware/wireless[name = $interface]"/>
        
        <tr class="horizontalborder">
          <td class="header">
            <xsl:value-of select="$strings/network/interface"/>
          </td>
          <td colspan="3">
            <xsl:value-of select="interface"/>
          </td>
        </tr>

        <tr>
          <td class="header">
            <xsl:value-of select="$strings/network/name"/>
          </td>
          <td colspan="3">
            <xsl:value-of select="name"/>
          </td>
        </tr>
        
        <xsl:if test="count($modes/modes)">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/network/modes"/>
            </td>
            <td colspan="3">
              <xsl:value-of select="$modes/modes"/>
            </td>
          </tr>
        </xsl:if>
        
        <xsl:if test="count(ipv4addresses/address) &gt; 0">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/network/ipv4addresses"/>
            </td>
            <td colspan="3">
              <xsl:for-each select="ipv4addresses/address">
                <xsl:if test="position() &gt; 1">
                  <br/>
                </xsl:if>
                
                <xsl:value-of select="."/>
              </xsl:for-each>
            </td>
          </tr>
        </xsl:if>
        
        <xsl:if test="count(ipv6addresses/address) &gt; 0">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/network/ipv6addresses"/>
            </td>
            <td colspan="3">
              <xsl:for-each select="ipv6addresses/address">
                <xsl:if test="position() &gt; 1">
                  <br/>
                </xsl:if>
                
                <xsl:value-of select="."/>
              </xsl:for-each>
            </td>
          </tr>
        </xsl:if>
        
        <xsl:if test="proxyautodiscovery = 'true'">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/network/proxyautodiscovery"/>
            </td>
            <td colspan="3">
              <xsl:apply-templates select="proxyautodiscovery"/>
            </td>
          </tr>
        </xsl:if>
        
        <xsl:if test="proxyautoconfig = 'true'">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/network/proxyautoconfig"/>
            </td>
            <td colspan="3">
              <xsl:value-of select="proxyautoconfigurl"/>
            </td>
          </tr>
        </xsl:if>

        <xsl:if test="count(proxies/proxy) &gt; 0">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/network/proxies"/>
            </td>
            <td class="proxy">
              <xsl:for-each select="proxies/proxy">
                <xsl:if test="position() &gt; 1">
                  <br/>
                </xsl:if>
                
                <xsl:value-of select="type"/>
                
                <xsl:if test="address">
                  <xsl:text> </xsl:text>
                  
                  <xsl:if test="user">
                    <xsl:value-of select="address"/>
                    <xsl:text>@</xsl:text>
                  </xsl:if>
                  
                  <xsl:value-of select="address"/>
                  
                  <xsl:if test="port">
                    <xsl:text>:</xsl:text>
                    <xsl:value-of select="port"/>
                  </xsl:if>
                </xsl:if>
              </xsl:for-each>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('GET(&quot;',$strings/network/about_proxies,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/network/about_proxies_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/network/about"/>
                <img class="buttonicon" src="image/webbrowserbutton.png"/>
              </button>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET('/openapplication//System/Library/PreferencePanes/Network.prefPane')</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of
                    select="$strings/network/review_proxies_title[@version = $prefversion or not(@version)]"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/network/review"/>
              </button>
            </td>
          </tr>
        </xsl:if>

      </xsl:for-each>

      <xsl:if test="icloudfree or icloudpendingfiles">
        <tr class="horizontalborder">
          <th colspan="4">
            <xsl:value-of select="$strings/network/icloud"/>
          </th>
        </tr>
      </xsl:if>
      
      <xsl:if test="icloudfree">
        <tr class="horizontalborder">
          <td class="header">
            <xsl:value-of select="$strings/network/icloudfree"/>
          </td>
          <td colspan="3">
            <xsl:apply-templates
              select="icloudfree" mode="units"/>
            <xsl:value-of select="$strings/available"/>
          </td>
        </tr>
      </xsl:if>
      
      <xsl:if test="icloudpendingfiles">
        
        <tr class="horizontalborder">
          <td class="header">
            <xsl:value-of select="$strings/network/icloudstatus"/>
          </td>
          <td colspan="3">
            <xsl:call-template name="plural">
              <xsl:with-param name="keys" select="$strings/network"/>
              <xsl:with-param name="key">
                <xsl:text>icloudpendingfiles</xsl:text>
              </xsl:with-param>
              <xsl:with-param name="value" select="icloudpendingfiles"/>
            </xsl:call-template>
          </td>
        </tr>
        
        <xsl:if test="icloudpendingfiles &lt; 20">
          <tr class="horizontalborder">
            <td class="header" colspan="2">
              <xsl:value-of select="$strings/network/pendingname"/>
            </td>
            <td class="header">
              <xsl:value-of select="$strings/network/status"/>
            </td>
            <td/>
          </tr>

          <xsl:for-each select="icloudpendingfilelist//file">
            <xsl:variable name="status" select="status"/>
            
            <tr>
              <td colspan="2"><xsl:value-of select="name_unsafe"/></td>
              <td>
                <xsl:value-of select="$strings/network/status/value[@key = $status]"/>
              </td>
              <td>
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/reveal/',path_unsafe,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of
                      select="$strings/network/reveal_in_finder_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/network/reveal_in_finder"/>
                  <img class="buttonicon" src="image/finderbutton.png"/>
                </button>
              </td>
            </tr>
          </xsl:for-each>
        </xsl:if>
      </xsl:if>
      
    </table>
    
    <xsl:if test="count(sharing/service) &gt; 0">
    
      <table class="tablelist">
        <tr>
          <th colspan="4">
            <xsl:value-of select="$strings/network/sharingservices"/>
          </th>
        </tr>

        <xsl:for-each select="sharing/service">
          <xsl:variable name="service" select="."/>
          <xsl:variable
            name="url"
            select="$strings/network/sharingurl/value[@key = $service]"/>
          <xsl:variable
            name="about"
            select="$strings/network/sharingabout/value[@key = $service]"/>

          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of
                select="$strings/network/sharing/value[@key = $service]"/>
              <xsl:value-of select="$strings/network/colon"/>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('GET(&quot;',$about,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/network/about_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/network/about"/>
                <img class="buttonicon" src="image/webbrowserbutton.png"/>
              </button>
            </td>
            <td>
              <xsl:value-of
                select="$strings/network/sharingon/value[@key = $service]"/>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('GET(&quot;',$url,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of
                    select="$strings/network/review_sharing_title[@version = $prefversion or not(@version)]"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/network/review"/>
              </button>
            </td>
          </tr>
        </xsl:for-each>
      </table>
    </xsl:if>

    <p class="description">
      <xsl:copy-of select="$strings/network/description/node()"/>

      <xsl:if test="icloudfree">
        <xsl:copy-of select="$strings/network/description_icloud/node()"/>
      </xsl:if>

      <xsl:if test="icloudpendingfiles">
        <xsl:copy-of select="$strings/network/description_icloudpendingfiles/node()"/>
      </xsl:if>
      
      <xsl:if test="icloudpendingfiles &gt;= 20">
        <xsl:copy-of select="$strings/network/description_icloudpendingfilescount/node()"/>
      </xsl:if>

      <xsl:choose>
        <xsl:when test="count(interfaces/networkinterface[proxyautodiscovery = 'true']) &gt; 0">
          <xsl:copy-of select="$strings/network/description_proxy[@version = $prefversion]/node()"/>
        </xsl:when>
        <xsl:when test="count(interfaces/networkinterface[proxyautoconfig = 'true']) &gt; 0">
          <xsl:copy-of select="$strings/network/description_proxy[@version = $prefversion]/node()"/>
        </xsl:when>
        <xsl:when test="count(interfaces//proxy) &gt; 0">
          <xsl:copy-of select="$strings/network/description_proxy[@version = $prefversion]/node()"/>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>

      <xsl:if test="count(sharing/service) &gt; 0">
        <xsl:copy-of select="$strings/security/description_sharing[@version = $prefversion]/node()"/>
      </xsl:if>
    </p>

  </xsl:template>
  
  <!-- Top processes by network use. -->
  <xsl:template match="networkusage">
  
    <xsl:variable name="thirdparty">
      <xsl:for-each select="process">
        <xsl:variable name="path_unsafe" select="path_unsafe"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when
              test="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source">
              <xsl:value-of
                select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        
        <xsl:choose>
          <xsl:when test="xcode = 'true'"/>
          <xsl:when test="other = 'true'"/>
          <xsl:when test="string-length($path_unsafe) &gt; 0 and not(starts-with($source, 'Apple'))">
            <xsl:text>1</xsl:text>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>
      </xsl:for-each>
    </xsl:variable>

    <table>
      <tr>
        <th colspan="2">
          <xsl:value-of select="$strings/networkprocesses/process"/>
        </th>
        <th><xsl:value-of select="$strings/processes/source"/></th>
        <th><xsl:value-of select="$strings/networkprocesses/input"/></th>
        <th colspan="2"><xsl:value-of select="$strings/networkprocesses/output"/></th>
      </tr>
      <xsl:for-each select="process">
        <xsl:variable name="path_unsafe" select="path_unsafe"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when
              test="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source">
              <xsl:value-of
                select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:variable
          name="identifier"
          select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/identifier"/>

        <xsl:variable name="iconname">
          <xsl:call-template name="processicon">
            <xsl:with-param name="identifier">
              <xsl:value-of select="$identifier"/>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:variable>

        <tr>
          <td>
            <xsl:choose>
              <xsl:when test="other = 'true'">
                <img class="appicon" src="image/app/nil">
                  <xsl:attribute name="alt">
                    <xsl:value-of select="$strings/cpu/app_icon"/>
                  </xsl:attribute>
                </img>
              </xsl:when>
              <xsl:otherwise>
                <img class="appicon">
                  <xsl:attribute name="src">
                    <xsl:value-of
                      select="concat('image/process/', $iconname)"/>
                  </xsl:attribute>
                  <xsl:attribute name="alt">
                    <xsl:value-of
                      select="$strings/networkprocesses/app_icon"/>
                  </xsl:attribute>
                </img>
              </xsl:otherwise>
            </xsl:choose>
          </td>
          <td>
            <span class="process">
              <xsl:value-of select="name"/>
              
              <xsl:if test="other = 'true'">
                <xsl:value-of select="$strings/networkprocesses/other"/>
              </xsl:if>
            </span>
          </td>
          
          <td><xsl:value-of select="$source"/></td>
          
          <td class="size">
            <xsl:apply-templates select="inputsize" mode="units"/>
          </td>
          <td class="size">
            <xsl:apply-templates select="outputsize" mode="units"/>
          </td>
          <td>
            <xsl:choose>
              <xsl:when test="translocated = 'true'">
                <xsl:value-of select="$strings/processes/translocated"/>
              </xsl:when>
              <xsl:when test="xcode = 'true'">
                <xsl:value-of select="$strings/processes/xcode"/>
              </xsl:when>
              <xsl:when test="string-length(location_unsafe) &gt; 0 and not(starts-with($source, 'Apple'))">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/reveal/',location_unsafe,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/processes/reveal_in_finder_title"/>
                  </xsl:attribute>
                  <xsl:value-of
                    select="$strings/processes/reveal_in_finder"/>
                  <img class="buttonicon" src="image/finderbutton.png"/>
                </button>
              </xsl:when>
              <xsl:when test="other = 'true'">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/openapplication//Applications/Utilities/Activity Monitor.app')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of
                      select="$strings/processes/activitymonitor_title"/>
                  </xsl:attribute>
                  <xsl:value-of
                    select="$strings/processes/activitymonitor"/>
                  <img class="buttonicon"
                    src="image/activitymonitorbutton.png"/>
                </button>
              </xsl:when>
              <xsl:when test="man">
                <!-- Provide a button to an Apple man page. -->
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/man/',man,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/processes/about_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/processes/about"/>
                  <img class="buttonicon" src="image/man.png"/>
                </button>
              </xsl:when>
              <xsl:otherwise/>
            </xsl:choose>
          </td>
        </tr>
      </xsl:for-each>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/networkprocesses/description/node()"/>

      <xsl:if test="count(process/man) &gt; 0">
        <xsl:copy-of select="$strings/processes/description_about/node()"/>
      </xsl:if>

      <xsl:if test="count(process[other = 'true']) &gt; 0">
        <xsl:copy-of select="$strings/processes/description_other/node()"/>
      </xsl:if>

      <xsl:if test="string-length($thirdparty) &gt; 0">
        <xsl:copy-of
          select="$strings/processes/description_thirdparty/node()"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- Configuration profile information. -->
  <xsl:template match="configurationprofiles" mode="top">
  
    <xsl:if
      test="count(configurationprofile//setting) &gt; 0 or hasconfigurationprofiles">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:text>configurationprofiles</xsl:text>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <div class="control">
            <xsl:if test="count(configurationprofile//setting) &gt; 0">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET('/openapplication//System/Library/PreferencePanes/Profiles.prefPane')</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of
                    select="$strings/profiles/open_profiles_title[@version = $prefversion]"/>
                </xsl:attribute>
                <xsl:value-of
                  select="$strings/profiles/open_profiles"/>
                <img class="buttonicon" src="image/profilesprefpanebutton.png"/>
              </button>
            </xsl:if>
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/link/https://support.apple.com/guide/mac-help/use-configuration-profiles-mh35561')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of
                  select="$strings/profiles/about_profiles_title"/>
              </xsl:attribute>
              <xsl:value-of
                select="$strings/profiles/about_profiles"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </div>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <!-- Configuration profile information. -->
  <xsl:template match="configurationprofiles">
  
    <xsl:choose>
      <xsl:when test="count(configurationprofile//setting) &gt; 0">
        <xsl:for-each select="configurationprofile">
          <table class="tablelist">
            <tr>
              <th>
                <xsl:value-of select="displayname"/>
                <xsl:if test="developer">
                  <br/>
                  <xsl:value-of select="developer"/>
                </xsl:if>
              </th>
            </tr>
            
            <xsl:for-each select="settings/type">
              
              <tr class="horizontalborder">
                <td class="header">
                  <xsl:value-of select="$strings/profiles/type"/>
                  <xsl:value-of select="name"/>
                </td>
              </tr>
              
              <xsl:for-each select="setting">
                <tr>
                  <td>
                    <xsl:value-of select="key"/>
                    <xsl:text> = </xsl:text>
                    <xsl:value-of select="value"/>
                  </td>
                </tr>
              </xsl:for-each>
            </xsl:for-each>
          </table>
        </xsl:for-each>
      </xsl:when>
      <xsl:when test="hasconfigurationprofiles">
        <table class="tablelist">
          <tr>
            <td class="header">
              <xsl:value-of
                select="$strings/profiles/hasconfigurationprofiles"/>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET('/openapplication//System/Library/PreferencePanes/Profiles.prefPane')</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of
                    select="$strings/profiles/open_profiles_title[@version = $prefversion]"/>
                </xsl:attribute>
                <xsl:value-of
                  select="$strings/profiles/open_profiles"/>
                <img class="buttonicon" src="image/profilesprefpanebutton.png"/>
              </button>
            </td>
          </tr>
        </table>
      </xsl:when>
      <xsl:otherwise/>
    </xsl:choose>
    
    <p class="description">
      <xsl:copy-of select="$strings/profiles/description[@version = $prefversion]/node()"/>
      
      <xsl:if test="/etrecheck/header/apptype = 'mas'">
        <xsl:copy-of select="$strings/profiles/mas_description/node()"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- Chrome defaults information. -->
  <xsl:template name="chromedefaults">

    <xsl:variable
      name="chromedefaultscount"
      select="count(/etrecheck/configurationprofiles/chrome/*)"/>
    
    <xsl:if test="$chromedefaultscount &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">chromedefaults</xsl:with-param>
        <xsl:with-param name="contents">
        
          <table class="tablelist">
          
            <xsl:apply-templates
              select="/etrecheck/configurationprofiles/chrome"/>

          </table>
          
          <p class="description">
            <xsl:copy-of
              select="$strings/profiles/description_chromedefaults/node()"/>
          </p>

        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/fix/chromedefaults')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of
                select="$strings/profiles/fix_chromedefaults_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/profiles/fix_chromedefaults"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>

  </xsl:template>

  <!-- Print a chrome setting. -->
  <xsl:template match="chrome/setting">
  
    <xsl:variable name="last_row">
      <xsl:choose>
        <xsl:when test="position() = last()">
          <xsl:text>true</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>false</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="setting">
      <xsl:value-of select="name"/>
    </xsl:variable>
    
    <tr class="horizontalborder">
      <td class="header">
        <xsl:value-of
          select="$strings/profiles/chromedefaults/value[@key = $setting]"/>
      </td>
      
      <td>
        <xsl:value-of select="value"/>
      </td>
    </tr>
    
  </xsl:template>

  <!-- Notification information. -->
  <xsl:template name="notifications">

    <xsl:variable
      name="webnotificationcount"
      select="count(/etrecheck/adware/notifications//notification[identifier_unsafe = 'com.apple.Safari'])"/>
    
    <xsl:variable
      name="appnotificationcount"
      select="count(/etrecheck/adware/notifications//notification)"/>
    
    <xsl:variable
      name="notificationcount"
      select="$webnotificationcount + $appnotificationcount"/>

    <xsl:variable name="try">
      <xsl:choose>
        <xsl:when
          test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$notificationcount &gt; 0"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$try = 'true'">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">notifications</xsl:with-param>
        <xsl:with-param name="contents">
        
          <table class="tablelist">
            <xsl:choose>
              <xsl:when test="$notificationcount = 0">
            
                <tr>
                  <td class="header">
                    <xsl:value-of
                      select="$strings/notifications/notificationsunavailable"/>
                  </td>
                  <td>
                    <button class="mas">
                      <xsl:attribute name="onclick">
                        <xsl:text>GET('/help/fulldriveaccess')</xsl:text>
                      </xsl:attribute>
                      <xsl:attribute name="title">
                        <xsl:value-of
                          select="$strings/notifications/fulldriveaccess_title"/>
                      </xsl:attribute>
                      <xsl:value-of
                        select="$strings/notifications/fulldriveaccess"/>
                    </button>
                  </td>
                </tr>

              </xsl:when>

              <xsl:otherwise>
            
                <xsl:apply-templates
                  select="/etrecheck/adware/notifications/notification_app"/>
              
              </xsl:otherwise>
            </xsl:choose>
          </table>
          
          <p class="description">
            <xsl:choose>
              <xsl:when test="$notificationcount = 0">
                <xsl:copy-of
                  select="$strings/notifications/description_notifications_fulldiskaccess/node()"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:copy-of
                  select="$strings/notifications/description_notifications[@version = $prefversion]/node()"/>
                
                <xsl:if test="$webnotificationcount &gt; 0">
                  <xsl:copy-of
                    select="$strings/notifications/description_notifications_web[@version = $prefversion]/node()"/>
                </xsl:if>
              </xsl:otherwise>
            </xsl:choose>
          </p>

        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/openapplication//System/Library/PreferencePanes/Notifications.prefPane')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of
                select="$strings/notifications/open_notifications_title[@version = $prefversion]"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/notifications/open_notifications"/>
            <img
              class="buttonicon"
              src="image/notificationsprefpanebutton.png"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    
  </xsl:template>

  <!-- Print a notification app. -->
  <xsl:template match="notification_app">
  
    <xsl:variable name="appname">
      <xsl:choose>
        <xsl:when test="translocated = 'true'">
          <xsl:value-of select="name"/>
        </xsl:when>
        <xsl:when test="xcode = 'true'">
          <xsl:value-of select="name"/>
        </xsl:when>
        <xsl:when test="name">
          <xsl:value-of select="name"/>
        </xsl:when>
        <xsl:when test="path">
          <xsl:value-of select="path"/>
        </xsl:when>
        <xsl:when test="identifier_unsafe">
          <xsl:value-of select="identifier_unsafe"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="notification[position() = 1]/title_unsafe"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="apppath">
      <xsl:choose>
        <xsl:when test="path_unsafe">
          <xsl:value-of select="path_unsafe"/>
        </xsl:when>
        <xsl:when test="identifier_unsafe">
          <xsl:value-of select="identifier_unsafe"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>Unknown</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="iconname">
      <xsl:call-template name="appicon">
        <xsl:with-param name="path">
          <xsl:value-of select="path_unsafe"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:variable name="last_row">
      <xsl:choose>
        <xsl:when test="position() = last()">
          <xsl:text>true</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>false</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <tr class="horizontalborder notification_app">
      <td colspan="2">
        <xsl:choose>
          <xsl:when test="string-length($iconname) &gt; 0">
            <img class="appicon">
              <xsl:attribute name="src">
                <xsl:value-of
                  select="concat('image/app/', $iconname)"/>
              </xsl:attribute>
              <xsl:attribute name="alt">
                <xsl:value-of select="$strings/applications/app_icon"/>
              </xsl:attribute>
            </img>
            <span class="app">
              <xsl:value-of select="$appname"/>
            </span>
          </xsl:when>
          <xsl:otherwise>
            <span class="app">
              <xsl:value-of select="$appname"/>
            </span>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:choose>
          <xsl:when test="identifier_unsafe = 'com.apple.Safari'">
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/openapplication//Applications/Safari.app')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of
                  select="$strings/safari/open_safari_title[@version = $prefversion]"/>
              </xsl:attribute>
              <xsl:value-of
                select="$strings/safari/open_safari"/>
              <img class="buttonicon" src="image/safaributton.png"/>
            </button>
          </xsl:when>
          <xsl:otherwise>
            <xsl:if test="string-length($iconname) &gt; 0">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('GET(&quot;/reveal/',$apppath,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of
                    select="$strings/applications/reveal_in_finder_title"/>
                </xsl:attribute>
                <xsl:value-of
                  select="$strings/applications/reveal_in_finder"/>
                <img class="buttonicon" src="image/finderbutton.png"/>
              </button>
            </xsl:if>
          </xsl:otherwise>
        </xsl:choose>
      </td>
    </tr>
    
    <tr>
      <xsl:attribute name="class">
        <xsl:text>horizontalborder</xsl:text>
        <xsl:if test="position() = last()">
          <xsl:text> last_child</xsl:text>
        </xsl:if>
      </xsl:attribute>
      <th class="notification">
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/notifications"/>
          <xsl:with-param name="key" select="'notifications'"/>
          <xsl:with-param name="value" select="count(notification)"/>
        </xsl:call-template>
      </th>
      <th>
        <div class="buttonwrapper">
          <button class="show_notifications">
            <xsl:attribute name="onclick">
              <xsl:value-of
                select="concat('showNotifications(this, &quot;', identifier, '&quot;,', $last_row, ')')"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/notifications/show_notifications"/>
          </button>
          <button class="hide_notifications">
            <xsl:attribute name="onclick">
              <xsl:value-of
                select="concat('hideNotifications(this, &quot;', identifier, '&quot;,', $last_row, ')')"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/notifications/hide_notifications"/>
          </button>
        </div>
      </th>
    </tr>

    <xsl:apply-templates select="notification"/>
    
  </xsl:template>
  
  <!-- Print a notification. -->
  <xsl:template match="notification">

    <xsl:variable name="iconname">
      <xsl:call-template name="appicon">
        <xsl:with-param name="path">
          <xsl:value-of select="../path_unsafe"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>

    <tr>
      <xsl:attribute name="class">
        <xsl:text>notification_row </xsl:text>
        <xsl:value-of select="identifier"/>
        <xsl:if test="position() = 1">
          <xsl:text> horizontalborder</xsl:text>
        </xsl:if>
      </xsl:attribute>

      <xsl:attribute name="id">
        <xsl:value-of select="concat('notification_', rec_id)"/>
      </xsl:attribute>
      
      <td class="notification">
        <div class="notification">
          <xsl:if test="string-length($iconname) &gt; 0">
            <img class="icon">
              <xsl:attribute name="src">
                <xsl:value-of
                  select="concat('image/app/', $iconname)"/>
              </xsl:attribute>
              <xsl:attribute name="alt">
                <xsl:value-of select="$strings/applications/app_icon"/>
              </xsl:attribute>
            </img>
          </xsl:if>
          <p class="title"><xsl:value-of select="title_unsafe"/></p>
        
          <xsl:if test="subtitle_unsafe">
            <p class="subtitle">
              <xsl:value-of
                disable-output-escaping="yes" select="subtitle_unsafe"/>
            </p>
          </xsl:if>
        
          <p class="body">
            <xsl:value-of
              disable-output-escaping="yes" select="body_unsafe"/>
          </p>
        </div>
      </td>
      
      <td class="date">
        <xsl:choose>
          <xsl:when test="status = 'delivered'">
            <xsl:value-of select="$strings/notifications/delivered"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$strings/notifications/scheduled"/>
          </xsl:otherwise>
        </xsl:choose>
        <br/>
        <xsl:value-of select="date"/>
      </td>
    </tr>
    
  </xsl:template>
  
  <!-- Security information. -->
  <xsl:template match="security">
  
    <xsl:variable name="gatekeeperstatus" select="gatekeeper/status"/>

    <xsl:variable name="gatekeeperstatusvalue">
      <xsl:choose>
        <xsl:when test="$strings/security/gatekeeper_setting/value[@key = $gatekeeperstatus]">
          <xsl:value-of select="$strings/security/gatekeeper_setting/value[@key = $gatekeeperstatus]"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$gatekeeperstatus"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <table>
      <tr>
        <th><xsl:value-of select="$strings/security/system"/></th>
        <th/>
        <th>
          <xsl:value-of select="$strings/security/status"/>
        </th>
        <th/>
      </tr>
      <tr>
        <td>
          <xsl:value-of select="$strings/security/gatekeeper"/>
        </td>
        <td>
          <!-- Provide a button to Apple information about gatekeeper. -->
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/link/https://support.apple.com/HT202491')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of
                select="$strings/security/about_gatekeeper_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/security/about_gatekeeper"/>
            <img class="buttonicon" src="image/webbrowserbutton.png"/>
          </button>
        </td>
        <td>
          <xsl:value-of select="$gatekeeperstatusvalue"/>
        </td>
        <td>
          <xsl:if test="gatekeeper/status = 'Anywhere'">
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/fix/gatekeeper')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of
                  select="$strings/security/fix_gatekeeper_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/security/fix_gatekeeper"/>
              <img class="buttonicon" src="image/systempreferencesbutton.png"/>
            </button>
          </xsl:if>
        </td>
      </tr>
      <xsl:if test="/etrecheck/systemsoftware/majorversion &gt;= 11">
        <tr>
          <td>
            <xsl:value-of select="$strings/security/sip"/>
          </td>
          <td>
            <!-- Provide a button to Apple information about SIP. -->
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/link/https://support.apple.com/HT204899')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/security/about_sip_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/security/about_sip"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </td>
          <td>
            <xsl:choose>
              <xsl:when test="SIP = 'enabled'">
                <xsl:value-of select="$strings/security/enabled"/>
              </xsl:when>
              <xsl:when test="SIP = 'disabled'">
                <xsl:value-of select="$strings/security/disabled"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="SIP"/>
              </xsl:otherwise>
            </xsl:choose>
          </td>
          <td>
            <xsl:if test="SIP != 'enabled'">
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET('/fix/SIP')</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/security/fix_sip_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/security/fix_sip"/>
                <img class="buttonicon" src="image/webbrowserbutton.png"/>
              </button>
            </xsl:if>
          </td>
        </tr>
      </xsl:if>

      <xsl:if test="secureboot">
        <xsl:variable name="secureboot">
          <xsl:value-of select="secureboot"/>
        </xsl:variable>
        
        <tr>
          <td><xsl:value-of select="$strings/security/secureboot"/></td>
          <td>
            <!-- Provide a button to Apple information about secure boot. -->
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/link/https://support.apple.com/102522')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/security/about_secureboot_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/security/about_secureboot"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </td>
          <td>
            <xsl:value-of
              select="$strings/security/secureboot_text/value[@key = $secureboot]"/>
          </td>
          <td/>
        </tr>
      </xsl:if>

      <xsl:if test="lockdownmode = 'true'">
        <tr>
          <td><xsl:value-of select="$strings/security/lockdownmode"/></td>
          <td>
            <!-- Provide a button to Apple information about lockdown mode. -->
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/link/https://support.apple.com/105120')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/security/about_lockdownmode_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/security/about_lockdownmode"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </td>
          <td>
            <xsl:value-of select="$strings/security/enabled"/>
          </td>
          <td/>
        </tr>
      </xsl:if>

      <xsl:if test="softwareupdate/automaticupdatesdisabled = 'true'">
        <tr>
          <td>
            <xsl:value-of select="$strings/security/automaticupdates"/>
          </td>
          <td>
            <!-- Provide a button to Apple information about software update. -->
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/link/https://support.apple.com/HT207005')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/security/about_updates_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/security/about_updates"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </td>
          <td>
            <xsl:value-of select="$strings/security/disabled"/>
          </td>
          <td>
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/fix/updates')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/security/fix_updates_title[@version = $prefversion]"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/security/fix_updates"/>
              <img class="buttonicon" src="image/systempreferencesbutton.png"/>
            </button>
          </td>
        </tr>
      </xsl:if>

      <xsl:if test="softwareupdate/criticalupdatesdisabled = 'true'">
        <tr>
          <td>
            <xsl:value-of select="$strings/security/criticalupdates"/>
          </td>
          <td>
            <!-- Provide a button to Apple information about software update. -->
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/link/https://support.apple.com/HT207005')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/security/about_updates_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/security/about_updates"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </td>
          <td>
            <xsl:value-of select="$strings/security/disabled"/>
          </td>
          <td>
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/fix/updates')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/security/fix_updates_title[@version = $prefversion]"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/security/fix_updates"/>
              <img class="buttonicon" src="image/systempreferencesbutton.png"/>
            </button>
          </td>
        </tr>
      </xsl:if>

      <xsl:if test="count(/etrecheck/adware//avtype) &gt; 0 or /etrecheck/security/appleav = 'false'">
        <xsl:variable
          name="count" select="count(/etrecheck/adware//avtype)"/>
    
        <tr class="avapps">
          <th><xsl:value-of select="$strings/security/avapps"/></th>
          <td/>
          <td colspan="2" class="securitystatus">
            <xsl:choose>
              <xsl:when test="count(/etrecheck/adware//avtype) &gt; 0">
                <xsl:for-each select="/etrecheck/adware//avtype">

                  <xsl:variable name="name" select="."/>
                  
                  <xsl:if test="position() &gt; 1 and $count &gt; 2">
                    <xsl:text>,</xsl:text>
                  </xsl:if>

                  <xsl:choose>
                    <xsl:when test="position() = $count and $count &gt; 1">
                      <xsl:value-of select="$strings/adware/and"/>
                    </xsl:when>
                    <xsl:when test="position() = 1"/>
                    <xsl:otherwise>
                      <xsl:text> </xsl:text>
                    </xsl:otherwise>
                  </xsl:choose>

                  <xsl:value-of select="$strings/av/value[@key = $name]"/>
                  
                </xsl:for-each>
              </xsl:when>
              <xsl:when test="/etrecheck/security/appleav = 'false'">
                <strong><xsl:value-of select="$strings/security/noavapps"/></strong>
              </xsl:when>
              <xsl:otherwise/>
            </xsl:choose>
          </td>
        </tr>
      </xsl:if>

      <xsl:if test="count(/etrecheck/network//proxies/proxy) &gt; 0">
        <tr class="proxies">
          <th><xsl:value-of select="$strings/security/proxies"/></th>
          <td>
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('GET(&quot;',$strings/network/about_proxies,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/network/about_proxies_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/network/about"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </td>
          <th/>
          <td>
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/openapplication//System/Library/PreferencePanes/Network.prefPane')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of
                  select="$strings/network/review_proxies_title[@version = $prefversion or not(@version)]"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/network/review"/>
            </button>
          </td>
        </tr>

        <xsl:for-each select="/etrecheck/network//proxies/proxy">
          <tr>
            <td>
              <xsl:value-of select="../../interface"/>
              <xsl:text>:</xsl:text>
              <xsl:value-of select="../../name"/>
            </td>
            <td/>
            <td colspan="2">
              <xsl:value-of select="type"/>
              <xsl:text> </xsl:text>
              <xsl:if test="address">
                <xsl:if test="user">
                  <xsl:value-of select="address"/>
                  <xsl:text>@</xsl:text>
                </xsl:if>
                
                <xsl:value-of select="address"/>
                
                <xsl:if test="port">
                  <xsl:text>:</xsl:text>
                  <xsl:value-of select="port"/>
                </xsl:if>
              </xsl:if>
            </td>
          </tr>
        </xsl:for-each>
      </xsl:if>

      <xsl:if test="/etrecheck/header/flashinstalled = 'true'">

        <tr class="securityrisks">
          <th><xsl:value-of select="$strings/security/securityrisks"/></th>
          <th/>
          <th/>
          <th/>
        </tr>

        <tr class="securityrisks">
          <td>Adobe Flash Player</td>
          <td>
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('GET(&quot;/link/',$strings/security/about_flash_url,'&quot;)')"/>
               </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/security/about_flash_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/security/about_flash"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </td>
          <td/>
          <td>
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('GET(&quot;/link/',$strings/security/uninstall_flash_url,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/security/uninstall_flash_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/security/uninstall_flash"/>
              <img class="buttonicon" src="image/webbrowserbutton.png"/>
            </button>
          </td>
        </tr>
      </xsl:if>

      <xsl:if test="count(/etrecheck/network/sharing/service) &gt; 0">
      
        <tr class="sharingservices">
          <th><xsl:value-of select="$strings/network/sharingservices"/></th>
          <th/>
          <th/>
          <th/>
        </tr>

        <xsl:for-each select="/etrecheck/network/sharing/service">
          <xsl:variable name="service" select="."/>
          <xsl:variable
            name="url"
            select="$strings/network/sharingurl/value[@key = $service]"/>
          <xsl:variable
            name="about"
            select="$strings/network/sharingabout/value[@key = $service]"/>
          
          <tr>
            <td>
              <xsl:value-of
                select="$strings/network/sharing/value[@key = $service]"/>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('GET(&quot;',$about,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/network/about_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/network/about"/>
                <img class="buttonicon" src="image/webbrowserbutton.png"/>
              </button>
            </td>
            <td>
              <xsl:value-of
                select="$strings/network/sharingon/value[@key = $service]"/>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('GET(&quot;',$url,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of
                    select="$strings/network/review_sharing_title[@version = $prefversion or not(@version)]"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/network/review"/>
              </button>
            </td>
          </tr>
        </xsl:for-each>
      </xsl:if>
    </table>

    <p class="description">
      <xsl:copy-of select="$strings/security/description/node()"/>
      
      <xsl:choose>
        <xsl:when test="/etrecheck/header/apptype = 'mas'">
          <xsl:copy-of
            select="$strings/security/description_gatekeeper_mas/node()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of
            select="$strings/security/description_gatekeeper/node()"/>
        </xsl:otherwise>
      </xsl:choose>

      <xsl:if test="gatekeeper/status = 'Anywhere'">
        <xsl:copy-of
          select="$strings/security/description_gatekeeper_fix[@version = $prefversion or not(@version)]/node()"/>
      </xsl:if>

      <xsl:if test="/etrecheck/systemsoftware/majorversion &gt;= 11">
        <xsl:copy-of select="$strings/security/description_sip/node()"/>
        <xsl:if test="SIP != 'enabled'">
          <xsl:copy-of select="$strings/security/description_sip_fix/node()"/>
        </xsl:if>
      </xsl:if>
      
      <xsl:if test="softwareupdate/automaticupdatesdisabled = 'true'">
        <xsl:copy-of select="$strings/security/description_automaticupdates/node()"/>
        <xsl:copy-of select="$strings/security/description_automaticupdates_fix[@version = $prefversion or not(@version)]/node()"/>
      </xsl:if>

      <xsl:if test="softwareupdate/criticalupdatesdisabled = 'true'">
        <xsl:copy-of select="$strings/security/description_criticalupdates/node()"/>
        <xsl:copy-of select="$strings/security/description_criticalupdates_fix[@version = $prefversion]/node()"/>
      </xsl:if>
      
      <xsl:copy-of select="$strings/security/description_avapps/node()"/>
      
      <xsl:if test="/etrecheck/adware/avcount3rdparty &gt; 0">
        <xsl:copy-of select="$strings/security/description_multipleav/node()"/>
      </xsl:if>

      <xsl:if test="count(/etrecheck/network//proxies/proxy) &gt; 0">
        <xsl:copy-of select="$strings/security/description_proxy[@version = $prefversion]/node()"/>
      </xsl:if>

      <xsl:if test="/etrecheck/header/flashinstalled = 'true'">
        <xsl:copy-of select="$strings/security/description_flash/node()"/>
      </xsl:if>
      
      <xsl:if test="count(/etrecheck/network/sharing/service) &gt; 0">
        <xsl:copy-of select="$strings/security/description_sharing[@version = $prefversion]/node()"/>
      </xsl:if>
    </p>
    
  </xsl:template>

  <!-- Adware information. -->
  <xsl:template match="adware" mode="top">

    <xsl:choose>
      <xsl:when test="count(launchdfiles/launchdfile) &gt; 0 or count(safariextensions/extension) &gt; 0 or count(loginitems/loginitem) &gt; 0">
        <xsl:call-template name="section-wrapper">
          <xsl:with-param name="name">
            <xsl:text>adware</xsl:text>
          </xsl:with-param>
          <xsl:with-param name="contents">
            <xsl:apply-templates select="."/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise/>
    </xsl:choose>

  </xsl:template>

  <xsl:template match="adware">

    <table class="tablelist">
      <xsl:for-each select="launchdfiles/launchdfile">

        <xsl:variable name="adware" select="adware"/>

        <tbody>
          <xsl:attribute name="class">
            <xsl:value-of select="identifier"/>
            <xsl:text> group</xsl:text>
          </xsl:attribute>
          
          <tr>
            <td class="data"><xsl:value-of select="path_unsafe"/></td>
            <td class="control" colspan="2">
              <button class="review">
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('GET(&quot;/review/launchd/',identifier,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/review/review_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/review/review"/>
              </button>
            </td>
          </tr>
        </tbody>
      </xsl:for-each>

      <xsl:for-each select="safariextensions/extension">

        <xsl:variable name="adware" select="adware"/>

        <tbody>
          <xsl:attribute name="class">
            <xsl:value-of select="identifier"/>
            <xsl:text> group</xsl:text>
          </xsl:attribute>
          
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of select="$strings/adware/type"/>
            </td>
            <td><xsl:value-of select="$strings/adware/safariextension"/></td>
            <td>
              <button class="remove">
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('GET(&quot;/safari/removeadware/',identifier,'&quot;)')"/>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/adware/remove_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/adware/remove"/>
                <img class="buttonicon" src="image/trashbutton.png"/>
              </button>
            </td>
          </tr>
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/adware/file"/>
            </td>
            <td colspan="2"><xsl:value-of select="displayname"/></td>
          </tr>
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/adware/reason"/>
            </td>
            <td colspan="2">
              <xsl:value-of
                select="$strings/adware/reason_text/value[@key = $adware]"/>
            </td>
          </tr>
        </tbody>
      </xsl:for-each>

      <xsl:for-each select="loginitems/loginitem">

        <xsl:variable name="adware" select="adware"/>

        <tbody>
          <xsl:attribute name="class">
            <xsl:value-of select="identifier"/>
            <xsl:text> group</xsl:text>
          </xsl:attribute>
          
          <xsl:apply-templates select="."/>
          
        </tbody>
      </xsl:for-each>

    </table>
    
    <p class="description">
      <xsl:choose>
        <xsl:when test="/etrecheck/header/apptype = 'mas'">
          <xsl:copy-of select="$strings/adware/description_mas/node()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy-of select="$strings/adware/description/node()"/>
        </xsl:otherwise>
      </xsl:choose>

      <xsl:if test="count(loginitems/loginitem) &gt; 0">
        <xsl:apply-templates
          select="$strings/adware/description_loginitems" mode="copy"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- Unsigned files information. -->
  <xsl:template match="unsigned" mode="top">

    <xsl:variable
      name="launchdfilecount"
      select="count(launchdfiles/launchdfile[apple != 'true'])"/>
      
    <xsl:variable
      name="launchdtaskcount"
      select="count(launchdtasks/launchdtask[apple != 'true'])"/>

    <xsl:variable
      name="loginitemcount" select="count(loginitems/loginitem)"/>

    <xsl:variable name="appcount" select="count(applications/application)"/>

    <xsl:variable name="print">
      <xsl:choose>
        <xsl:when test="$hideunsignedfiles ='true'">
          <xsl:value-of select="$launchdtaskcount &gt; 0 or $appcount &gt; 0"/>
        </xsl:when>
        <xsl:when test="$launchdfilecount &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when test="$launchdtaskcount &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when test="$loginitemcount &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when test="$appcount &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:choose>
      <xsl:when test="$print = 'true' and $hideunsignedfiles = 'true'">
        <xsl:call-template name="section-wrapper">
          <xsl:with-param name="name">
            <xsl:value-of select="local-name()"/>
          </xsl:with-param>
          <xsl:with-param name="contents">
            <table class="tablelist">
              <tr>
                <td>
                  <strong>
                    <xsl:value-of select="$strings/unsigned/hidden"/>
                  </strong>
                  <button class="show_unsigned_files">
                    <xsl:attribute name="onclick">
                      <xsl:text>showUnsignedFiles()</xsl:text>
                    </xsl:attribute>
                    <xsl:attribute name="title">
                      <xsl:value-of select="$strings/unsigned/show_title[@version = $prefversion]"/>
                    </xsl:attribute>
                    <xsl:value-of select="$strings/unsigned/show"/>
                  </button>
                </td>
              </tr>
              <xsl:if test="$launchdtaskcount &gt; 0">
                <tr class="horizontalborder">
                  <td>
                    <xsl:call-template name="plural">
                      <xsl:with-param name="keys" select="$strings/unsigned"/>
                      <xsl:with-param name="key" select="'unsignedtasks'"/>
                      <xsl:with-param name="value" select="$launchdtaskcount"/>
                    </xsl:call-template>
                  </td>
                </tr>
              </xsl:if>
              <xsl:if test="$appcount &gt; 0">
                <tr>
                  <xsl:if test="$launchdtaskcount = 0">
                    <xsl:attribute name="class">
                      <xsl:text>horizontalborder</xsl:text>
                    </xsl:attribute>
                  </xsl:if>
                  <td>
                    <xsl:call-template name="plural">
                      <xsl:with-param name="keys" select="$strings/unsigned"/>
                      <xsl:with-param name="key" select="'unsignedapps'"/>
                      <xsl:with-param name="value" select="$appcount"/>
                    </xsl:call-template>
                  </td>
                </tr>
              </xsl:if>
            </table>
            
            <p class="description">
              <xsl:copy-of
                 select="$strings/unsigned/description_hidden/node()"/>
            </p>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:when>
      <xsl:when test="$print != 'true'"/>
      <xsl:otherwise>
        <xsl:call-template name="section-wrapper">
          <xsl:with-param name="name">
            <xsl:value-of select="local-name()"/>
          </xsl:with-param>
          <xsl:with-param name="contents">
            <xsl:apply-templates select="."/>
          </xsl:with-param>
          <xsl:with-param name="control">
            <button class="hide_unsigned_files">
              <xsl:attribute name="onclick">
                <xsl:text>hideUnsignedFiles()</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/unsigned/hide_title[@version = $prefversion]"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/unsigned/hide"/>
            </button>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
    
  </xsl:template>

  <!-- Print unsigned files. -->
  <xsl:template match="unsigned">
  
    <xsl:if
      test="count(launchdfiles/launchdfile[apple != 'true' and (not(details) or (details != 'signatureoverride'))]) &gt; 0">
      <table class="tablelist">
        <xsl:apply-templates select="launchdfiles" mode="unsigned"/>
      </table>
    </xsl:if>

    <xsl:if
      test="count(launchdtasks/launchdtask[apple != 'true' and (not(details) or (details != 'signatureoverride'))]) &gt; 0">
      <table class="tablelist">
        <xsl:apply-templates select="launchdtasks" mode="unsigned"/>
      </table>
    </xsl:if>

    <xsl:if test="count(loginitems/loginitem) &gt; 0">
      <table class="tablelist">
        <xsl:apply-templates select="loginitems" mode="unsigned"/>
      </table>
    </xsl:if>

    <xsl:if test="count(applications/application) &gt; 0">
      <table class="tablelist">
        <xsl:apply-templates select="applications" mode="unsigned"/>
      </table>
    </xsl:if>

    <p class="description">    
      <xsl:copy-of select="$strings/unsigned/description[@version = $prefversion]/node()"/>
    
      <xsl:apply-templates
        select="$strings/unsigned/description_apple" mode="copy"/>
    </p>
    
  </xsl:template>

  <!-- Print launchd files. -->
  <xsl:template match="launchdfiles" mode="unsigned">
  
    <thead>
      <tr>
        <th colspan="2">
          <xsl:value-of select="$strings/unsigned/launchdfiles"/>
        </th>
      </tr>
    </thead>
    
    <xsl:apply-templates select="launchdfile" mode="unsigned"/>

  </xsl:template>
  
  <!-- Print an unsigned launchd file. -->
  <xsl:template match="launchdfile" mode="unsigned">
  
    <xsl:if test="apple != 'true' and (not(details) or (details != 'signatureoverride'))">
      <xsl:apply-templates select="."/>
    </xsl:if>
    
  </xsl:template>

  <!-- Print launchd tasks. -->
  <xsl:template match="launchdtasks" mode="unsigned">
  
    <thead>
      <tr>
        <th colspan="2">
          <xsl:value-of select="$strings/unsigned/launchdtasks"/>
        </th>
      </tr>
    </thead>
    
    <xsl:apply-templates select="launchdtask" mode="unsigned"/>

  </xsl:template>
  
  <!-- Print an unsigned launchd task. -->
  <xsl:template match="launchdtask" mode="unsigned">
  
    <xsl:variable name="path_unsafe" select="executable_unsafe"/>
    <xsl:variable name="bundle_unsafe" select="bundle_unsafe"/>
    
    <xsl:variable
      name="identifier"
      select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/identifier"/>

    <xsl:variable
      name="app_path"
      select="/etrecheck/header/context/application[path_unsafe = $bundle_unsafe]/path_unsafe"/>

    <xsl:variable name="iconpath">
      <xsl:choose>
        <xsl:when test="string-length($app_path) &gt; 0">
          <xsl:variable name="iconname">
            <xsl:call-template name="appicon">
              <xsl:with-param name="path">
                <xsl:value-of select="$app_path"/>
              </xsl:with-param>
            </xsl:call-template>
          </xsl:variable>
          <xsl:value-of select="concat('image/app/', $iconname)"/>
        </xsl:when>
        <xsl:when test="string-length($identifier) &gt; 0">
          <xsl:variable name="iconname">
            <xsl:call-template name="processicon">
              <xsl:with-param name="identifier">
                <xsl:value-of select="$identifier"/>
              </xsl:with-param>
            </xsl:call-template>
          </xsl:variable>
          <xsl:value-of select="concat('image/process/', $iconname)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>image/app/nil</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="name">
      <xsl:choose>
        <xsl:when test="name">
          <xsl:value-of select="name"/>
        </xsl:when>
        <xsl:when test="label">
          <xsl:value-of select="label"/>
        </xsl:when>
        <xsl:when test="filename_unsafe">
          <xsl:value-of select="filename_unsafe"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="path_unsafe"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:if test="apple != 'true' and (not(details) or (details != 'signatureoverride'))">
      
      <tbody>
        <xsl:attribute name="class">
          <xsl:value-of select="identifier"/>
          <xsl:text> group</xsl:text>
        </xsl:attribute>
        
        <tr class="name">
          <td>
            <img class="appicon">
              <xsl:attribute name="src">
                <xsl:value-of select="$iconpath"/>
              </xsl:attribute>
              <xsl:attribute name="alt">
                <xsl:value-of select="$strings/unsigned/app_icon"/>
              </xsl:attribute>
            </img>

            <xsl:value-of select="$name"/>
          </td>
          <td>
            <button class="review">
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('GET(&quot;/review/launchd/',identifier,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/review/review_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/review/review"/>
            </button>
          </td>
        </tr>

      </tbody>
    </xsl:if>
    
  </xsl:template>
  
  <!-- Print unsigned Safari extensions. -->
  <xsl:template match="safariextensions" mode="unsigned">
  
    <thead>
      <tr>
        <th colspan="2">
          <div class="headerwrapper">
            <xsl:value-of select="$strings/unsigned/safariextensions"/>
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/openapplication//Applications/Safari.app')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of
                  select="$strings/safari/open_safari_title[@version = $prefversion]"/>
              </xsl:attribute>
              <xsl:value-of
                select="$strings/safari/open_safari"/>
              <img class="buttonicon" src="image/safaributton.png"/>
            </button>
          </div>
        </th>
      </tr>
    </thead>
    
    <xsl:apply-templates select="extension" mode="unsigned"/>
    
  </xsl:template>
  
  <!-- Print an unsigned Safari extension. -->
  <xsl:template match="extension" mode="unsigned">
  
    <tbody>
      <xsl:attribute name="class">
        <xsl:value-of select="identifier"/>
        <xsl:text> group</xsl:text>
      </xsl:attribute>
      <tr class="horizontalborder">
        <td class="header">
          <xsl:value-of select="$strings/unsigned/safariextension"/>
        </td>
        <td class="data">
          <xsl:value-of select="displayname"/>
        </td>
      </tr>
      <tr>
        <td class="header">
          <xsl:value-of select="$strings/unsigned/installdate"/>
        </td>
        <td class="data">
          <xsl:value-of select="installdate"/>
        </td>
      </tr>
    </tbody>
    
  </xsl:template>

  <!-- Print unsigned Login items. -->
  <xsl:template match="loginitems" mode="unsigned">
  
    <thead>
      <tr>
        <th colspan="3">
          <div class="headerwrapper">
            <xsl:value-of select="$strings/unsigned/loginitems"/>
          </div>
        </th>
      </tr>
    </thead>
    
    <xsl:apply-templates select="loginitem" mode="unsigned"/>
    
  </xsl:template>

  <!-- Print an unsigned login item. -->
  <xsl:template match="loginitem" mode="unsigned">
  
    <tbody>
      <xsl:attribute name="class">
        <xsl:value-of select="identifier"/>
        <xsl:text> group</xsl:text>
      </xsl:attribute>
      
      <xsl:apply-templates select="."/>
      
    </tbody>
    
  </xsl:template>

  <!-- Print unsigned Plugins. -->
  <xsl:template match="plugins" mode="unsigned">
  
    <thead>
      <tr>
        <th colspan="3">
          <div class="headerwrapper">
            <xsl:value-of select="$strings/unsigned/plugins"/>
          </div>
        </th>
      </tr>
    </thead>
    <tbody>
      <xsl:attribute name="class">
        <xsl:value-of select="identifier"/>
        <xsl:text> group</xsl:text>
      </xsl:attribute>
      
      <tr>
        <td colspan="2">
          <strong>
            <xsl:value-of select="$strings/plugins/name"/>
          </strong>
        </td>
        <td>
          <strong>
            <xsl:value-of select="$strings/plugins/date"/>
          </strong>
        </td>
      </tr>
    
      <xsl:apply-templates select="plugin" mode="unsigned"/>
    </tbody>
    
  </xsl:template>

  <!-- Print unsigned Preference panes. -->
  <xsl:template match="preferencepanes" mode="unsigned">
  
    <thead>
      <tr>
        <th colspan="4">
          <div class="headerwrapper">
            <xsl:value-of select="$strings/unsigned/preferencepanes[@version = $prefversion]"/>
          </div>
        </th>
      </tr>
    </thead>
    <tbody>
      <xsl:attribute name="class">
        <xsl:value-of select="identifier"/>
        <xsl:text> group</xsl:text>
      </xsl:attribute>
      
      <tr>
        <td>
          <strong>
            <xsl:value-of select="$strings/preferencepane/name"/>
          </strong>
        </td>
        <td>
          <strong>
            <xsl:value-of select="$strings/preferencepane/developer"/>
          </strong>
        </td>
        <td>
          <strong>
            <xsl:value-of select="$strings/preferencepane/date"/>
          </strong>
        </td>
        <td/>
      </tr>
    
      <xsl:apply-templates select="preferencepane"/>
    </tbody>
    
  </xsl:template>

  <!-- Print unsigned apps. -->
  <xsl:template match="applications" mode="unsigned">
  
    <thead>
      <tr class="last_child">
        <th colspan="3">
          <strong>
            <xsl:call-template name="plural">
              <xsl:with-param name="keys" select="$strings/unsigned"/>
              <xsl:with-param name="key" select="'unsignedapps'"/>
              <xsl:with-param name="value" select="count(application)"/>
            </xsl:call-template>
          </strong>
        </th>
        <th>
          <div class="buttonwrapper">
            <button class="show_unsigned_apps">
              <xsl:attribute name="onclick">
                <xsl:text>showUnsignedApps(this)</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/unsigned/show_apps_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/unsigned/show_apps"/>
            </button>
            <button class="hide_unsigned_apps">
              <xsl:attribute name="onclick">
                <xsl:text>hideUnsignedApps(this)</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/unsigned/hide_apps_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/unsigned/hide_apps"/>
            </button>
          </div>
        </th>
      </tr>
    </thead>
    
    <tbody class="unsignedapps group">
      <tr>
        <td class="header">
          <xsl:value-of select="$strings/applications/name"/>
        </td>
        <td class="header">
          <span class="version">
            <xsl:value-of select="$strings/applications/version"/>
          </span>
        </td>
        <td class="header">
          <span class="last_used">
            <xsl:value-of select="$strings/applications/last_used"/>
          </span>
        </td>
        <td/>
      </tr>
    
      <xsl:apply-templates select="application"/>
    </tbody>
    
  </xsl:template>

  <!-- A menu for the software page. -->
  <xsl:template name="softwaremenu">
  
    <xsl:variable name="extensionsid">
      <xsl:choose>
        <xsl:when
          test="count(/etrecheck/systemextensions/systemextension) &gt; 0">
          <xsl:text>systemextensions</xsl:text>
        </xsl:when>
        <xsl:when
          test="count(/etrecheck/kernelextensions/*) &gt; 0">
          <xsl:text>kernelextensions</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>omit</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="launchdid">
      <xsl:choose>
        <xsl:when
          test="count(/etrecheck/launchdaemons/*) &gt; 0">
          <xsl:text>launchdaemons</xsl:text>
        </xsl:when>
        <xsl:when
          test="count(/etrecheck/launchagents/*) &gt; 0">
          <xsl:text>launchagents</xsl:text>
        </xsl:when>
        <xsl:when
          test="count(/etrecheck/userlaunchagents/*) &gt; 0">
          <xsl:text>userlaunchagents</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>omit</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="loginitemsid">
      <xsl:choose>
        <xsl:when
          test="count(/etrecheck/loginitems/*) &gt; 0">
          <xsl:text>loginitems</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>omit</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="appsid">
      <xsl:text>applications</xsl:text>
    </xsl:variable>

    <xsl:variable name="pluginsid">
      <xsl:choose>
        <xsl:when
          test="count(/etrecheck/modernplugins/*) &gt; 0">
          <xsl:text>modernplugins</xsl:text>
        </xsl:when>
        <xsl:when
          test="count(/etrecheck/internetplugins/*) &gt; 0">
          <xsl:text>internetplugins</xsl:text>
        </xsl:when>
        <xsl:when
          test="count(/etrecheck/userinternetplugins/*) &gt; 0">
          <xsl:text>userinternetplugins</xsl:text>
        </xsl:when>
        <xsl:when
          test="count(/etrecheck/audioplugins/*) &gt; 0">
          <xsl:text>audioplugins</xsl:text>
        </xsl:when>
        <xsl:when
          test="count(/etrecheck/coremediaioplugins/*) &gt; 0">
          <xsl:text>coremediaioplugins</xsl:text>
        </xsl:when>
        <xsl:when
          test="count(/etrecheck/useraudioplugins/*) &gt; 0">
          <xsl:text>useraudioplugins</xsl:text>
        </xsl:when>
        <xsl:when
          test="count(/etrecheck/usercoremediaioplugins/*) &gt; 0">
          <xsl:text>usercoremediaioplugins</xsl:text>
        </xsl:when>
        <xsl:when
          test="count(/etrecheck/itunesplugins/*) &gt; 0">
          <xsl:text>itunesplugins</xsl:text>
        </xsl:when>
        <xsl:when
          test="count(/etrecheck/useritunesplugins/*) &gt; 0">
          <xsl:text>useritunesplugins</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>omit</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="installsid">
      <xsl:choose>
        <xsl:when
          test="count(/etrecheck/install/package) &gt; 0">
          <xsl:text>install</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>omit</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="cleanupid">
      <xsl:choose>
        <xsl:when
          test="count(/etrecheck/cleanup/*) &gt; 0">
          <xsl:text>cleanup</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>omit</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <div class="menu">
      <table class="menu">
        <tr>
          <xsl:if test="$extensionsid != 'omit'">
            <td>
              <a href="#">
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('menuJumpToId(&quot;',$extensionsid,'&quot;)')"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/softwaremenu/extensions"/>
              </a>
            </td>
          </xsl:if>
          <xsl:if test="$launchdid != 'omit'">
            <td>
              <a href="#">
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('menuJumpToId(&quot;',$launchdid,'&quot;)')"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/softwaremenu/launchd"/>
              </a>
            </td>
          </xsl:if>
          <xsl:if test="$loginitemsid != 'omit'">
            <td>
              <a href="#">
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('menuJumpToId(&quot;',$loginitemsid,'&quot;)')"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/softwaremenu/loginitems"/>
              </a>
            </td>
          </xsl:if>
          <xsl:if test="$appsid != 'omit'">
            <td>
              <a href="#">
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('menuJumpToId(&quot;',$appsid,'&quot;)')"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/softwaremenu/apps"/>
              </a>
            </td>
          </xsl:if>
          <xsl:if test="$pluginsid != 'omit'">
            <td>
              <a href="#">
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('menuJumpToId(&quot;',$pluginsid,'&quot;)')"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/softwaremenu/plugins"/>
              </a>
            </td>
          </xsl:if>
          <xsl:if test="$installsid != 'omit'">
            <td>
              <a href="#">
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('menuJumpToId(&quot;',$installsid,'&quot;)')"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/softwaremenu/installs"/>
              </a>
            </td>
          </xsl:if>
          <xsl:if test="$cleanupid != 'omit'">
            <td>
              <a href="#">
                <xsl:attribute name="onclick">
                  <xsl:value-of
                    select="concat('menuJumpToId(','&quot;',$cleanupid,'&quot;)')"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/softwaremenu/cleanup"/>
              </a>
            </td>
          </xsl:if>
        </tr>
      </table>
    </div>
    
    <div class="menuspacer"/>
    
  </xsl:template>
  
  <!-- Apps. -->
  <xsl:template match="applications" mode="top">
  
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">applications</xsl:with-param>
      <xsl:with-param name="contents">
        <xsl:apply-templates select="."/>
      </xsl:with-param>
    </xsl:call-template>
    
  </xsl:template>

  <xsl:template match="applications">

    <table class="tablelist">
      <xsl:call-template name="appsheader"/>
      <xsl:call-template name="developerheader"/>

      <xsl:for-each select="developer">
        <xsl:call-template name="developer"/>
      </xsl:for-each>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/applications/description/node()"/>

      <xsl:choose>
        <xsl:when
          test="@count > 0 and $report/etrecheck/hardware/platform = 'arm64'">
          <xsl:copy-of
            select="$strings/applications/description_intel/node()"/>
          <xsl:copy-of
            select="$strings/applications/description_old/node()"/>
        </xsl:when>

        <xsl:when test="$report/etrecheck/systemsoftware/majorversion &lt; 15 and @count &gt; 0">
          <xsl:copy-of
            select="$strings/applications/description_32bit/node()"/>
        </xsl:when>
        
        <xsl:when test="$report/etrecheck/systemsoftware/majorversion &gt;= 15 and @count &gt; 0">
          <xsl:copy-of
            select="$strings/applications/description_intel/node()"/>
        </xsl:when>

        <xsl:otherwise/>
        
      </xsl:choose>

    </p>

  </xsl:template>
  
  <xsl:template name="appsheader">
  
    <xsl:variable name="oldkey">
      <xsl:choose>
        <xsl:when
          test="$report/etrecheck/systemsoftware/majorversion &lt; 15 and @oldcount &gt; 0">
          <xsl:text>apps32</xsl:text>
        </xsl:when>
        <xsl:when
          test="$report/etrecheck/systemsoftware/majorversion &gt;= 15 and @oldcount &gt; 0">
          <xsl:text>appsintel</xsl:text>
        </xsl:when>
        <xsl:otherwise/>
      </xsl:choose>
    </xsl:variable>
    
    <tr>
      <td/>
      <td/>
      <td class="apps">
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/applications"/>
          <xsl:with-param name="key" select="'apps'"/>
          <xsl:with-param name="value" select="@count"/>
        </xsl:call-template>
      </td>
      <xsl:choose>
        <xsl:when test="@oldcount &gt; 0">
          <td class="apps">
            <xsl:call-template name="plural">
              <xsl:with-param name="keys" select="$strings/applications"/>
              <xsl:with-param name="key" select="'oldapps'"/>
              <xsl:with-param name="value" select="@oldcount"/>
            </xsl:call-template>
          </td>
          <td>
            <xsl:choose>
              <xsl:when
                test="$report/etrecheck/systemsoftware/majorversion &lt; 15 and @oldcount &gt; 0">
                <button class="vintage">
                  <xsl:attribute name="onclick">
                    <xsl:text>GET("/link/https://support.apple.com/HT208436");</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/applications/about_old_apps_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/applications/about_old_apps"/>
                  <img class="buttonicon" src="image/webbrowserbutton.png"/>
                </button>
              </xsl:when>
              <xsl:when
                test="$report/etrecheck/systemsoftware/majorversion &gt;= 15 and @oldcount &gt; 0">
                <button class="vintage">
                  <xsl:attribute name="onclick">
                    <xsl:text>GET("/link/https://support.apple.com/HT211861");</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/applications/about_intel_apps_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/applications/about_intel_apps"/>
                  <img class="buttonicon" src="image/webbrowserbutton.png"/>
                </button>
              </xsl:when>
              <xsl:otherwise/>
            </xsl:choose>
          </td>
        </xsl:when>
        <xsl:otherwise>
          <td/>
          <td/>
        </xsl:otherwise>
      </xsl:choose>
    </tr>
    
  </xsl:template>

  <xsl:template name="developerheader">
  
    <tr class="apps horizontalborder">
      <th colspan="2">
        <xsl:value-of select="$strings/applications/developer"/>
      </th>
      <th>
        <xsl:value-of select="$strings/applications/count"/>
      </th>
      <th>
        <xsl:value-of select="$strings/applications/oldcount"/>
      </th>
      <th/>
    </tr>
  
  </xsl:template>
  
  <xsl:template name="developer">
  
    <tr class="apps horizontalborder">
      <td colspan="2">
        <xsl:choose>
          <xsl:when test="name = '?'">
            <xsl:value-of select="$strings/applications/unsigned"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="name"/>
          </xsl:otherwise>
        </xsl:choose>
      </td>
      <td class="apps">
        <xsl:call-template name="plural">
          <xsl:with-param
            name="keys" select="$strings/applications"/>
          <xsl:with-param name="key" select="'apps'"/>
          <xsl:with-param name="value" select="@count"/>
        </xsl:call-template>
      </td>
      <td class="apps">
        <xsl:if test="@oldcount &gt; 0">
          <xsl:call-template name="plural">
            <xsl:with-param
              name="keys" select="$strings/applications"/>
            <xsl:with-param name="key" select="'oldapps'"/>
            <xsl:with-param name="value" select="@oldcount"/>
          </xsl:call-template>
        </xsl:if>
      </td>
      <td>
        <button>
          <xsl:attribute name="onclick">
            <xsl:value-of
              select="concat('GET(&quot;/showapps/',@developerid,'&quot;)')"/>
          </xsl:attribute>
          <xsl:attribute name="title">
            <xsl:value-of
              select="$strings/review/review_title"/>
          </xsl:attribute>
          <xsl:value-of select="$strings/review/review"/>
        </button>
      </td>
    </tr>

  </xsl:template>
  
  <!-- Print an application. -->
  <xsl:template match="application">
  
    <xsl:variable name="iconname">
      <xsl:call-template name="appicon">
        <xsl:with-param name="path">
          <xsl:value-of select="path_unsafe"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>

    <tr class="horizontalborder">
      <td>
        <span class="name">
          <img class="appicon">
            <xsl:attribute name="src">
              <xsl:value-of
                select="concat('image/app/', $iconname)"/>
            </xsl:attribute>
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/applications/app_icon"/>
            </xsl:attribute>
          </img>
          <xsl:value-of select="name"/>
        </span>
      </td>
      <td><span class="version"><xsl:value-of select="version"/></span></td>
      <td><span class="last_used"><xsl:value-of select="substring(access_date, 1, 10)"/></span></td>
      <td>
        <button>
          <xsl:attribute name="onclick">
            <xsl:value-of
              select="concat('GET(&quot;/reveal/',path_unsafe,'&quot;)')"/>
          </xsl:attribute>
          <xsl:attribute name="title">
            <xsl:value-of select="$strings/applications/reveal_in_finder_title"/>
          </xsl:attribute>
          <xsl:value-of
            select="$strings/applications/reveal_in_finder"/>
          <img class="buttonicon" src="image/finderbutton.png"/>
        </button>
      </td>
    </tr>
    
  </xsl:template>

  <!-- Print a directory. -->
  <xsl:template match="directory">
  
    <xsl:variable name="iconname">
      <xsl:call-template name="appicon">
        <xsl:with-param name="path">
          <xsl:value-of select="path_unsafe"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>

    <tr class="horizontalborder">
      <td colspan="3">
        <span>
          <img class="appicon" src="image/folder">
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/applications/folder_icon"/>
            </xsl:attribute>
          </img>
          <xsl:value-of select="name_unsafe"/>
          <xsl:text> (</xsl:text>
          <xsl:call-template name="plural">
            <xsl:with-param
              name="keys" select="$strings/applications"/>
            <xsl:with-param name="key" select="'oldapps'"/>
            <xsl:with-param name="value" select="@count"/>
          </xsl:call-template>
          <xsl:text>)</xsl:text>
        </span>
      </td>
      <td>
        <button>
          <xsl:attribute name="onclick">
            <xsl:value-of
              select="concat('GET(&quot;/openfolder/',path_unsafe,'&quot;)')"/>
          </xsl:attribute>
          <xsl:attribute name="title">
            <xsl:value-of select="$strings/applications/reveal_in_finder_title"/>
          </xsl:attribute>
          <xsl:value-of
            select="$strings/applications/reveal_in_finder"/>
          <img class="buttonicon" src="image/finderbutton.png"/>
        </button>
      </td>
    </tr>
    
  </xsl:template>

  <!-- System extensions. -->
  <xsl:template match="systemextensions" mode="top">
  
    <xsl:if test="count(systemextension) &gt; 0">

      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <!-- System extensions. -->
  <xsl:template match="systemextensions">
  
    <table class="tablelist">

      <xsl:for-each select="systemextension">
      
        <xsl:apply-templates select="."/>

      </xsl:for-each>
      
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/systemextensions/description/node()"/>

      <xsl:if
        test="count(systemextension[status = 'activated_waiting_for_user' or status = 'activated_disabled']) &gt; 0">
        <xsl:copy-of
          select="$strings/systemextensions/description_blocked[@version = $prefversion]/node()"/>
      </xsl:if>

      <xsl:copy-of
        select="$strings/systemextensions/description_uninstall/node()"/>
      
      <xsl:if
        test="count(systemextension[orphan = 'true' and status = 'terminated_waiting_to_uninstall_on_reboot']) &gt; 0">
        <xsl:copy-of
          select="$strings/systemextensions/description_restart/node()"/>
      </xsl:if>

      <xsl:if
        test="count(systemextension[orphan = 'true' and status = 'activated_enabled']) &gt; 0">
        <xsl:copy-of
          select="$strings/systemextensions/description_orphan/node()"/>
      </xsl:if>
    </p>
    
  </xsl:template>

  <!-- System extension. -->
  <xsl:template match="systemextension">
  
    <xsl:variable name="status" select="status"/>
    
    <!-- Get the localized bus speed name. -->
    <xsl:variable name="statusvalue">
      <xsl:value-of
        select="$strings/systemextensions/status_text/value[@key = $status]"/>
    </xsl:variable>

    <!-- If the status value is empty, just use what I have. -->
    <xsl:variable name="statusstring">
      <xsl:choose>
        <xsl:when test="string-length($statusvalue)">
          <xsl:value-of select="$statusvalue"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$status"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="iconname">
      <xsl:call-template name="appicon">
        <xsl:with-param name="path">
          <xsl:value-of select="app_path_unsafe"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>

    <tr>
      <th>
        <xsl:value-of select="$strings/systemextensions/name"/>
      </th>
      <th>
        <xsl:if
          test="orphan = 'false' and app_exists = 'true' and string-length($iconname) &gt; 0">
          <img class="appicon">
            <xsl:attribute name="src">
              <xsl:value-of
                select="concat('image/app/', $iconname)"/>
            </xsl:attribute>
            <xsl:attribute name="alt">
              <xsl:value-of
                select="$strings/systemextensions/app_icon"/>
            </xsl:attribute>
          </img>
        </xsl:if>
        
        <xsl:value-of select="name"/>
        
        <xsl:choose>
          <xsl:when test="app_exists = 'true' and app_path_unsafe">
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('GET(&quot;/reveal/',app_path_unsafe,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of
                  select="$strings/systemextensions/reveal_in_finder_title"/>
              </xsl:attribute>
              <xsl:value-of
                select="$strings/systemextensions/reveal_in_finder"/>
              <img class="buttonicon" src="image/finderbutton.png"/>
            </button>
          </xsl:when>
          <xsl:when test="orphan = 'true' and status = 'terminated_waiting_to_uninstall_on_reboot'">
            <button>
              <xsl:attribute name="onclick">
                <xsl:text>GET('/restart')</xsl:text>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/cleanup/remove_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/cleanup/remove"/>
              <img class="buttonicon" src="image/trashbutton.png"/>
            </button>
          </xsl:when>
          <xsl:when test="false() and orphan = 'true' and status != 'terminated_waiting_to_uninstall_on_reboot'">
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('GET(&quot;/systemextension/uninstall/',identifier,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/cleanup/remove_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/cleanup/remove"/>
              <img class="buttonicon" src="image/trashbutton.png"/>
            </button>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>
      </th>
    </tr>
    
    <tr class="horizontalborder">
      <td class="header">
        <xsl:value-of select="$strings/systemextensions/version"/>
        <br/>
        
        <xsl:value-of select="$strings/systemextensions/install_date"/>
        <br/>

        <xsl:value-of select="$strings/systemextensions/developer"/>
        <br/>
        
        <xsl:value-of select="$strings/systemextensions/status"/>
        <br/>
        
        <xsl:value-of select="$strings/systemextensions/application"/>
        
        <xsl:choose>
          <xsl:when test="orphan = 'false' and app_exists = 'true'">
              
            <xsl:if test="version != app_version">
              <br/>
              <xsl:value-of
                select="$strings/systemextensions/app_version"/>
            </xsl:if>

            <xsl:if
              test="substring(installdate, 1, 10) != substring(app_installdate, 1, 10)">
              <br/>
              <xsl:value-of
                select="$strings/systemextensions/app_install_date"/>
            </xsl:if>

            <xsl:if test="developer != app_developer">
              <br/>
              <xsl:value-of
                select="$strings/systemextensions/app_developer"/>
            </xsl:if>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>

        <xsl:if test="description">
          <br/>
          <xsl:value-of
            select="$strings/systemextensions/extensiondescription"/>
        </xsl:if>
        
      </td>
      <td>
        <xsl:value-of select="version"/>
        <br/>
          
        <xsl:value-of select="installdate"/>
        <br/>
          
        <xsl:value-of select="developer"/>
        <br/>
        
        <xsl:value-of select="$statusstring"/>
        
        <xsl:if test="status = 'activated_waiting_for_user'">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/locallink/x-apple.systempreferences:com.apple.preference.security?General')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of
                select="$strings/systemextensions/open_security_privacy_title[@version = $prefversion]"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/systemextensions/open_security_privacy"/>
            <img
              class="buttonicon"
              src="image/securityprefpanebutton.png"/>
          </button>
        </xsl:if>
        
        <xsl:choose>
          <xsl:when test="orphan = 'false' and app_exists = 'true'">
            <br/>
            <xsl:value-of select="app_name"/>
            
            <xsl:if test="version != app_version">
              <br/>
              <xsl:value-of select="app_version"/>
            </xsl:if>

            <xsl:if
              test="substring(installdate, 1, 10) != substring(app_installdate, 1, 10)">
              <br/>
              <xsl:value-of select="app_installdate"/>
            </xsl:if>

            <xsl:if test="developer != app_developer">
              <br/>
              <xsl:value-of select="app_developer"/>
            </xsl:if>
          </xsl:when>
          <xsl:otherwise>
            <br/>
            <xsl:value-of select="$strings/systemextensions/notfound"/>
          </xsl:otherwise>
        </xsl:choose>

        <xsl:if test="description">
          <br/>
          <xsl:value-of select="description"/>
        </xsl:if>

      </td>
    </tr>

  </xsl:template>

  <!-- Kernel extensions. -->
  <xsl:template match="kernelextensions">
  
    <table class="tablelist">

      <xsl:for-each select="directory">
      
        <xsl:variable name="iconname">
          <xsl:call-template name="appicon">
            <xsl:with-param name="path">
              <xsl:value-of select="path_unsafe"/>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:variable>

        <tr>
          <th>
            <xsl:value-of select="$strings/kernelextensions/container"/>
          </th>
          <th>
            <xsl:choose>
              
              <xsl:when test="name">
                <xsl:if test="string-length($iconname) &gt; 0">
                  <img class="appicon">
                    <xsl:attribute name="src">
                      <xsl:value-of
                        select="concat('image/app/', $iconname)"/>
                    </xsl:attribute>
                    <xsl:attribute name="alt">
                      <xsl:value-of
                        select="$strings/kernelextensions/app_icon"/>
                    </xsl:attribute>
                  </img>
                </xsl:if>
                
                <xsl:value-of select="name"/>
              </xsl:when>
              
              <xsl:otherwise>
                <xsl:value-of select="path"/>
              </xsl:otherwise>
              
            </xsl:choose>
            
            <xsl:choose>
            
              <xsl:when test="path = '/Library/Extensions'">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/openfolder_restricted//Library/Extensions')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of
                      select="$strings/kernelextensions/open_in_finder_title"/>
                  </xsl:attribute>
                  <xsl:value-of
                    select="$strings/kernelextensions/open_in_finder"/>
                  <img class="buttonicon" src="image/finderbutton.png"/>
                </button>
            
              </xsl:when>
            
              <xsl:when test="path = '/System/Library/Extensions'">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/openfolder_restricted//System/Library/Extensions')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of
                      select="$strings/kernelextensions/open_in_finder_title"/>
                  </xsl:attribute>
                  <xsl:value-of
                    select="$strings/kernelextensions/open_in_finder"/>
                  <img class="buttonicon" src="image/finderbutton.png"/>
                </button>
            
              </xsl:when>

              <xsl:otherwise>
              
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/reveal/',path_unsafe,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of
                      select="$strings/kernelextensions/reveal_in_finder_title"/>
                  </xsl:attribute>
                  <xsl:value-of
                    select="$strings/kernelextensions/reveal_in_finder"/>
                  <img class="buttonicon" src="image/finderbutton.png"/>
                </button>
                
              </xsl:otherwise>
            
            </xsl:choose>
            
          </th>
        </tr>

        <xsl:for-each select="extensions/extension">
        
          <xsl:variable name="status" select="status"/>
          
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of select="$strings/kernelextensions/extension"/>
              <br/>
              <xsl:value-of select="$strings/kernelextensions/label"/>
              
              <xsl:if test="version or osversion">
                <br/>
                <xsl:value-of select="$strings/kernelextensions/version"/>
              </xsl:if>
              
              <xsl:if test="developer">
                <br/>
                <xsl:value-of select="$strings/kernelextensions/developer"/>
              </xsl:if>
              
              <xsl:if test="/etrecheck/header/apptype = 'pro'">
                <br/>
                <xsl:value-of select="$strings/kernelextensions/status"/>
              </xsl:if>
            </td>
            <td>
              <xsl:value-of select="filename"/>
              <br/>
              <xsl:value-of select="bundleid"/>
              
              <xsl:if test="version or osversion">
                <br/>
                
                <xsl:if test="version">
                  <xsl:value-of select="version"/>
                </xsl:if>
                
                <xsl:if test="osversion">
                  <xsl:if test="version">
                    <xsl:text> - </xsl:text>
                  </xsl:if>
                  <xsl:value-of select="osversion"/>
                </xsl:if>
              </xsl:if>
              
              <xsl:if test="developer">
                <br/>
                <xsl:value-of select="developer"/>
              </xsl:if>

              <xsl:if test="/etrecheck/header/apptype = 'pro'">
                <br/>
                <xsl:value-of
                  select="$strings/kernelextensions/status_text/value[@key = $status]"/>
              </xsl:if>
              
              <xsl:if test="status = 'blocked'">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/locallink/x-apple.systempreferences:com.apple.preference.security?General')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/kernelextensions/open_security_privacy_title[@version = $prefversion]"/>
                  </xsl:attribute>
                  <xsl:value-of
                    select="$strings/kernelextensions/open_security_privacy"/>
                  <img class="buttonicon" src="image/securityprefpanebutton.png"/>
                </button>
              </xsl:if>
            </td>
          </tr>
        </xsl:for-each>
      </xsl:for-each>
      
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/kernelextensions/description/node()"/>

      <xsl:if
        test="count(directory/extensions/extension[status = 'blocked']) &gt; 0">
        <xsl:copy-of
          select="$strings/kernelextensions/description_blocked[@version = $prefversion]/node()"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- Print system launch agents. -->
  <xsl:template match="systemlaunchagents">
  
    <xsl:call-template name="applelaunchdfiles"/>
    
    <p class="description">
      <xsl:copy-of select="$strings/systemlaunchagents/description/node()"/>
    </p>

  </xsl:template>
  
  <!-- Print system launch daemons. -->
  <xsl:template match="systemlaunchdaemons">
  
    <xsl:call-template name="applelaunchdfiles"/>
    
    <p class="description">
      <xsl:copy-of select="$strings/systemlaunchdaemons/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print Apple launchd files. -->
  <xsl:template name="applelaunchdfiles">
  
    <table class="tablelist">
  
      <xsl:apply-templates select="applelaunchdfile"/>
      
    </table>
    
  </xsl:template>

  <!-- Print Apple launchd files. -->
  <xsl:template match="applelaunchdfile">
  
    <xsl:variable name="type">
    
      <xsl:choose>
        <xsl:when test="name(..) = 'systemlaunchagents'">
          <xsl:text>agent</xsl:text>
        </xsl:when>
        <xsl:when test="name(..) = 'systemlaunchdaemons'">
          <xsl:text>daemon</xsl:text>
        </xsl:when>
      </xsl:choose>
      
    </xsl:variable>
    
    <xsl:variable name="status" select="status"/>
    
    <xsl:variable name="last_row">
      <xsl:choose>
        <xsl:when test="position() = last()">
          <xsl:text>true</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>false</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <tr>
      <xsl:if test="position() = last()">
        <xsl:attribute name="class">
          <xsl:text>last_child</xsl:text>
        </xsl:attribute>
      </xsl:if>
      
      <th>
        <xsl:value-of select="$strings/launchd/status"/>
        <xsl:text> </xsl:text>
        <xsl:value-of
          select="$strings/launchd/status_text/value[@key = $status]"/>
        <xsl:text> - </xsl:text>
        <xsl:call-template name="plural">
          <xsl:with-param name="keys" select="$strings/launchd"/>
          <xsl:with-param name="key" select="'apple_files'"/>
          <xsl:with-param name="value" select="count"/>
        </xsl:call-template>
      </th>
      <th>
        <div class="buttonwrapper">
          <button class="show_apple_files">
            <xsl:attribute name="onclick">
              <xsl:value-of
                select="concat('showAppleFiles(this, &quot;', status, '&quot;,&quot;', $type, '&quot;,', $last_row, ')')"/>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/launchd/show_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/launchd/show"/>
          </button>
          <button class="hide_apple_files">
            <xsl:attribute name="onclick">
              <xsl:value-of
                select="concat('hideAppleFiles(this, &quot;', status, '&quot;,&quot;', $type, '&quot;,', $last_row, ')')"/>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/launchd/hide_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/launchd/hide"/>
          </button>
        </div>
      </th>
    </tr>

    <xsl:for-each select="../launchdfile[status = $status]">

      <tr>
        <xsl:attribute name="class">
          <xsl:value-of select="concat('apple horizontalborder ', $status, ' ', $type)"/>
        </xsl:attribute>
        
        <td class="label">
          <xsl:value-of select="label"/>
        
          <xsl:if test="$status = 'failed'">
            <xsl:value-of
              select="concat(' (', $strings/launchd/result, ' ', lastexitcode, ')')"/>
          </xsl:if>
          
        </td>
        
        <td>
          <xsl:if test="man">
            <!-- Provide a button to an Apple man page. -->
            <button>
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('GET(&quot;/man/',man,'&quot;)')"/>
              </xsl:attribute>
              <xsl:attribute name="title">
                <xsl:value-of select="$strings/diagnostics/about_title"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/diagnostics/about"/>
              <img class="buttonicon" src="image/man.png"/>
            </button>
          </xsl:if>
        </td>
      </tr>

    </xsl:for-each>
    
  </xsl:template>

  <!-- Print launch agents. -->
  <xsl:template match="launchagents">
  
    <xsl:call-template name="launchdfiles"/>
    
    <p class="description">
      <xsl:copy-of select="$strings/launchagents/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print launch daemons. -->
  <xsl:template match="launchdaemons">
  
    <xsl:call-template name="launchdfiles"/>
    
    <p class="description">
      <xsl:copy-of select="$strings/launchdaemons/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print user launch agents. -->
  <xsl:template match="userlaunchagents">
  
    <xsl:call-template name="launchdfiles"/>
    
    <p class="description">
      <xsl:copy-of select="$strings/userlaunchagents/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print launchd filess. -->
  <xsl:template name="launchdfiles">
  
    <table class="tablelist">
  
      <xsl:apply-templates select="launchdfile"/>
      
    </table>
    
  </xsl:template>
  
  <!-- Print a launchd file. -->
  <xsl:template match="launchdfile">
  
    <tbody>
      <xsl:attribute name="class">
        <xsl:value-of select="identifier"/>
        <xsl:text> group</xsl:text>
      </xsl:attribute>

      <xsl:variable name="name">
        <xsl:choose>
          <xsl:when test="filename_unsafe and filename_unsafe = filename">
            <xsl:value-of select="filename_unsafe"/>
          </xsl:when>
          <xsl:when test="name">
            <xsl:value-of select="name"/>
          </xsl:when>
          <xsl:when test="label">
            <xsl:value-of select="label"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="path_unsafe"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <tr>
        <td class="data">
          <xsl:value-of select="$name"/>
        </td>
        <td>
          <button class="review">
            <xsl:attribute name="onclick">
              <xsl:value-of
                select="concat('GET(&quot;/review/launchd/',identifier,'&quot;)')"/>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/review/review_title"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/review/review"/>
          </button>
        </td>
      </tr>
    </tbody>
    
  </xsl:template>

  <!-- Print login items. -->
  <xsl:template match="loginitems" mode="top">
  
    <xsl:if test="count(loginitem) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="loginitems">
  
    <table class="tablelist">
      
      <xsl:apply-templates select="loginitem"/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/loginitems/description[@version = $prefversion]/node()"/>
      
      <xsl:if test="$apptype = 'mas'">
        <xsl:copy-of select="$strings/loginitems/description_mas/node()"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- Print a single login item. -->
  <xsl:template match="loginitem">
  
    <!-- Save the type to a variable. -->
    <xsl:variable name="rawtype" select="type"/>
    
    <!-- Get the localized type. -->
    <xsl:variable name="localizedtype">
      <xsl:value-of
        select="$strings/loginitems/types/value[@key = $rawtype]"/>
    </xsl:variable>

    <!-- If the localized type is empty, just use what I have. -->
    <xsl:variable name="type">
      <xsl:choose>
        <xsl:when test="string-length($localizedtype)">
          <xsl:value-of select="$localizedtype"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$rawtype"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="iconname">
      <xsl:call-template name="appicon">
        <xsl:with-param name="path">
          <xsl:value-of select="bundle_unsafe"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="iconpath">
      <xsl:choose>
        <xsl:when test="type = 'directory'">
          <xsl:text>image/folder</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat('image/app/', $iconname)"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="name">
      <xsl:choose>
        <xsl:when test="bundlename_unsafe">
          <xsl:value-of select="bundlename_unsafe"/>
        </xsl:when>
        <xsl:when test="name_unsafe">
          <xsl:value-of select="name_unsafe"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="path_unsafe"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <tr class="horizontalborder">

      <td class="appcolumn">
        <img class="appicon">
          <xsl:attribute name="src">
            <xsl:value-of select="$iconpath"/>
          </xsl:attribute>
          <xsl:attribute name="alt">
            <xsl:value-of select="$strings/applications/app_icon"/>
          </xsl:attribute>
        </img>
        <xsl:value-of select="$name"/>
      </td>
      <td class="namecolumn">
        <xsl:if test="name_unsafe != bundlename_unsafe">
          <span><xsl:value-of select="name_unsafe"/></span>
        </xsl:if>
      </td>
      <td class="data buttoncolumn">
        <button class="review">
          <xsl:attribute name="onclick">
            <xsl:value-of
              select="concat('GET(&quot;/review/loginitem/',identifier,'&quot;)')"/>
          </xsl:attribute>
          <xsl:attribute name="title">
            <xsl:value-of select="$strings/review/review_title"/>
          </xsl:attribute>
          <xsl:value-of select="$strings/review/review"/>
        </button>
      </td>
    </tr>
    
  </xsl:template>

  <!-- Print modern plugins. -->
  <xsl:template match="modernplugins" mode="top">
  
    <xsl:if test="count(extensionpoint) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button class="manage">
            <xsl:attribute name="onclick">
              <xsl:text>GET('/manage/appextensions')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/manage_title[@version = $prefversion or not(@version)]"/>
            </xsl:attribute>
            <xsl:value-of select="$strings/plugins/manage"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="modernplugins">
  
    <table class="tablelist">
    
      <xsl:for-each select="extensionpoint">
    
        <xsl:if test="count(plugin) &gt; 0">
          <xsl:variable name="type" select="type"/>
          
          <xsl:variable name="safari">
            <xsl:choose>
              <xsl:when test="$type = 'adblocker'">
                <xsl:value-of select="true()"/>
              </xsl:when>
              <xsl:when test="$type = 'safari'">
                <xsl:value-of select="true()"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="false()"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:variable>
                  
          <tr>
            <th class="header" colspan="3">
              <xsl:value-of
                select="$strings/plugins/extensionpoints/value[@key = $type]"/>
            </th>
            <th>
              <xsl:if test="$safari = 'true'">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/openapplication//Applications/Safari.app')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of
                      select="$strings/safari/open_safari_title[@version = $prefversion]"/>
                  </xsl:attribute>
                  <xsl:value-of
                    select="$strings/safari/open_safari"/>
                  <img class="buttonicon" src="image/safaributton.png"/>
                </button>
              </xsl:if>
            </th>
          </tr>
          
          <xsl:for-each select="plugin">
            <xsl:choose>
              <xsl:when test="$type = 'legacyquicklook'">
                <xsl:call-template name="quicklook"/>
              </xsl:when>
              <xsl:when test="$type = 'quicklookpreview'">
                <xsl:call-template name="quicklook"/>
              </xsl:when>
              <xsl:when test="$type = 'quicklookthumbnail'">
                <xsl:call-template name="quicklook"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:call-template name="appextension"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:for-each>
        </xsl:if>
      </xsl:for-each>
        
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/plugins/description/node()"/>
      
      <xsl:copy-of
        select="$strings/plugins/description_ui[@version = $prefversion]/node()"/>

      <xsl:if
        test="count(extensionpoint[type = 'safari']) &gt; 0 or count(extensionpoint[type = 'adblocker']) &gt; 0">
        <xsl:copy-of
          select="$strings/plugins/description_safari[@version = $prefversion]/node()"/>
      </xsl:if>

      <xsl:if test="count(extensionpoint[type = 'legacyquicklook']) &gt; 0">
        <xsl:copy-of select="$strings/plugins/description_legacy/node()"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- Print modern plugins. -->
  <xsl:template name="quicklook">

    <xsl:call-template name="appextension">
      <xsl:with-param name="border" select="true()"/>
    </xsl:call-template>
    
    <xsl:for-each select="types/type">
      <tr>
        <td/>
        <td>
          <xsl:value-of select="uti"/>
        </td>
        <td colspan="2">
          <xsl:if test="extension">
            <xsl:text> *.</xsl:text>
            <xsl:value-of select="extension"/>
          </xsl:if>
        </td>
      </tr>
    </xsl:for-each>
    
  </xsl:template>

  <!-- Print modern plugins. -->
  <xsl:template name="appextension">
    <xsl:param name="border"/>
    
    <xsl:variable name="iconname">
      <xsl:call-template name="appicon">
        <xsl:with-param name="path">
          <xsl:value-of select="bundle_unsafe"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>

    <tr>
      <xsl:if test="$border = 'true' or position() = 1">
        <xsl:attribute name="class">
          <xsl:text>horizontalborder</xsl:text>
        </xsl:attribute>
      </xsl:if>
      <td colspan="3">
        <xsl:choose>
          <xsl:when test="string-length($iconname) &gt; 0">
            <img class="appicon">
              <xsl:attribute name="src">
                <xsl:value-of
                  select="concat('image/app/', $iconname)"/>
              </xsl:attribute>
              <xsl:attribute name="alt">
                <xsl:value-of select="$strings/applications/app_icon"/>
              </xsl:attribute>
            </img>
            <xsl:value-of select="bundle_name"/>
            <xsl:text> (</xsl:text>
            <xsl:value-of select="name"/>
            <xsl:text>)</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="bundle_name"/>
            <xsl:text> (</xsl:text>
            <xsl:value-of select="name"/>
            <xsl:text>)</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </td>
      <td>
        <button>
          <xsl:attribute name="onclick">
            <xsl:value-of
              select="concat('GET(&quot;/reveal/',path_unsafe,'&quot;)')"/>
          </xsl:attribute>
          <xsl:attribute name="title">
            <xsl:value-of
              select="$strings/plugins/reveal_in_finder_title"/>
          </xsl:attribute>
          <xsl:value-of
            select="$strings/plugins/reveal_in_finder"/>
          <img class="buttonicon" src="image/finderbutton.png"/>
        </button>
      </td>
    </tr>
            
  </xsl:template>

  <!-- Print internet plugins. -->
  <xsl:template match="internetplugins" mode="top">
  
    <xsl:if test="count(plugin) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/openfolder//Library/Internet Plug-Ins')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/plugins/open_in_finder"/>
            <img class="buttonicon" src="image/finderbutton.png"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="internetplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/developer"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/internetplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print user internet plugins. -->
  <xsl:template name="userinternetplugins">
  
    <xsl:variable name="try">
      <xsl:choose>
        <xsl:when test="count(userinternetplugins/plugin) &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when
          test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
          <xsl:value-of select="false()"/>
        </xsl:when>

        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:if test="$try = 'true'">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">userinternetplugins</xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:choose>
            <xsl:when
              test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
              <p>
                <strong>
                  <xsl:value-of
                    select="$strings/userinternetplugins/needsfulldriveaccess"/>
                </strong>
              </p>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="userinternetplugins"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/openfolder/~/Library/Internet Plug-Ins')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/plugins/open_in_finder"/>
            <img class="buttonicon" src="image/finderbutton.png"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="userinternetplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/developer"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of
        select="$strings/userinternetplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print audio plugins. -->
  <xsl:template match="audioplugins" mode="top">
  
    <xsl:if test="count(plugin) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/openfolder//Library/Audio/Plug-Ins')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/plugins/open_in_finder"/>
            <img class="buttonicon" src="image/finderbutton.png"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="audioplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/developer"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/audioplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print CoreMediaIO plugins. -->
  <xsl:template match="coremediaioplugins" mode="top">
  
    <xsl:if test="count(plugin) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/openfolder//Library/CoreMediaIO/Plug-Ins')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/plugins/open_in_finder"/>
            <img class="buttonicon" src="image/finderbutton.png"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="coremediaioplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/developer"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/coremediaioplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print user audio plugins. -->
  <xsl:template name="useraudioplugins">
  
    <xsl:variable name="try">
      <xsl:choose>
        <xsl:when test="count(useraudioplugins/plugin) &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when
          test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
          <xsl:value-of select="false()"/>
        </xsl:when>

        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$try = 'true'">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">useraudioplugins</xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:choose>
            <xsl:when
              test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
              <p>
                <strong>
                  <xsl:value-of
                    select="$strings/useraudioplugins/needsfulldriveaccess"/>
                </strong>
              </p>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="useraudioplugins"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/openfolder/~/Library/Audio/Plug-Ins')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/plugins/open_in_finder"/>
            <img class="buttonicon" src="image/finderbutton.png"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="useraudioplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/developer"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/useraudioplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print user CoreMediaIO plugins. -->
  <xsl:template name="usercoremediaioplugins">
  
    <xsl:variable name="try">
      <xsl:choose>
        <xsl:when test="count(usercoremediaioplugins/plugin) &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when
          test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
          <xsl:value-of select="false()"/>
        </xsl:when>

        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$try = 'true'">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">usercoremediaioplugins</xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:choose>
            <xsl:when
              test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
              <p>
                <strong>
                  <xsl:value-of
                    select="$strings/usercoremediaioplugins/needsfulldriveaccess"/>
                </strong>
              </p>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="usercoremediaioplugins"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/openfolder/~/Library/CoreMediaIO/Plug-Ins')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/plugins/open_in_finder"/>
            <img class="buttonicon" src="image/finderbutton.png"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="usercoremediaioplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/developer"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of
        select="$strings/usercoremediaioplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print iTunes plugins. -->
  <xsl:template match="itunesplugins" mode="top">
  
    <xsl:if test="count(plugin) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/openfolder//Library/iTunes/iTunes Plug-ins')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/plugins/open_in_finder"/>
            <img class="buttonicon" src="image/finderbutton.png"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="itunesplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/developer"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/itunesplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print user iTunes plugins. -->
  <xsl:template name="useritunesplugins">
  
    <xsl:variable name="try">
      <xsl:choose>
        <xsl:when test="count(useritunesplugins/plugin) &gt; 0">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:when
          test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
          <xsl:value-of select="false()"/>
        </xsl:when>

        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:if test="$try = 'true'">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">useritunesplugins</xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:choose>
            <xsl:when
              test="$fulldriveaccess != 'true' and /etrecheck/header/apptype = 'mas'">
              <p>
                <strong>
                  <xsl:value-of
                    select="$strings/useritunesplugins/needsfulldriveaccess"/>
                </strong>
              </p>
            </xsl:when>
            <xsl:otherwise>
              <xsl:apply-templates select="useritunesplugins"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/openfolder/~/Library/iTunes/Plug-Ins')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/plugins/open_in_finder_title"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/plugins/open_in_finder"/>
            <img class="buttonicon" src="image/finderbutton.png"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>

  </xsl:template>

  <xsl:template match="useritunesplugins">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/plugins/name"/></th>
        <th><xsl:value-of select="$strings/plugins/version"/></th>
        <th><xsl:value-of select="$strings/plugins/developer"/></th>
        <th><xsl:value-of select="$strings/plugins/date"/></th>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/useritunesplugins/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print a single plugin. -->
  <xsl:template match="plugin">
  
    <tr>
      <td><xsl:value-of select="name"/></td>
      <td><xsl:value-of select="version"/></td>
      <td><xsl:value-of select="developer"/></td>
      <td><xsl:value-of select="installdate"/></td>
    </tr>

  </xsl:template>

  <!-- Print a single plugin. -->
  <xsl:template match="plugin" mode="unsigned">
  
    <tr>
      <td>
        <xsl:value-of select="name"/>
      </td>
      <td>
        <button>
          <xsl:attribute name="onclick">
            <xsl:value-of
              select="concat('GET(&quot;/reveal/',path_unsafe,'&quot;)')"/>
          </xsl:attribute>
          <xsl:attribute name="title">
            <xsl:value-of
              select="$strings/plugins/reveal_in_finder_title"/>
          </xsl:attribute>
          <xsl:value-of
            select="$strings/plugins/reveal_in_finder"/>
          <img class="buttonicon" src="image/finderbutton.png"/>
        </button>
      </td>
      <td><xsl:value-of select="installdate"/></td>
    </tr>

  </xsl:template>

  <!-- Print Safari extensions. -->
  <xsl:template match="safariextensions" mode="top">
  
    <xsl:if test="count(extension) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:text>GET('/openapplication//Applications/Safari.app')</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of
                select="$strings/safari/open_safari_title[@version = $prefversion]"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/safari/open_safari"/>
            <img class="buttonicon" src="image/safaributton.png"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>
  
  <xsl:template match="safariextensions">
  
    <table class="tablelist">
    
      <tr>
        <th colspan="2">
          <xsl:value-of select="$strings/safari/extension"/>
        </th>
        <th><xsl:value-of select="$strings/safari/install_date"/></th>
      </tr>
      
      <xsl:apply-templates select="extension"/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/safari/description[@version = $prefversion]/node()"/>
    </p>

  </xsl:template>

  <!-- Print a single Safari extension. -->
  <xsl:template match="extension">
  
    <xsl:variable name="status" select="status"/>
    
    <xsl:variable name="identifier" select="identifier"/>
    <xsl:variable
      name="adware"
      select="count(/etrecheck/adware//extension[identifier = $identifier]) &gt; 0"/>
    
    <tr>
      <xsl:attribute name="class">
        <xsl:value-of select="concat('horizontalborder ', identifier)"/>
      </xsl:attribute>

      <td class="header">
        <xsl:value-of select="$strings/safari/name"/>
        
        <br/>
        <xsl:value-of select="$strings/safari/developer"/>
        
        <xsl:if test="not($adware)">
          <xsl:if test="url">
            <br/>
            <xsl:value-of select="$strings/safari/url"/>
          </xsl:if>
        </xsl:if>
      </td>
      <td>
        <xsl:value-of select="displayname"/>

        <br/>
        
        <xsl:choose>
          <xsl:when test="$adware">
            <xsl:value-of select="$strings/safari/adware"/>
          </xsl:when>
          <xsl:when test="developer">
            <xsl:value-of select="developer"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$strings/safari/unknown"/>
          </xsl:otherwise>
        </xsl:choose>
        
        <xsl:if test="not($adware)">
          <xsl:if test="url">
            <br/>
          
            <xsl:choose>
              <xsl:when test="$printing = 'false'">
                <a href="#">
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/link/',url,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:value-of select="url"/>
                </a>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="url"/>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:if>
        </xsl:if>

      </td>
      <td>
        <xsl:value-of select="installdate"/>
      </td>
    </tr>
    
  </xsl:template>

  <!-- Print Preference panes. -->
  <xsl:template match="preferencepanes" mode="top">
  
    <xsl:if test="count(preferencepane) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
        <xsl:with-param name="control">
          <button>
            <xsl:attribute name="onclick">
              <xsl:choose>
                <xsl:when test="/etrecheck/systemsoftware/majorversion &gt;= 18">
                  <xsl:text>GET('/openapplication//System/Applications/System Settings.app')</xsl:text>
                </xsl:when>
                <xsl:when test="/etrecheck/systemsoftware/majorversion &gt;= 15">
                  <xsl:text>GET('/openapplication//System/Applications/System Preferences.app')</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:text>GET('/openapplication//Applications/System Preferences.app')</xsl:text>
                </xsl:otherwise>
              </xsl:choose>
            </xsl:attribute>
            <xsl:attribute name="title">
              <xsl:value-of select="$strings/preferencepane/open_system_preferences_title[@version = $prefversion]"/>
            </xsl:attribute>
            <xsl:value-of
              select="$strings/preferencepane/open_system_preferences"/>
            <img class="buttonicon" src="image/systempreferencesbutton.png"/>
          </button>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="preferencepanes">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/preferencepane/name"/></th>
        <th><xsl:value-of select="$strings/preferencepane/developer"/></th>
        <th><xsl:value-of select="$strings/preferencepane/date"/></th>
        <th/>
      </tr>
    
      <xsl:apply-templates/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/preferencepane/description[@version = $prefversion]/node()"/>
    </p>

  </xsl:template>

  <!-- Print a single Preference pane. -->
  <xsl:template match="preferencepane">
  
    <xsl:variable name="iconname">
      <xsl:call-template name="appicon">
        <xsl:with-param name="path">
          <xsl:value-of select="path_unsafe"/>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:variable>

    <tr>
      <td><xsl:value-of select="name"/></td>
      <td><xsl:value-of select="developer"/></td>
      <td><xsl:value-of select="installdate"/></td>
      <td>
        <button>
          <xsl:attribute name="onclick">
            <xsl:value-of
              select="concat('GET(&quot;/openapplication/',path_unsafe,'&quot;)')"/>
          </xsl:attribute>
          <xsl:attribute name="title">
            <xsl:call-template name="format">
              <xsl:with-param
                name="format"
                select="$strings/preferencepane/open_preference_pane_title"/>
              <xsl:with-param name="value" select="name"/>
            </xsl:call-template>
          </xsl:attribute>
          <xsl:value-of select="$strings/preferencepane/open"/>
          <img class="buttonicon">
            <xsl:attribute name="src">
              <xsl:value-of
                select="concat('image/app/', $iconname)"/>
            </xsl:attribute>
            <xsl:attribute name="alt">
              <xsl:value-of select="$strings/applications/app_icon"/>
            </xsl:attribute>
          </img>
        </button>
      </td>
    </tr>
    
  </xsl:template>

  <!-- Print software install information. -->
  <xsl:template match="install" mode="top">
  
    <xsl:if test="count(package) &gt; 0">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">
          <xsl:value-of select="local-name()"/>
        </xsl:with-param>
        <xsl:with-param name="contents">
          <xsl:apply-templates select="."/>
        </xsl:with-param>
      </xsl:call-template>
    
    </xsl:if>
    
  </xsl:template>

  <xsl:template match="install">
  
    <table>
      <tr>
        <th><xsl:value-of select="$strings/install/name"/></th>
        <th><xsl:value-of select="$strings/install/version"/></th>
        <th><xsl:value-of select="$strings/install/date"/></th>
      </tr>
    
      <xsl:apply-templates select="package"/>
    
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/install/description/node()"/>
    </p>

  </xsl:template>

  <!-- Print an install package. -->
  <xsl:template match="package">
  
    <tr>
      <td><xsl:value-of select="name"/></td>
      <td><xsl:value-of select="version"/></td>
      <td><xsl:value-of select="installdate"/></td>
    </tr>

  </xsl:template>

  <!-- Clean up information. -->
  <xsl:template name="cleanup">
  
    <xsl:if test="/etrecheck/cleanup/launchdfiles">
    
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">cleanup</xsl:with-param>
        <xsl:with-param name="contents">
        
          <table class="tablelist">
          
            <xsl:for-each select="/etrecheck/cleanup/launchdfiles/launchdfile">
          
              <xsl:variable name="name">
                <xsl:choose>
                  <xsl:when test="filename_unsafe and filename_unsafe = filename">
                    <xsl:value-of select="filename_unsafe"/>
                  </xsl:when>
                  <xsl:when test="name">
                    <xsl:value-of select="name"/>
                  </xsl:when>
                  <xsl:when test="label">
                    <xsl:value-of select="label"/>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:value-of select="path_unsafe"/>
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:variable>

              <tbody>
                <xsl:attribute name="class">
                  <xsl:value-of select="identifier"/>
                  <xsl:text> group</xsl:text>
                </xsl:attribute>
                
                <tr>
                  <td>
                    <xsl:value-of select="$name"/>
                  </td>
                  <td class="control">
                    <button class="review">
                      <xsl:attribute name="onclick">
                        <xsl:value-of
                          select="concat('GET(&quot;/review/launchd/',identifier,'&quot;)')"/>
                      </xsl:attribute>
                      <xsl:attribute name="title">
                        <xsl:value-of select="$strings/review/review_title"/>
                      </xsl:attribute>
                      <xsl:value-of select="$strings/review/review"/>
                    </button>
                  </td>
                </tr>
              </tbody>
              
            </xsl:for-each>
              
          </table>
        
          <p class="description">
            <xsl:copy-of select="$strings/cleanup/description/node()"/>
            
            <xsl:copy-of
              select="$strings/cleanup/description_remove/node()"/>
          </p>
      
        </xsl:with-param>
      </xsl:call-template>
      
    </xsl:if>
    
  </xsl:template>

  <!-- A menu for the performance page. -->
  <xsl:template name="performancemenu">
  
    <xsl:variable name="performanceid">
      <xsl:text>performance</xsl:text>
    </xsl:variable>
    
    <xsl:variable name="analyticsid">
      <xsl:text>analytics</xsl:text>
    </xsl:variable>

    <xsl:variable name="cpuid">
      <xsl:text>cpuusage</xsl:text>
    </xsl:variable>

    <xsl:variable name="memoryid">
      <xsl:text>memory</xsl:text>
    </xsl:variable>

    <xsl:variable name="energyid">
      <xsl:text>energy</xsl:text>
    </xsl:variable>

    <div class="menu">
      <table class="menu">
        <tr>
          <td>
            <a href="#">
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('menuJumpToId(&quot;',$performanceid,'&quot;)')"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/performancemenu/performance"/>
            </a>
          </td>
          <td>
            <a href="#">
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('menuJumpToId(&quot;',$analyticsid,'&quot;)')"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/performancemenu/analytics"/>
            </a>
          </td>
          <td>
            <a href="#">
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('menuJumpToId(&quot;',$cpuid,'&quot;)')"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/performancemenu/cpu"/>
            </a>
          </td>
          <td>
            <a href="#">
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('menuJumpToId(&quot;',$memoryid,'&quot;)')"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/performancemenu/memory"/>
            </a>
          </td>
          <td>
            <a href="#">
              <xsl:attribute name="onclick">
                <xsl:value-of
                  select="concat('menuJumpToId(&quot;',$energyid,'&quot;)')"/>
              </xsl:attribute>
              <xsl:value-of select="$strings/performancemenu/energy"/>
            </a>
          </td>
        </tr>
      </table>
    </div>
    
    <div class="menuspacer"/>
    
  </xsl:template>
  
  <!-- Show overall performance. -->
  <xsl:template name="performance">
  
    <xsl:variable name="insights">
      <xsl:choose>
        <xsl:when test="/etrecheck/systemsoftware/majorversion &gt;= 13">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable
      name="performance" select="/etrecheck/performance/score"/>

    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">performance</xsl:with-param>
      <xsl:with-param name="contents">
        <table class="tablelist">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/performance/systemload"/>
            </td>
            <td colspan="2">
              <img class="loadgraph" src="image/loadgraph.png">
                <xsl:attribute name="alt">
                  <xsl:value-of
                    select="$strings/performance/loadgraph_alt"/>
                </xsl:attribute>
              </img>
              <img class="loadgraph_dark" src="image/loadgraphDarkAqua.png">
                <xsl:attribute name="alt">
                  <xsl:value-of
                    select="$strings/performance/loadgraph_alt"/>
                </xsl:attribute>
              </img>
            </td>
          </tr>
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of select="$strings/performance/etrecheckruntime"/>
            </td>
            <td colspan="2">
              <xsl:value-of select="/etrecheck/header/runtime"/>
              <xsl:text> </xsl:text>
              <xsl:value-of select="$strings/performance/runtimeunits"/>
            </td>
          </tr>
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of select="$strings/performance/nominalio"/>
            </td>
            <td colspan="2">
              <xsl:apply-templates
                select="/etrecheck/performance/nominalio" mode="units"/>
            </td>
          </tr>
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of select="$strings/performance/filesystemtime"/>
            </td>
            <td colspan="2">
              <xsl:value-of select="/etrecheck/performance/filesystemtime"/>
              <xsl:text> </xsl:text>
              <xsl:value-of select="$strings/performance/seconds"/>
              
              <xsl:if test="/etrecheck/performance/filesystemtimeout = 'true' or /etrecheck/performance/filesystemfailure = 'true'">
                <xsl:text> (</xsl:text>
                <xsl:if test="/etrecheck/performance/filesystemtimeout = 'true'">
                  <xsl:value-of select="$strings/performance/timeout"/>
                </xsl:if>
                <xsl:if test="/etrecheck/performance/filesystemfailure = 'true'">
                  <xsl:if test="/etrecheck/performance/filesystemtimeout = 'true'">
                    <xsl:text>, </xsl:text>
                  </xsl:if>
                  <xsl:value-of select="$strings/performance/failure"/>
                </xsl:if>
                <xsl:text>)</xsl:text>
              </xsl:if>
            </td>
          </tr>
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of select="$strings/performance/writespeed"/>
            </td>
            <td colspan="2">
              <xsl:apply-templates
                select="/etrecheck/performance/writespeed" mode="units"/>
              <xsl:if test="/etrecheck/performance/writetimeout = 'true'">
                <xsl:value-of select="$strings/performance/timeout"/>
              </xsl:if>
            </td>
          </tr>
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of select="$strings/performance/readspeed"/>
            </td>
            <td colspan="2">
              <xsl:apply-templates
                select="/etrecheck/performance/readspeed" mode="units"/>
              <xsl:if test="/etrecheck/performance/readtimeout = 'true'">
                <xsl:value-of select="$strings/performance/timeout"/>
              </xsl:if>
            </td>
          </tr>
          <tr class="horizontalborder">
            <td class="header">
              <xsl:value-of
                select="$strings/performance/etrecheckperformance"/>
            </td>
            <td colspan="2">
              <div id="score">
                <xsl:value-of
                  select="$strings/performance/performance_rating/value[@key = $performance]"/>

                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/help/performance')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/performance/about_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/performance/about"/>
                </button>
                <xsl:if test="$insights = 'true'">
                  <button>
                    <xsl:attribute name="onclick">
                      <xsl:text>GET('/show/insights')</xsl:text>
                    </xsl:attribute>
                    <xsl:attribute name="title">
                      <xsl:value-of select="$strings/performance/show_insights"/>
                    </xsl:attribute>
                    <xsl:value-of select="$strings/performance/show"/>
                  </button>
                </xsl:if>
              </div>
            </td>
          </tr>
        </table>
        
        <p class="description">
          <xsl:copy-of select="$strings/performance/description/node()"/>

          <xsl:if test="$insights = 'true'">
            <xsl:copy-of select="$strings/performance/description_compare/node()"/>
          </xsl:if>
        </p>
    
      </xsl:with-param>
    </xsl:call-template>
    
  </xsl:template>

  <!-- Print a CPU usage information. -->
  <xsl:template name="cpu">
  
    <xsl:variable name="width">
      <xsl:choose>
        <xsl:when test="count(/etrecheck/cpu//cores/core) &gt; 16">
          <xsl:value-of select="240 div count(/etrecheck/cpu//cores/core)"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="15"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:if test="/etrecheck/cpu/usage">
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">cpuusage</xsl:with-param>
        <xsl:with-param name="contents">
          <table class="tablelist">
            <tr>
              <td class="header verticalborder">
                <xsl:value-of select="$strings/cpuusage/type"/>
              </td>
              
              <td class="verticalborder centerheader" colspan="2">
                <strong>
                  <xsl:value-of select="$strings/cpuusage/overall"/>
                </strong>
              </td>
              
              <td class="centerheader">
                <strong>
                  <xsl:value-of select="$strings/cpuusage/cores"/>
                </strong>
              </td>
            </tr>
            <tr class="horizontalborder">
              <td class="header verticalborder">
                <xsl:value-of select="$strings/cpuusage/system"/>
                <br/>
                <xsl:value-of select="$strings/cpuusage/user"/>
                <br/>
                <xsl:value-of select="$strings/cpuusage/idle"/>
              </td>
              <td>
                <span class="systemcolor">
                  <xsl:apply-templates
                    select="/etrecheck/cpu/usage/system" mode="units"/>
                </span>
                <br/>
                <span class="usercolor">
                  <xsl:apply-templates
                    select="/etrecheck/cpu/usage/user" mode="units"/>
                </span>
                <br/>
                <span>
                  <xsl:apply-templates
                    select="/etrecheck/cpu/usage/idle" mode="units"/>
                </span>
              </td>
              <td class="verticalborder horizontalborder">
                <xsl:call-template name="cpumeter">
                  <xsl:with-param name="height" select="70"/>
                  <xsl:with-param name="width" select="20"/>
                  <xsl:with-param name="system" select="/etrecheck/cpu/usage/system"/>
                  <xsl:with-param name="user" select="/etrecheck/cpu/usage/user"/>
                  <xsl:with-param name="idle" select="/etrecheck/cpu/usage/idle"/>
                </xsl:call-template>
              </td>
              <td class="horizontalborder">
                <xsl:for-each select="/etrecheck/cpu//cores/core/usage">
                  <xsl:call-template name="cpumeter">
                    <xsl:with-param name="height" select="50"/>
                    <xsl:with-param name="width" select="$width"/>
                    <xsl:with-param name="system" select="system"/>
                    <xsl:with-param name="user" select="user"/>
                    <xsl:with-param name="idle" select="idle"/>
                  </xsl:call-template>
                </xsl:for-each>
              </td>
            </tr>
          </table>
          
          <p class="description">
            <xsl:copy-of select="$strings/cpuusage/description/node()"/>
            
            <xsl:if test="count(/etrecheck/cpu//cores/core) &gt; /etrecheck/hardware/corecount">
              <xsl:copy-of select="$strings/cpuusage/description_virtualcores/node()"/>
            </xsl:if>
          </p>
      
        </xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    
  </xsl:template>

  <!-- Print a CPU process usage information. -->
  <xsl:template match="cpu">
  
    <xsl:variable name="thirdparty">
      <xsl:for-each select="process">
        <xsl:variable name="path_unsafe" select="path_unsafe"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when
              test="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source">
              <xsl:value-of
                select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        
        <xsl:choose>
          <xsl:when test="xcode = 'true'"/>
          <xsl:when test="other = 'true'"/>
          <xsl:when test="string-length($path_unsafe) &gt; 0 and not(starts-with($source, 'Apple'))">
            <xsl:text>1</xsl:text>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>
      </xsl:for-each>
    </xsl:variable>

    <table>
      <tr>
        <th colspan="2"><xsl:value-of select="$strings/processes/process"/></th>
        <th><xsl:value-of select="$strings/processes/source"/></th>
        <th colspan="2"><xsl:value-of select="$strings/cpu/usage"/></th>
      </tr>
      <xsl:for-each select="process">
        <xsl:variable name="path_unsafe" select="path_unsafe"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when
              test="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source">
              <xsl:value-of
                select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:variable
          name="identifier"
          select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/identifier"/>

        <xsl:variable name="iconname">
          <xsl:call-template name="processicon">
            <xsl:with-param name="identifier">
              <xsl:value-of select="$identifier"/>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:variable>

        <tr>
          <td>
            <xsl:choose>
              <xsl:when test="other = 'true'">
                <img class="appicon" src="image/app/nil">
                  <xsl:attribute name="alt">
                    <xsl:value-of select="$strings/cpu/app_icon"/>
                  </xsl:attribute>
                </img>
              </xsl:when>
              <xsl:otherwise>
                <img class="appicon">
                  <xsl:attribute name="src">
                    <xsl:value-of
                      select="concat('image/process/', $iconname)"/>
                  </xsl:attribute>
                  <xsl:attribute name="alt">
                    <xsl:value-of select="$strings/cpu/app_icon"/>
                  </xsl:attribute>
                </img>
              </xsl:otherwise>
            </xsl:choose>
          </td>
          <td>
            <span class="process">
              <xsl:value-of select="name"/>
              
              <xsl:if test="count &gt; 1">
                <xsl:value-of select="concat(' (',count,')')"/>
              </xsl:if>
              
              <xsl:if test="other = 'true'">
                <xsl:value-of select="$strings/cpu/other"/>
              </xsl:if>
            </span>
          </td>
          <td><xsl:value-of select="$source"/></td>
          <td class="percentage">
            <xsl:apply-templates select="usage" mode="units"/>
          </td>
          <td>
            <xsl:choose>
              <xsl:when test="translocated = 'true'">
                <xsl:value-of select="$strings/processes/translocated"/>
              </xsl:when>
              <xsl:when test="xcode = 'true'">
                <xsl:value-of select="$strings/processes/xcode"/>
              </xsl:when>
              <xsl:when test="path = 'kernel_task' and usage &gt; 150">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET("/link/https://support.apple.com/HT207359");</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/cpu/runaway_kernel_button_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/cpu/runaway_kernel_button"/>
                  <img class="buttonicon" src="image/webbrowserbutton.png"/>
                </button>
              </xsl:when>
              <xsl:when
                test="string-length(location_unsafe) &gt; 0 and not(starts-with($source, 'Apple'))">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/reveal/',location_unsafe,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/processes/reveal_in_finder_title"/>
                  </xsl:attribute>
                  <xsl:value-of
                    select="$strings/processes/reveal_in_finder"/>
                  <img class="buttonicon" src="image/finderbutton.png"/>
                </button>
              </xsl:when>
              <xsl:when test="other = 'true'">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/openapplication//Applications/Utilities/Activity Monitor.app')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of
                      select="$strings/processes/activitymonitor_title"/>
                  </xsl:attribute>
                  <xsl:value-of
                    select="$strings/processes/activitymonitor"/>
                  <img class="buttonicon"
                    src="image/activitymonitorbutton.png"/>
                </button>
              </xsl:when>
              <xsl:when test="man">
                <!-- Provide a button to an Apple man page. -->
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/man/',man,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/processes/about_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/processes/about"/>
                  <img class="buttonicon" src="image/man.png"/>
                </button>
              </xsl:when>
              <xsl:otherwise/>
            </xsl:choose>
          </td>
        </tr>
      </xsl:for-each>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/cpu/description/node()"/>

      <xsl:if test="count(process/man) &gt; 0">
        <xsl:copy-of select="$strings/processes/description_about/node()"/>
      </xsl:if>

      <xsl:if test="count(process[other = 'true']) &gt; 0">
        <xsl:copy-of select="$strings/processes/description_other/node()"/>
      </xsl:if>

      <xsl:if test="string-length($thirdparty) &gt; 0">
        <xsl:copy-of
          select="$strings/processes/description_thirdparty/node()"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- Print a memory usage information. -->
  <xsl:template match="memory">
  
    <xsl:variable name="thirdparty">
      <xsl:for-each select="process">
        <xsl:variable name="path_unsafe" select="path_unsafe"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when
              test="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source">
              <xsl:value-of
                select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        
        <xsl:choose>
          <xsl:when test="xcode = 'true'"/>
          <xsl:when test="other = 'true'"/>
          <xsl:when test="string-length($path_unsafe) &gt; 0 and not(starts-with($source, 'Apple'))">
            <xsl:text>1</xsl:text>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>
      </xsl:for-each>
    </xsl:variable>

    <table>
      <tr>
        <th colspan="2">
          <xsl:value-of select="$strings/processes/process"/>
        </th>
        <th><xsl:value-of select="$strings/processes/source"/></th>
        <th colspan="2"><xsl:value-of select="$strings/memory/usage"/></th>
      </tr>
      <xsl:for-each select="process">
        <xsl:variable name="path_unsafe" select="path_unsafe"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when
              test="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source">
              <xsl:value-of
                select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:variable
          name="identifier"
          select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/identifier"/>
        
        <xsl:variable name="iconname">
          <xsl:call-template name="processicon">
            <xsl:with-param name="identifier">
              <xsl:value-of select="$identifier"/>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:variable>

        <tr>
          <td>
            <xsl:choose>
              <xsl:when test="other = 'true'">
                <img class="appicon" src="image/app/nil">
                  <xsl:attribute name="alt">
                    <xsl:value-of select="$strings/cpu/app_icon"/>
                  </xsl:attribute>
                </img>
              </xsl:when>
              <xsl:otherwise>
                <img class="appicon">
                  <xsl:attribute name="src">
                    <xsl:value-of
                      select="concat('image/process/', $iconname)"/>
                  </xsl:attribute>
                  <xsl:attribute name="alt">
                    <xsl:value-of select="$strings/memory/app_icon"/>
                  </xsl:attribute>
                </img>
              </xsl:otherwise>
            </xsl:choose>
          </td>
          <td>
            <span class="process">
              <xsl:value-of select="name"/>
              
              <xsl:if test="count &gt; 1">
                <xsl:value-of select="concat(' (',count,')')"/>
              </xsl:if>
            </span>
          </td>
          <td><xsl:value-of select="$source"/></td>
          <td class="size"><xsl:apply-templates select="size" mode="units"/></td>
          <td>
            <xsl:choose>
              <xsl:when test="translocated = 'true'">
                <xsl:value-of select="$strings/processes/translocated"/>
              </xsl:when>
              <xsl:when test="xcode = 'true'">
                <xsl:value-of select="$strings/processes/xcode"/>
              </xsl:when>
              <xsl:when test="man">
                <!-- Provide a button to an Apple man page. -->
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/man/',man,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/processes/about_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/processes/about"/>
                  <img class="buttonicon" src="image/man.png"/>
                </button>
              </xsl:when>
              <xsl:otherwise>
                <xsl:if test="string-length(location_unsafe) &gt; 0 and not(starts-with($source, 'Apple'))">
                  <button>
                    <xsl:attribute name="onclick">
                      <xsl:value-of
                        select="concat('GET(&quot;/reveal/',location_unsafe,'&quot;)')"/>
                    </xsl:attribute>
                    <xsl:attribute name="title">
                      <xsl:value-of select="$strings/processes/reveal_in_finder_title"/>
                    </xsl:attribute>
                    <xsl:value-of
                      select="$strings/processes/reveal_in_finder"/>
                    <img class="buttonicon" src="image/finderbutton.png"/>
                  </button>
                </xsl:if>
              </xsl:otherwise>
            </xsl:choose>
          </td>
        </tr>
      </xsl:for-each>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/memory/description/node()"/>

      <xsl:if test="count(process/man) &gt; 0">
        <xsl:copy-of select="$strings/processes/description_about/node()"/>
      </xsl:if>

      <xsl:if test="string-length($thirdparty) &gt; 0">
        <xsl:copy-of
          select="$strings/processes/description_thirdparty/node()"/>
      </xsl:if>
    </p>

  </xsl:template>

  <!-- Print a energy usage information. -->
  <xsl:template match="energy">
  
    <xsl:variable name="thirdparty">
      <xsl:for-each select="process">
        <xsl:variable name="path_unsafe" select="path_unsafe"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when
              test="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source">
              <xsl:value-of
                select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        
        <xsl:choose>
          <xsl:when test="xcode = 'true'"/>
          <xsl:when test="other = 'true'"/>
          <xsl:when test="string-length($path_unsafe) &gt; 0 and not(starts-with($source, 'Apple'))">
            <xsl:text>1</xsl:text>
          </xsl:when>
          <xsl:otherwise/>
        </xsl:choose>
      </xsl:for-each>
    </xsl:variable>

    <table>
      <tr>
        <th colspan="2">
          <xsl:value-of select="$strings/processes/process"/>
        </th>
        <th><xsl:value-of select="$strings/processes/source"/></th>
        <th colspan="2"><xsl:value-of select="$strings/energy/usage"/></th>
      </tr>
      <xsl:for-each select="process">
        <xsl:variable name="path_unsafe" select="path_unsafe"/>
        <xsl:variable name="source">
          <xsl:choose>
            <xsl:when
              test="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source">
              <xsl:value-of
                select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/source"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>?</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:variable
          name="identifier"
          select="/etrecheck/header/context/process[path_unsafe = $path_unsafe]/identifier"/>
        
        <xsl:variable name="iconname">
          <xsl:call-template name="processicon">
            <xsl:with-param name="identifier">
              <xsl:value-of select="$identifier"/>
            </xsl:with-param>
          </xsl:call-template>
        </xsl:variable>

        <tr>
          <td>
            <xsl:choose>
              <xsl:when test="other = 'true'">
                <img class="appicon" src="image/app/nil">
                  <xsl:attribute name="alt">
                    <xsl:value-of select="$strings/cpu/app_icon"/>
                  </xsl:attribute>
                </img>
              </xsl:when>
              <xsl:otherwise>
                <img class="appicon">
                  <xsl:attribute name="src">
                    <xsl:value-of
                      select="concat('image/process/', $iconname)"/>
                  </xsl:attribute>
                  <xsl:attribute name="alt">
                    <xsl:value-of select="$strings/energy/app_icon"/>
                  </xsl:attribute>
                </img>
              </xsl:otherwise>
            </xsl:choose>
          </td>
          <td>
            <span class="process">
              <xsl:value-of select="name"/>
              
              <xsl:if test="count &gt; 1">
                <xsl:value-of select="concat(' (',count,')')"/>
              </xsl:if>
            </span>
          </td>
          <td><xsl:value-of select="$source"/></td>
          <td><xsl:value-of select="energy"/></td>
          <td>
            <xsl:choose>
              <xsl:when test="translocated = 'true'">
                <xsl:value-of select="$strings/processes/translocated"/>
              </xsl:when>
              <xsl:when test="xcode = 'true'">
                <xsl:value-of select="$strings/processes/xcode"/>
              </xsl:when>
              <xsl:when test="man">
                <!-- Provide a button to an Apple man page. -->
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:value-of
                      select="concat('GET(&quot;/man/',man,'&quot;)')"/>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/processes/about_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/processes/about"/>
                  <img class="buttonicon" src="image/man.png"/>
                </button>
              </xsl:when>
              <xsl:otherwise>
                <xsl:if test="string-length(location_unsafe) &gt; 0 and not(starts-with($source, 'Apple'))">
                  <button>
                    <xsl:attribute name="onclick">
                      <xsl:value-of
                        select="concat('GET(&quot;/reveal/',location_unsafe,'&quot;)')"/>
                    </xsl:attribute>
                    <xsl:attribute name="title">
                      <xsl:value-of select="$strings/processes/reveal_in_finder_title"/>
                    </xsl:attribute>
                    <xsl:value-of
                      select="$strings/processes/reveal_in_finder"/>
                    <img class="buttonicon" src="image/finderbutton.png"/>
                  </button>
                </xsl:if>
              </xsl:otherwise>
            </xsl:choose>
          </td>
        </tr>
      </xsl:for-each>
    </table>
    
    <p class="description">
      <xsl:copy-of select="$strings/energy/description/node()"/>

      <xsl:if test="count(process/man) &gt; 0">
        <xsl:copy-of select="$strings/processes/description_about/node()"/>
      </xsl:if>

      <xsl:if test="string-length($thirdparty) &gt; 0">
        <xsl:copy-of
          select="$strings/processes/description_thirdparty/node()"/>
      </xsl:if>
    </p>

  </xsl:template>
  
  <!-- Show analytics. -->
  <xsl:template name="analytics">
  
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name">analytics</xsl:with-param>
      <xsl:with-param name="contents">
        <table class="tablelist">
          <tr>
            <td class="header">
              <xsl:value-of select="$strings/analytics/title"/>
            </td>
            <td>
              <button>
                <xsl:attribute name="onclick">
                  <xsl:text>GET('/show/analytics')</xsl:text>
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="$strings/analytics/show_title"/>
                </xsl:attribute>
                <xsl:value-of select="$strings/analytics/show"/>
              </button>
            </td>
          </tr>
        </table>
        
        <p class="description">
          <xsl:copy-of select="$strings/analytics/description/node()"/>
        </p>
    
      </xsl:with-param>
    </xsl:call-template>
    
  </xsl:template>

  <!-- Show storage. -->
  <xsl:template name="storage">
  
    <xsl:variable name="show">
      <xsl:choose>
        <xsl:when test="$serial != /etrecheck/hardware/serial">
          <xsl:value-of select="false()"/>
        </xsl:when>
        <xsl:when test="$apptype = 'pro' and /etrecheck/systemsoftware/majorversion &gt;= 11">
          <xsl:value-of select="true()"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="false()"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
      
    <xsl:if test="$show = 'true'">
      <xsl:call-template name="section-wrapper">
        <xsl:with-param name="name">storage</xsl:with-param>
        <xsl:with-param name="contents">
          <table class="tablelist">
            <tr>
              <td class="header">
                <xsl:value-of select="$strings/storage/title"/>
              </td>
              <td class="actioncolumn">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/show/storage')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of select="$strings/storage/show_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/storage/show"/>
                </button>
              </td>
            </tr>
            <tr class="horizontalborder">
              <td class="header">
                <xsl:value-of select="$strings/storage/cleanup_info"/>
              </td>
              <td class="actioncolumn">
                <button>
                  <xsl:attribute name="onclick">
                    <xsl:text>GET('/fix/deletelocalsnapshots')</xsl:text>
                  </xsl:attribute>
                  <xsl:attribute name="title">
                    <xsl:value-of
                      select="$strings/storage/cleanup_title"/>
                  </xsl:attribute>
                  <xsl:value-of select="$strings/storage/cleanup"/>
                </button>
              </td>
            </tr>
          </table>

          <p class="description">
            <xsl:copy-of select="$strings/storage/description/node()"/>
            
            <xsl:copy-of
              select="$strings/storage/description_cleanup/node()"/>
          </p>
      
        </xsl:with-param>
      </xsl:call-template>
    </xsl:if>
    
  </xsl:template>

  <!-- Print a EtreCheck deleted files. -->
  <xsl:template match="etrecheckdeletedfiles">
  
    <xsl:if test="count(deletedfile) &gt; 0">
      <xsl:call-template name="header"/>

      <table>
        <xsl:call-template name="tablestyle"/>
        
        <xsl:for-each select="deletedfile">
          <tr>
            <td>
              <xsl:call-template name="style"/>
              <xsl:value-of select="date"/>
            </td>
            <td>
              <xsl:call-template name="style"/>
              <xsl:value-of select="path"/>
            </td>
            <td>
              <xsl:call-template name="style"/>
              <xsl:value-of select="reason"/>
            </td>
          </tr>
        </xsl:for-each>
        
      </table>
      
    </xsl:if>
    
  </xsl:template>

  <!-- Display a free space gauge. -->
  <xsl:template name="freespace_gauge">
    <xsl:param name="size"/>
    <xsl:param name="free"/>
    
    <xsl:variable name="expanded_size">
      <xsl:call-template name="expand">
        <xsl:with-param name="number" select="$size"/>
      </xsl:call-template>
    </xsl:variable>
    
    <xsl:variable name="expanded_free">
      <xsl:call-template name="expand">
        <xsl:with-param name="number" select="$free"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable
      name="percentage"
      select="($expanded_size - $expanded_free) * 100 div $expanded_size"/>

    <div class="meter">
      <xsl:call-template name="diskmeter">
        <xsl:with-param name="size" select="$expanded_size"/>
        <xsl:with-param name="free" select="$expanded_free"/>
        <xsl:with-param name="percentage" select="$percentage"/>
        <xsl:with-param name="units" select="$strings/volume/percent_used"/>
      </xsl:call-template>
    </div>
    
  </xsl:template>
  
  <!-- Display a CPU meter -->
  <xsl:template name="cpumeter">
    <xsl:param name="height">50</xsl:param>
    <xsl:param name="width">20</xsl:param>
    <xsl:param name="system"/>
    <xsl:param name="user"/>
    <xsl:param name="idle"/>

    <xsl:variable name="minimum" select="100 div $height"/>
    
    <xsl:variable name="systemheight">
      <xsl:choose>
        <xsl:when test="$system &gt; $minimum">
          <xsl:value-of select="$system * $height div 100"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="0"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="userheight">
      <xsl:choose>
        <xsl:when test="$user &gt; $minimum">
          <xsl:value-of select="$user * $height div 100"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="0"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <div class="cpumeter">
      <xsl:attribute name="style">
        <xsl:value-of
          select="concat('height: ', $height + 4, 'px; width: ', $width, 'px;')"/>
      </xsl:attribute>

      <div class="cpusystem">
        <xsl:attribute name="style">
          <xsl:value-of
            select="concat('height: ', $systemheight, 'px;')"/>
        </xsl:attribute>
      </div>
    
      <div class="cpuuser">
        <xsl:attribute name="style">
          <xsl:value-of
            select="concat('height: ', $userheight, 'px; bottom: ', $systemheight + 2, 'px;')"/>
        </xsl:attribute>
      </div>

    </div>
    
  </xsl:template>

  <!-- Display a meter -->
  <xsl:template name="meter">
    <xsl:param name="percentage"/>
    <xsl:param name="units"/>
    
    <div class="range">
    
      <xsl:if test="$percentage &gt; 5">
        <div class="level">
          <xsl:attribute name="style">
            <xsl:value-of
              select="concat('width: ', $percentage, 'px;')"/>
            <xsl:choose>
              <xsl:when test="$percentage &gt; 85">
                <xsl:text>background: red</xsl:text>
              </xsl:when>
              <xsl:when test="$percentage &gt; 70">
                <xsl:text>background: yellow</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>background: rgb(0, 242, 0)</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:attribute>
        </div>
      </xsl:if>
    
    </div>
    
    <span>
      <xsl:value-of select="concat(round($percentage), $units)"/>
    </span>
    
  </xsl:template>
  
  <!-- Display a disk meter -->
  <xsl:template name="diskmeter">
    <xsl:param name="size"/>
    <xsl:param name="free"/>
    <xsl:param name="percentage"/>
    <xsl:param name="units"/>
    
    <div class="range">
    
      <xsl:if test="$percentage &gt; 5">
        <div class="level">
          <xsl:attribute name="style">
            <xsl:value-of
              select="concat('width: ', $percentage, 'px;')"/>
            <xsl:choose>
              <xsl:when test="$free &gt; 50000000000">
                <xsl:text>background: rgb(0, 242, 0)</xsl:text>
              </xsl:when>
              <xsl:when test="$free &gt; 30000000000">
                <xsl:text>background: yellow</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>background: red</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:attribute>
        </div>
      </xsl:if>
    
    </div>
    
    <span>
      <xsl:value-of select="concat(round($percentage), $units)"/>
    </span>
    
  </xsl:template>

  <!-- Display a progress bar -->
  <xsl:template name="progress">
    <xsl:param name="percentage"/>
    <xsl:param name="units"/>
    
    <div class="range">
    
      <xsl:if test="$percentage &gt; 5">
        <div class="level">
          <xsl:attribute name="style">
            <xsl:value-of
              select="concat('width: ', $percentage, 'px;')"/>
          </xsl:attribute>
        </div>
      </xsl:if>
    
    </div>
    
    <span>
      <xsl:value-of select="concat(round($percentage), $units)"/>
    </span>
    
  </xsl:template>

  <!-- Print only the day portion of a warranty expiry date. -->
  <xsl:template match="warrantyexpiry">
  
    <xsl:value-of select="substring(., 1, 10)"/>
    
  </xsl:template>

  <!-- Top-level wrapper that creates the section container and title. -->
  <xsl:template match="*" mode="top">

    <!-- Grab the name of the current node and its transformed content. -->
    <xsl:call-template name="section-wrapper">
      <xsl:with-param name="name" select="local-name()"/>
      <xsl:with-param name="contents">
        <xsl:apply-templates select="."/>
      </xsl:with-param>
    </xsl:call-template>
    
  </xsl:template>
 
  <xsl:include href="ModernLibrary.xsl" />

  <xsl:include href="Library.xsl" />

</xsl:stylesheet>
